/*****************************************************************************
 *
 * coserror.h - CPKOS Error Code Definitions
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                         PROPRIETARY NOTICE
 *
 *  (c)Copyright 1988-2009 Dialogic Corporation.  All Rights Reserved
 *
 *  The following is licensed program material and is considered
 *  proprietary and confidential.  No part of this program may be
 *  reproduced or used in any form or by any means without the express
 *  written permission of Dialogic Corporation.
 *
 *****************************************************************************/

#ifndef _COSERROR_H
#define _COSERROR_H

/*****************************************************************************
 * Error code definitions
 *****************************************************************************/

#define CPI_ERRCODE_BASE            0x00001B00      /* Base address for error codes returned by the CPI library */
#define CPI_INVALID_HANDLE_PASSED   0x00001B58      /* Invalid handle provided */
#define CPI_INVALID_CHANNEL         0x00001B59      /* Channel number out of range */
#define CPI_INVALID_BOARD           0x00001B5A      /* Unable to open connection to board */
#define CPI_INVALID_MODE            0x00001B5B      /* Invalid operation for channel mode */
#define CPI_LENGTHS_DONT_MATCH      0x00001B5C      /* Mismatch on length verification */
#define CPI_NO_OPEN_HANDLES         0x00001B5D      /* No channels available (all in use) */
#define CPI_INSUFFICIENT_BUFFER_SIZE 0x00001B5E     /* Buffer too small for RX msg */
#define CPI_CHANNEL_IN_USE          0x00001B5F      /* Channel already in use */
#define CPI_CHANREG_FAIL            0x00001B60      /* Failure while registering channel */
#define CPI_CHANDEREG_FAIL          0x00001B61      /* Unable to deregister channel */
#define CPI_NO_MEMORY               0x00001B62      /* Insufficient memory to complete request */
#define CPI_UNSUPPORTED_OPER        0x00001B63      /* Unsupported operation */
#define CPI_TOO_BIG                 0x00001B64      /* Buffer size is too large for request */
#define CPI_NOT_READY               0x00001B65      /* Interface not ready for operation */
#define CPI_INVALID_OFFSET          0x00001B66      /* Provided offset out of range */
#define CPI_NOT_BOOT                0x00001B67      /* TX board has not been booted */
#define CPI_NOT_LOADING             0x00001B68      /* TX board not currently in loading state */
#define CPI_INVALID_BOOT_SIZE       0x00001B69      /* Bootstrap code size is invalid */
#define CPI_INVALID_LED             0x00001B6A      /* Invalid LED value requested */
#define CPI_READ_TIMEOUT            0x00001B6B      /* Read operation has timed out */
#define CPI_QUEUE_FULL              0x00001B6C      /* Channel queue is currently full */
#define CPI_CHANNEL_STATE           0x00001B6D      /* Channel in invalid state for request */
#define CPI_INVALID_ARGUMENT        0x00001B6E      /* Invalid argument provided in request */
#define CPI_BOARD_STATE             0x00001B6F      /* TX board in invalid state for request */
#define CPI_INVALID_DATA            0x00001B70      /* An unsupported data type was requested */
#define CPI_INVALID_SIZE            0x00001B71      /* Invalid buffer size associated with request */
#define CPI_NOT_CONTIGUOS           0x00001B72      /* Unable to create contiguous block */
#define CPI_NO_CHANNEL              0x00001B73      /* No channel associated with connection */
#define CPI_NO_BOARD                0x00001B74      /* No TX board associated with connection */
#define CPI_CORRUPT_MSG             0x00001B75      /* Corrupted TX communication message detected */
#define CPI_TX_REJECTED             0x00001B76      /* Request rejected by the TX board */
#define CPI_RESET                   0x00001B77      /* Request failed due to reset of TX board */
#define CPI_THREAD_FAIL             0x00001B78      /* Error occurred related to helper thread */
#define CPI_COMM_ERROR              0x00001B79      /* Communication error with driver */
#define CPI_COMM_UNKNOWN            0x00001B7A      /* Unknown communication message from driver */
#define CPI_UNKNOWN_DRIVER_ERROR    0x00001B7B      /* Unknown error returned from driver */
#define CPI_SEQNUM_MISMATCH         0x00001B7C      /* Received sequence number does not match expected */
#define CPI_DRIVER_OPEN             0x00001B7D      /* Unable to open TX device driver */
#define CPI_RX_FAIL                 0x00001B7E      /* Error while attempting to receive message from driver */
#define CPI_LOCK_FAIL               0x00001B7F      /* Unable to obtain a required LOCK level */
#define CPI_IN_PROGRESS             0x00001B80      /* Operation is currently in progress */
#define CPI_NOT_INITIALIZED         0x00001B81      /* CPI library has not been initialized */
#define CPI_OVERFLOW                0x00001B82      /* Overflow condition detected */
#define CPI_UNDERRUN                0x00001B83      /* Underrun condition detected */

#define SCC_ERROR_BASE              0x01000000      /* Base error code (lowest code [board side]) */

/* ----- memory region errors ----- */
#define SCCMRG_BASE                 0x01000000
#define SCCMRG_NOT_SUPPORTED        0x01000001      /* Type not supported */
#define SCCMRG_INVALID_ARG          0x01000002      /* Invalid argument provided */
#define SCCMRG_MEM_FULL             0x01000003      /* Insufficient free memory */
#define SCCMRG_NOT_ALLOCED          0x01000004      /* Memory region not currently allocated */
#define SCCMRG_NOT_OWNER            0x01000005      /* Current task not owner of region */
#define SCCMGR_TOO_SMALL            0x01000006      /* Insufficient size provided */
#define SCCMGR_INVALID_STATE        0x01000007      /* Invalid state for operation */
#define SCCMGR_INVALID_PARAM        0x01000008      /* Invalid parameter */
#define SCCMRG_INVALID_SIZE         0x01000009      /* Invalid/Inconsistant Size */

/* ----- trap (cpk_trap) errors ----- */
#define SCCTRAP_BASE                0x02000000
#define SCCTRAP_INVALID_TYPE        0x02000001      /* Invalid request type */
#define SCCTRAP_NOT_SUPPORTED       0x02000002      /* Function not supported */

/* ----- PCI bridge errors ----- */
#define SCCPCI_BASE                 0x03000000
#define SCCPCI_NOT_FOUND            0x03000001      /* No valid PCI bridge device found */
#define SCCPCI_UNKNOWN_TYPE         0x03000002      /* Unknown bridge device type */

/* ----- T1/E1 device errors ----- */
#define SCCT1E1_BASE                0x04000000
#define SCCT1E1_UNKNOWN_TYPE        0x04000001      /* Unknown T1/E1 device type */
#define SCCT1E1_FULL                0x04000002      /* Internal table full */
#define SCCT1E1_UNKNOWN_CMD         0x04000003      /* Unknown T1/E1 command */
#define SCCT1E1_INVALID_STATE       0x04000004      /* T1/E1 framer in invalid state for operation */
#define SCCT1E1_INVALID_MODE        0x04000005      /* T1/E1 framer in invalid mode for operation */
#define SCCT1E1_INVALID_VALUE       0x04000006      /* T1/E1 parameter value invalid */

/* ----- hardware device errors ----- */
#define SCCHWDEV_BASE               0x05000000
#define SCCHWDEV_UNKNOWN_TYPE       0x05000001      /* Unknown hardware device type */

/* ----- task control errors ----- */
#define SCCTASK_BASE                0x06000000
#define SCCTASK_EMPTY_RING          0x06000001      /* Empty priority ring */
#define SCCTASK_INVALID_CODE        0x06000002      /* Invalid code execution */
#define SCCTASK_INVALID_PARAM       0x06000003      /* Invalid parameter(s) provided */
#define SCCTASK_ALREADY_EXISTS      0x06000004      /* Task already exists */
#define SCCTASK_NO_TCB_AVAIL        0x06000005      /* No TCBs available */
#define SCCTASK_INVALID_TCB         0x06000006      /* Invalid TCB provided */
#define SCCTASK_INVALID_MSG         0x06000007      /* Invalid message provided */
#define SCCTASK_NOT_EXIST           0x06000008      /* Task does not exist */
#define SCCTASK_SQE_STATE           0x06000009      /* Service queue element state invalid for operation */
#define SCCTASK_EVENT_INUSE         0x0600000A      /* Event already in use to other task */
#define SCCTASK_MEM_FULL            0x0600000B      /* Insufficient free memory */
#define SCCTASK_TIMEOUT             0x0600000C      /* Timeout while performing task control operation */

/* ----- TX 4000 hardware specific errors ----- */
#define SCCTX4000_BASE              0x07000000
#define SCCTX4000_NOT_SUPPORTED     0x07000001      /* Function not supported */

/* ----- GPLIB Memory errors ----- */
#define SCCGPM_BASE                 0x08000000
#define SCCGPM_INVALID_ID           0x08000001      /* Invalid ID */
#define SCCGPM_NOT_START            0x08000002      /* Resource has not been started */
#define SCCGPM_NO_EVT               0x08000003      /* Event not found */
#define SCCGPM_NO_STATE             0x08000004      /* Not a single state found */
#define SCCGPM_INV_FREE_STATE       0x08000005      /* Invalid free state of pooltable */
#define SCCGPM_INV_ALOC_STATE       0x08000006      /* Invalid allocate state of pooltable */
#define SCCGPM_UNKOWN_STATE         0x08000007      /* Unknown/intermidiate state of pooltable */
#define SCCGPM_INVALID_ARG          0x08000008      /* Invalid argument */
#define SCCGPM_MSG_INUSE            0x08000009      /* Message in use */
#define SCCGPM_MEM_INUSE            0x0800000A      /* Memory in use */
#define SCCGPM_INV_POOL_TYPE        0x0800000B      /* Invalid pooled memory type */
#define SCCGPM_NOT_REGISTERED       0x0800000C      /* Not registered */
#define SCCGPM_REGISTERED           0x0800000D      /* Already registered */

/* ----- NAMED OBJECT errors ----- */
#define SCCNOBJ_BASE                0x09000000
#define SCCNOBJ_UNKNOWN_TYPE        0x09000001      /* Unknown object type provided */
#define SCCNOBJ_NULL_OBJECT         0x09000002      /* NULL object provided */
#define SCCNOBJ_INVALID_NAME        0x09000003      /* Invalid object name provided */
#define SCCNOBJ_MEM_FULL            0x09000004      /* Insufficient memory for object creation */
#define SCCNOBJ_ALREADY_EXISTS      0x09000005      /* Object specified already exists */
#define SCCNOBJ_NOT_FOUND           0x09000006      /* Requested object not found */
#define SCCNOBJ_INVALID_OBJECT      0x09000007      /* Invalid object pointer provided */

/* ----- System Information errors ----- */
#define SCCSYSI_BASE                0x0A000000
#define SCCSYSI_ALLOC_FAILED        0x0A000001      /* Allocation of System Information block failed */
#define SCCSYSI_NOT_FOUND           0x0A000002      /* System Information block not found */

/* ----- GPLIB Timer errors ----- */
#define SCCGPT_BASE                 0x0B000000
#define SCCGPT_NO_SYSINFO           0x0B000001      /* Unable to locate System Information block */
#define SCCGPT_OUT_OF_MEMORY        0x0B000002      /* Unable to allocate memory */
#define SCCGPT_UNKNOWN_TYPE         0x0B000003      /* Unknown type provided */
#define SCCGPT_NULL_FUNCTION        0x0B000004      /* NULL function provided */

/* ----- VSOCK errors ----- */
#define SCCVSOCK_BASE               0x0C000000
#define SCCVSOCK_MEM_FULL           0x0C000001      /* Out of memory */
#define SCCVSOCK_INVRXTXD           0x0C000002      /* Invlid RXTXD */
#define SCCVSOCK_RXTXD_BUF_SIZE     0x0C000003      /* Buffer size too small */
#define SCCVSOCK_RXTXD_ADJUST       0x0C000004      /* Error adjusting header */
#define SCCVSOCK_INV_STATE          0x0C000005      /* Invalid state */
#define SCCVSOCK_INV_HANDLE         0x0C000006      /* Invalid VSOCK handle */
#define SCCVSOCK_INV_NAME           0x0C000007      /* Invalid resource name */
#define SCCVSOCK_INV_FUNCTION       0x0C000008      /* Invalid function */
#define SCCVSOCK_TX_OVERRUN         0x0C000009      /* Overran the TX queue */
#define SCCVSOCK_DUPLICATE          0x0C00000A      /* Duplicate resource */
#define SCCVSOCK_NOSUCHRESTYPE      0x0C00000B      /* No such resource type */
#define SCCVSOCK_DUPHASHKEY         0x0C00000C      /* Duplicate hash key */
#define SCCVSOCK_INV_PARM           0x0C00000D      /* Invalid Parameter */

/* ----- IPETH Errors -----*/
#define SCCIPETH_BASE               0x0D000000
#define SCCIPETH_OUT_OF_MEMORY      0x0D000001      /* Memory allocate error */
#define SCCIPETH_INVHANDLE          0x0D000002      /* Invalid resource handle */
#define SCCIPETH_INVSTATE           0x0D000003      /* Invalid state for operation */
#define SCCIPETH_INVKEY             0x0D000004      /* Lower interface key invalid */
#define SCCIPETH_LEV1DEFINE         0x0D000005      /* Level 1 driver define error */
#define SCCIPETH_TXOVERFLOW         0x0D000006      /* Transmit overflow */
#define SCCIPETH_DUPLICATE          0x0D000007      /* Duplicate resource type registered */
#define SCCIPETH_NOSUCHTYPE         0x0D000008      /* No such resource type registered */
#define SCCIPETH_NOROUTE            0x0D000009      /* No route available */
#define SCCIPETH_DUPNAME            0x0D00000A      /* Duplicate resource or CPT name */
#define SCCIPETH_DUPHASHKEY         0x0D00000B      /* Duplicate hash key */
#define SCCIPETH_ARPERROR           0x0D00000C      /* ARP lookup problem */
#define SCCIPETH_INVNAME            0x0D00000D      /* Invalid resource name */
#define SCCIPETH_INVLENGTH          0x0D00000E      /* Packet size <=0 or >= 1518 */
#define SCCIPETH_LINKDOWN           0x0D00000F      /* Ethernet link is down */
#define SCCIPETH_SENDTOMBX          0x0D000010      /* Send to another task's mailbox failed */
#define SCCIPETH_RXTXD_BUF_LIMIT    0x0D000011      /* Add to RXTXD buffer chain failed */
#define SCCIPETH_INVRXTXD           0x0D000012      /* Invalid RXTXD pointer passed */
#define SCCIPETH_RXTXD_ADJUST       0x0D000013      /* Tried to adjust RXTXD buffer pointer error */
#define SCCIPETH_INVMSGTYPE         0x0D000014      /* IPMgr RX'd unknown message type */
#define SCCIPETH_IPROUTE_ADDFAIL    0x0D000015      /* Unable to add IP route to table */
#define SCCIPETH_IPCHKSUM           0x0D000016      /* IP header checksum error */
#define SCCIPETH_TTLEXPIRE          0x0D000017      /* IP TTL expired */
#define SCCIPETH_CPU_SYNC           0x0D000018      /* CPU sync error */
#define SCCIPETH_CU_TIMEOUT         0x0D000019      /* CU timeout */
#define SCCIPETH_INVALID_FUNCTION   0x0D00001A      /* Invalid/unsupported function */
#define SCCIPETH_MAX_CONPTS         0x0D00001B      /* Cannot support more connect points */
#define SCCIPETH_ETH1_LINK_FAIL     0x0D00001C      /* Ethernet link 1 failure */
#define SCCIPETH_ETH2_LINK_FAIL     0x0D00001D      /* Ethernet link 2 failure */
#define SCCIPETH_ETH1_LINK_ACTIVE   0x0D00001E      /* Ethernet link 1 active */
#define SCCIPETH_ETH2_LINK_ACTIVE   0x0D000020      /* Ethernet link 2 active */
#define SCCIPETH_MBOX_PKT_DROP      0x0D000021      /* Mailbox full; packet dropped */
#define SCCIPETH_NOTFOUND           0x0D000022      /* Item not found */
#define SCCIPETH_INVPARAM           0x0D000023      /* Invalid parameter */
#define SCCIPETH_CHECKSUM_FAIL      0x0D000024      /* Checksum verification failed */
#define SCCIPETH_INV_LINK           0x0D000025      /* Invalid interface/link specified */
#define SCCIPETH_LOOP_DETECT        0x0D000026      /* Looped route detected */

#define SCCIPETH_NDERROR            0x0D000080      /* IPv6 neighbor discovery error */
#define SCCIPETH_INVNEXTHDR         0x0D000081      /* IPv6 invalid next header type */
#define SCCIPETH_PROTOCOL_VIOLATION 0x0D000082      /* IPv6 protocol error */
#define SCCIPETH_IPV6DEST_NOTFOUND  0x0D000083      /* IPv6 dest entry for IP addr not found */
#define SCCIPETH_INV_IPV6ADDR       0x0D000084      /* IPv6 invalid address */
#define SCCIPETH_IPV6ND_NOTFOUND    0x0D000085      /* IPv6 ND entry for IP addr not found */
#define SCCIPETH_IPV6ND_INVEVENT    0x0D000086      /* IPv6 ND SEM, invalid event */
#define SCCIPETH_IPV6ND_ERROR       0x0D000087      /* IPv6 generic ND error */
#define SCCIPETH_IPV6IF_NOTFOUND    0x0D000088      /* IPv6 interface entry not found in the interface table */
#define SCCIPETH_IPV6MC_NOTFOUND    0x0D000089      /* IPv6 multicast address not found in the interface table */
#define SCCIPETH_IPV6ND_DUPADDRDET  0x0D00008A      /* IPv6 duplicate address detected */
#define SCCIPETH_IPV6ND_TIMERERR    0x0D00008B      /* IPv6 error with ND timer */
#define SCCIPETH_IPV6RTR_TIMERERR   0x0D00008C      /* IPv6 error with router timer */

#define SCCIPETH_IPV4ARP_TIMERERR   0x0D000090      /* Ipv4 error with ARP timer */
#define SCCIPETH_IPV4ARP_INVENTRY   0x0D000091      /* Ipv4 invalid ARP entry */
#define SCCIPETH_IPV4ARP_INVEVENT   0x0D000092      /* Ipv4 ARP SEM, invalid event */

/* ----- Switching Manager errors ----- */
#define SCCSWI_BASE                 0x0E000000
#define SCCSWI_MODE_NOT_SUPPORTED   0x0E000001      /* Requested mode not supported */
#define SCCSWI_INVALID_STREAM       0x0E000002      /* Invalid stream number provided */
#define SCCSWI_INVALID_MODE         0x0E000003      /* Invalid mode provided */
#define SCCSWI_INVALID_BUS          0x0E000004      /* Invalid bus type provided */
#define SCCSWI_INVALID_CAM          0x0E000005      /* Invalid CAM type provided */
#define SCCSWI_INVALID_DIRECTION    0x0E000006      /* Invalid direction provided */
#define SCCSWI_NOT_EXIST            0x0E000007      /* Requested item does not exist */
#define SCCSWI_FULL                 0x0E000008      /* Internal table full */
#define SCCSWI_TOO_SMALL            0x0E000009      /* Insufficient size provided */
#define SCCSWI_ALREADY_EXISTS       0x0E00000A      /* Indicated element already exists */
#define SCCSWI_INVALID_PARAM        0x0E00000B      /* Invalid parameter provided */
#define SCCSWI_CONNECT_FAILED       0x0E00000C      /* Failed to make requested connection */
#define SCCSWI_BREAK_FAILED         0x0E00000D      /* Failed to break requested connection */
#define SCCSWI_CORRUPTION           0x0E00000E      /* Internal corruption deteceted */
#define SCCSWI_NOT_SUPPORTED        0x0E00000F      /* Requested function not currently supported */
#define SCCSWI_INVALID_TIMESLOT     0x0E000010      /* Invalid timeslot provided */
#define SCCSWI_INVALID_CLOCK        0x0E000011      /* Invalid clock provided */
#define SCCSWI_INVALID_NETWORK      0x0E000012      /* Invalid network number provided */
#define SCCSWI_INVALID_SPEED        0x0E000013      /* Invalid clock speed provided */
#define SCCSWI_OUT_OF_MEMORY        0x0E000014      /* Unable to allocate memory */
#define SCCSWI_INVALID_CARRIER      0x0E000015      /* Invalid carrier number provided */
#define SCCSWI_NO_REARIO            0x0E000016      /* No Rear IO card detected */
#define SCCSWI_ERROR                0x0E000017      /* Error */
#define SCCSWI_NO_LICENSE           0x0E000018      /* No license for the type of the port */

/* ----- HBUS errors ----- */
#define SCCHBUS_BASE                0x0F000000
#define SCCHBUS_INVHANDLE           0x0F000001      /* Invalid handle */
#define SCCHBUS_INVINDEX            0x0F000002      /* Invalid index */
#define SCCHBUS_INVSTATE            0x0F000003      /* Invalid state */
#define SCCHBUS_INVIADD             0x0F000004      /* Invalid internal address*/
#define SCCHBUS_INVREQ              0x0F000005      /* Invalid request */
#define SCCHBUS_INVFLAGS            0x0F000006      /* Invalid message */
#define SCCHBUS_INVPARM             0x0F000007      /* Invalid parameter */
#define SCCHBUS_INVFUNC             0x0F000008      /* Function not supported */
#define SCCHBUS_INVKEY              0x0F000009      /* Invalid lower layer key */
#define SCCHBUS_INVERSION           0x0F00000A      /* Invalid version */
#define SCCHBUS_TXQFULL             0x0F00000B      /* TX queue full */
#define SCCHBUS_DEVERR              0x0F00000C      /* Device error */
#define SCCHBUS_UNAVAIL             0x0F00000D      /* Device unavailable */
#define SCCHBUS_NXST                0x0F00000E      /* Non-existant item */
#define SCCHBUS_LARGEMSG            0x0F00000F      /* Message too large */
#define SCCHBUS_SMALLMSG            0x0F000010      /* Message too small */
#define SCCHBUS_MISSPARM            0x0F000011      /* Missing parameter */
#define SCCHBUS_TXOVERFLOW          0x0F000012      /* TX overflow */
#define SCCHBUS_RXOVERFLOW          0x0F000013      /* RX overflow */
#define SCCHBUS_NOTSWPRIV           0x0F000014      /* Requestor is not software privileged */
#define SCCHBUS_OUT_OF_MEMORY       0x0F000015      /* Out of memory */
#define SCCHBUS_LEV1DEFINE          0x0F000016      /* Error defining level 1 driver */
#define SCCHBUS_HOSTNOTSYNC         0x0F000017      /* Host-HBUS not in sync */
#define SCCHBUS_NOT_SUPPORTED       0x0F000018      /* Function not supported */
#define SCCHBUS_RXQFULL             0x0F000019      /* RX queue full */

/* ----- ISR Handler ----- */
#define SCCISR_BASE                 0x10000000
#define SCCISR_OUT_OF_MEMORY        0x10000001      /* Error allocating memory */
#define SCCISR_OUT_OF_RANGE         0x10000002      /* ISR # specified is out of range */
#define SCCISR_DUPLICATE_ENTRY      0x10000003      /* ISR table entry already in use */
#define SCCISR_NOSUCH_ENTRY         0x10000004      /* ISR table entry doesn't exist */
#define SCCISR_INVALID_KEY          0x10000005      /* ISR Key is invalid */

/* ----- SYS LOG Errors -----*/
#define SCCLOG_BASE                 0x11000000
#define SCCLOG_UNKNOWN_RES          0x11000001      /* Unknown resource type */
#define SCCLOG_NOT_SUPPORTED        0x11000002      /* Requested function not supported */
#define SCCLOG_INVALID_STATE        0x11000003      /* Ring (or VSOCK) in invalid state */
#define SCCLOG_UNKNOWN_NOTIF        0x11000004      /* Unknown type of notification */

/* ----- HDLC/SCC Errors -----*/
#define SCCHDLC_BASE                0x12000000
#define SCCHDLC_OUT_OF_MEMORY       0x12000001      /* Memory allocate error */
#define SCCHDLC_INVHANDLE           0x12000002      /* Invalid resource handle */
#define SCCHDLC_INVSTATE            0x12000003      /* Invalid state for operation */
#define SCCHDLC_INVKEY              0x12000004      /* Lower interface key invalid */
#define SCCHDLC_LEV1DEFINE          0x12000005      /* Level 1 driver define error */
#define SCCHDLC_TXOVERFLOW          0x12000006      /* Transmit overflow */
#define SCCHDLC_DUPLICATE           0x12000007      /* Duplicate resource type registered */
#define SCCHDLC_NOSUCHTYPE          0x12000008      /* No such resource type registered */
#define SCCHDLC_INV_LINK            0x12000009      /* Invalid link # specified */
#define SCCHDLC_DUPNAME             0x1200000A      /* Duplicate resource or CPT name */
#define SCCHDLC_DUPHASHKEY          0x1200000B      /* Duplicate hash key */
#define SCCHDLC_INVALID_FUNCTION    0x1200000C      /* Function not supported */
#define SCCHDLC_INVNAME             0x1200000D      /* Invalid resource name */
#define SCCHDLC_INVLENGTH           0x1200000E      /* Packet size <=0 or >= 1518 */
#define SCCHDLC_LINKDOWN            0x1200000F      /* HDLC link is down */
#define SCCHDLC_SENDTOMBX           0x12000010      /* Send to another task's mailbox failed */
#define SCCHDLC_RXTXD_BUF_LIMIT     0x12000011      /* Add to RXTXD buffer chain failed */
#define SCCHDLC_INVRXTXD            0x12000012      /* Invalid RXTXD pointer passed */
#define SCCHDLC_RXTXD_ADJUST        0x12000013      /* Tried to adjust RXTXD buffer pointer error */
#define SCCHDLC_INVMSGTYPE          0x12000014      /* HDLC RX'd unknown message type */
#define SCCHDLC_NO_CHANNEL          0x12000015      /* Received message for unregistered channel */
#define SCCHDLC_NO_OWNER            0x12000016      /* no task has attached as owner of HDLC */

#define SCCMGR_BASE                 0x13000000
#define SCCMGR_NXST                 0x13000001      /* Item does not exist */
#define SCCMGR_INVSRVNUM            0x13000002      /* Invalid service number */
#define SCCMGR_INVINDEX             0x13000003      /* Invalid index */
#define SCCMGR_INVPOS               0x13000004      /* Invalid file position */
#define SCCMGR_EMPTY                0x13000005      /* Empty record */
#define SCCMGR_INSUFFMEM            0x13000006      /* Insufficient memory */
#define SCCMGR_FLASH_VERIFY         0x13000007      /* Flash verification error */
#define SCCMGR_INV_IMAGE            0x13000008      /* Invalid image detected */
#define SCCMGR_INV_SEQ              0x13000009      /* Invalid sequence number */
#define SCCMGR_INV_SIZE             0x1300000A      /* Invalid size detected */
#define SCCMGR_INV_CHKSUM           0x1300000B      /* Invalid checksum detected */
#define SCCMGR_INV_SECTOR           0x1300000C      /* Invalid sector detected */
#define SCCMGR_INV_BUSWIDTH         0x1300000D      /* Invalid bus width detected */
#define SCCMGR_INV_CMDSET           0x1300000E      /* Invalid command set detected */

/* ----- GT Serial Controller Errors -----*/
#define SCCMPSC_BASE                0x14000000
#define SCCMPSC_OUT_OF_MEMORY       0x14000001      /* Memory allocate error */
#define SCCMPSC_INVHANDLE           0x14000002      /* Invalid resource handle */
#define SCCMPSC_INVSTATE            0x14000003      /* Invalid state for operation */
#define SCCMPSC_INVKEY              0x14000004      /* Lower interface key invalid */
#define SCCMPSC_LEV1DEFINE          0x14000005      /* Level 1 driver define error */
#define SCCMPSC_TXOVERFLOW          0x14000006      /* Transmit overflow */
#define SCCMPSC_DUPLICATE           0x14000007      /* Duplicate resource type registered */
#define SCCMPSC_NOSUCHTYPE          0x14000008      /* No such resource type registered */
#define SCCMPSC_INV_LINK            0x14000009      /* Invalid link # specified */
#define SCCMPSC_DUPNAME             0x1400000A      /* Duplicate resource or CPT name */
#define SCCMPSC_DUPHASHKEY          0x1400000B      /* Duplicate hash key */
#define SCCMPSC_INVALID_FUNCTION    0x1400000C      /* Function not supported */
#define SCCMPSC_INVNAME             0x1400000D      /* Invalid resource name */
#define SCCMPSC_INVLENGTH           0x1400000E      /* Packet size <=0 or >= 1518 */
#define SCCMPSC_LINKDOWN            0x1400000F      /* MPSC link is down */
#define SCCMPSC_SENDTOMBX           0x14000010      /* Send to another task's mailbox failed */
#define SCCMPSC_RXTXD_BUF_LIMIT     0x14000011      /* Add to RXTXD buffer chain failed */
#define SCCMPSC_INVRXTXD            0x14000012      /* Invalid RXTXD pointer passed */
#define SCCMPSC_RXTXD_ADJUST        0x14000013      /* Tried to adjust RXTXD buffer pointer error */
#define SCCMPSC_INVMSGTYPE          0x14000014      /* MPSC RX'd unknown message type */

/* ----- $loader error codes -----*/
#define SCCLOAD_BASE                0x15000000
#define SCCLOAD_FILEERROR           0x15000001      /* ELF file error */
#define SCCLOAD_OUTOFMEMORY         0x15000002      /* Out of memory */
#define SCCLOAD_INVDATA             0x15000003      /* Invalid data type received */
#define SCCLOAD_INVREQ              0x15000004      /* Invalid request type received */
#define SCCLOAD_INVPARM             0x15000005      /* Invalid parameter value */
#define SCCLOAD_INVHANDLE           0x15000006      /* Invalid DLM handle */
#define SCCLOAD_OVERFLOW            0x15000007      /* DLM buffer overflow */
#define SCCLOAD_SYMBOLNOTFOUND      0x15000008      /* Symbol not found */
#define SCCLOAD_WRONGCPU            0x15000009      /* Wrong CPU type in DLM file */
#define SCCLOAD_SYS_NOTDEFINED      0x1500000A      /* Symbol table context not defined */
#define SCCLOAD_DLM_NOTDEFINED      0x1500000B      /* DLM not defined */
#define SCCLOAD_NO_UNRESOLVEDSYM    0x1500000C      /* No unresolved symbol found */
#define SCCLOAD_NOT_IMPLEMENTED     0x1500000D      /* Function not implemented */
#define SCCLOAD_EMPTYBUF            0x1500000E      /* ELF buffer is empty */
#define SCCLOAD_TSKEXEFAIL          0x1500000F      /* Task create failed */
#define SCCLOAD_NOLOADSECT          0x15000010      /* No loadable program section in ELF image */
#define SCCLOAD_LINKERERROR         0x15000011      /* Linker error */
#define SCCLOAD_INCOMPLETE          0x15000012      /* Loader accepted data, image not yet complete */
#define SCCLOAD_COMPLETE            0x15000013      /* Loader accepted data, image complete */
#define SCCLOAD_INVKERNEL           0x15000014      /* No valid kernel ELF image */
#define SCCLOAD_NO_SYM_TABLE        0x15000015      /* No valid symbol table */

/* ----- $MailBox error codes -----*/
#define SCCMAIL_BASE                0x16000000
#define SCCMAIL_OUT_OF_MEMORY       0x16000001      /* Memory allocate error */
#define SCCMAIL_INVHANDLE           0x16000002      /* Invalid resource handle */
#define SCCMAIL_INVSTATE            0x16000003      /* Invalid state for operation */

#define SCCMAIL_INVALID_FUNCTION    0x1600000B      /* Function not supported */

/* ----- MTP1 Errors -----*/
#define SCCMTP1_BASE                0x17000000
#define SCCMTP1_OUT_OF_MEMORY       0x17000001      /* Memory allocate error */
#define SCCMTP1_INVHANDLE           0x17000002      /* Invalid resource handle */
#define SCCMTP1_INVSTATE            0x17000003      /* Invalid state for operation */
#define SCCMTP1_INVKEY              0x17000004      /* Lower interface key invalid */
#define SCCMTP1_LEV1DEFINE          0x17000005      /* Level 1 driver define error */
#define SCCMTP1_TXOVERFLOW          0x17000006      /* Transmit overflow */
#define SCCMTP1_DUPLICATE           0x17000007      /* Duplicate resource type registered */
#define SCCMTP1_NOSUCHTYPE          0x17000008      /* No such resource type registered */
#define SCCMTP1_INV_LINK            0x17000009      /* Invalid link # specified */
#define SCCMTP1_DUPNAME             0x1700000A      /* Duplicate resource or CPT name */
#define SCCMTP1_DUPHASHKEY          0x1700000B      /* Duplicate hash key */
#define SCCMTP1_INVALID_FUNCTION    0x1700000C      /* Function not supported */
#define SCCMTP1_INVNAME             0x1700000D      /* Invalid resource name */
#define SCCMTP1_INVLENGTH           0x1700000E      /* Packet size <=0 or >= 1518 */
#define SCCMTP1_LINKDOWN            0x1700000F      /* MTP1 link is down */
#define SCCMTP1_SENDTOMBX           0x17000010      /* Send to another task's mailbox failed */
#define SCCMTP1_RXTXD_BUF_LIMIT     0x17000011      /* Add to RXTXD buffer chain failed */
#define SCCMTP1_INVRXTXD            0x17000012      /* Invalid RXTXD pointer passed */
#define SCCMTP1_RXTXD_ADJUST        0x17000013      /* Tried to adjust RXTXD buffer pointer error */
#define SCCMTP1_INVMSGTYPE          0x17000014      /* MTP1 RX'd unknown message type */
#define SCCMTP1_INV_PARAMETER       0x17000015      /* Invalid or out of range parameter */
#define SCCMTP1_TIMEOUT             0x17000016      /* Timeout occurred when waiting op. complete */
#define SCCMTP1_NO_OWNER            0x17000017      /* No MTP1 owner application has registered */

/* ----- Diagnostic task errors ----- */
#define SCCDIAG_BASE                0x18000000
#define SCCDIAG_INVSRV              0x18000001      /* Invalid service */
#define SCCDIAG_NOTSUP              0x18000002      /* Function not supported */
#define SCCDIAG_INVOPT              0x18000003      /* Invalid option */

/* ----- UART task errors ----- */
#define SCCUART_BASE                0x19000000
#define SCCUART_OUT_OF_MEMORY       0x19000001
#define SCCUART_INVHANDLE           0x19000002
#define SCCUART_INVSTATE            0x19000003
#define SCCUART_INVKEY              0x19000004
#define SCCUART_LEV1DEFINE          0x19000005
#define SCCUART_TXOVERFLOW          0x19000006
#define SCCUART_TXTIMEOUT           0x19000007
#define SCCUART_RXTIMEOUT           0x19000008
#define SCCUART_LINKDOWN            0x19000009
#define SCCUART_INVLENGTH           0x1900000A
#define SCCUART_NO_OWNER            0x1900000B      /* No task has attached as owner of UART */

/* ----- Serial task errors ----- */
#define SCCSERIAL_BASE              0x1A000000
#define SCCSERIAL_OUT_OF_MEMORY     0x1A000001
#define SCCSERIAL_NOT_FOUND         0x1A000002

/* ----- Command errors ----- */
#define SCCCMD_BASE                 0x1B000000
#define SCCCMD_IN_PROGRESS          0x1B000001      /* Command could not complete immediately, async notification will be generated when command completes */
#define SCCCMD_NOT_SUPPORTED        0x1B000002      /* Requested function or option is not currently supported */

/* ----- TX IP API errors ----- */
#define SCCIP_BASE                  0x1C000000
#define SCCIP_TOO_SMALL             0x1C000001      /* Provided buffer size is too small */
#define SCCIP_INVALID_TYPE          0x1C000002      /* Invalid (unknown) type encountered */
#define SCCIP_UNKNOWN_COMMAND       0x1C000003      /* Unknown command encountered */
#define SCCIP_INDEX_OUT_OF_RANGE    0x1C000004      /* Index is out of valid range */
#define SCCIP_INVALID_SIZE          0x1C000005      /* Invalid buffer size encountered */
#define SCCIP_NULL_POINTER          0x1C000006      /* Invalid NULL pointer detected */
#define SCCIP_UNKNOWN_CONFIG        0x1C000007      /* Unknown configuration field encountered */
#define SCCIP_OUT_OF_MEMORY         0x1C000008      /* Unable to allocate required memory */
#define SCCIP_NOT_IMPLEMENTED       0x1C000009      /* Requested operation has not been implemented */
#define SCCIP_NO_ROUTE              0x1C00000A      /* No route matching requested addressing */
#define SCCIP_TABLE_FULL            0x1C00000B      /* Given table is full - no more entries available */
#define SCCIP_NOT_FOUND             0x1C00000C      /* Indicated entity could not be found */
#define SCCIP_RESOURCE_NOT_AVAIL    0x1C00000D      /* Required resource not available */
#define SCCIP_INVALID_STATE         0x1C00000E      /* Entity in invalid state for requested operation */
#define SCCIP_INVALID_ID_KEY        0x1C00000F      /* Invalid identification key encountered */
#define SCCIP_INVALID_MAGIC         0x1C000010      /* Invalid 'magic number' encountered */
#define SCCIP_INVALID_HDR_TYPE      0x1C000011      /* Invalid header type encountered */
#define SCCIP_INVALID_ADDRESS       0x1C000012      /* Invalid address encountered */
#define SCCIP_FIELD_MISSING         0x1C000013      /* Missing a required field */
#define SCCIP_ARP_DELETED           0x1C000014      /* ARP entry was deleted */
#define SCCIP_ARP_TX_OVERFLOW       0x1C000015      /* ARP transmit queue overflow */
#define SCCIP_NO_IP_FORWARDING      0x1C000016      /* IP forwarding indicated when IP forwarding disabled */
#define SCCIP_NO_HW_RX_ROUTE        0x1C000017      /* No route found when routing receive at the hardware layer */
#define SCCIP_NO_IP_RX_ROUTE        0x1C000018      /* No route for received IP address */
#define SCCIP_GW_NO_RX_ROUTE        0x1C000019      /* Gateway route (on receive) does not map to active interface route */
#define SCCIP_RX_LOOP_ROUTE         0x1C00001A      /* Looped route entry detected on RX lookup */
#define SCCIP_NO_PROTO_RX_ROUTE     0x1C00001B      /* No route for received IP protocol */
#define SCCIP_NO_PORT_RX_ROUTE      0x1C00001C      /* No route for received protocol-specific port number */
#define SCCIP_NO_IFACE_ROUTE        0x1C00001D      /* No interface route to transmit IP packet over */
#define SCCIP_GW_NO_IFACE_ROUTE     0x1C00001E      /* Gateway route does not map to active interface route */
#define SCCIP_NO_ETH_RX_ROUTE       0x1C00001F      /* No route for received Ethernet packet */
#define SCCIP_ROUTE_NOT_EXIST       0x1C000020      /* Referenced route does not exist */
#define SCCIP_RX_TIMEOUT            0x1C000021      /* Timeout occurred while processing receive */
#define SCCIP_TOO_BIG               0x1C000022      /* Complete buffer size is too big to be processed */

/* ----- Ethernet error codes ----- */
#define SCCETH_BASE                 0x1D000000

/* Receive Error Codes */
#define SCCETH_RXMISS_FRAME         0x1D000001      /* Frame len >= max */
#define SCCETH_RXLARGE_FRAME        0x1D000002      /* Frame len >= max */
#define SCCETH_RXSHORT_FRAME        0x1D000003      /* Frame len < min eth len (64B) */
#define SCCETH_RXNONALIGNED_FRAME   0x1D000004      /* Non-octet algined frame */
#define SCCETH_RXCRCERROR_FRAME     0x1D000005      /* CRC error */
#define SCCETH_RXOVERRUN_FRAME      0x1D000006      /* FIFO overrun while receiving frame */
#define SCCETH_RXTRUNCATED_FRAME    0x1D000007      /* Truncated received frame */
#define SCCETH_RXMULTIPLE_FRAME     0x1D000008      /* Received data in multiple frames */

/* Transmit Error Codes */
#define SCCETH_TXDEFERRED_FRAME     0x1D001001      /* Transmitted frame was deferred */
#define SCCETH_TXLATECOL_FRAME      0x1D001002      /* A collision occured after 64 bytes were sent */
#define SCCETH_TXABORT_FRAME        0x1D001003      /* Transmission failed, max retry limit + 1 reached */
#define SCCETH_TXUNDERRUN_FRAME     0x1D001004      /* Underrun occured while sending out buffer */
#define SCCETH_TXTRUNCATED_FRAME    0x1D001005      /* TX frame len > max allowed, so truncated and sent out */
#define SCCETH_TXRETRY_FRAME        0x1D001006      /* One or more attempt were needed to send out the frame */
#define SCCETH_TXCARRIER_LOST       0x1D001007      /* Carrier sense lost */
#define SCCETH_NOT_SUPPORTED        0x1D002000      /* Not supported */

/* ----- I2C task errors ----- */
#define SCCI2C_BASE                 0x1E000000
#define SCCI2C_OUT_OF_MEMORY        0x1E000001      /* Insufficient memory */
#define SCCI2C_INVALID_HANDLE       0x1E000002      /* invalid handle */
#define SCCI2C_INVALID_STATE        0x1E000003      /* Invalid state */
#define SCCI2C_INVALID_KEY          0x1E000004      /* Invalid key */
#define SCCI2C_NOT_SUPPORTED        0x1E000005      /* Function not supported */
#define SCCI2C_IN_USE               0x1E000006      /* Device already in use */
#define SCCI2C_INVALID_ID           0x1E000007      /* Invalid device ID */
#define SCCI2C_FULL                 0x1E000008      /* DEvice queue full */
#define SCCI2C_TXOVERFLOW           0x1E000009      /* TX overflow */
#define SCCI2C_NO_OWNER             0x1E00000B      /* No task has attached as owner of I2C */

/* ----- SYSMON errors ----- */
#define SCCSMON_BASE                0x1F000000
#define SCCSMON_OUT_OF_MEMORY       0x1F000001      /* Unable to allocate required memory */

/* ----- MCC errors ----- */
#define SCCMCC_BASE                 0x20000000
#define SCCMCC_NOT_EXIST            0x20000001      /* MCC does not exist */
#define SCCMCC_TIMEOUT              0x20000002      /* Timeout while waiting for MCC command to commplete */

/* ----- WD errors ----- */
#define SCCWD_BASE                  0x21000000
#define SCCWD_EMPTY                 0x21000000
#define SCCWD_INVALID_PARAMETER     0x21000001
#define SCCWD_INVALID_MESSAGE_TYPE  0x21000002
#define SCCWD_TX_WINDOW_FULL        0x21000003
#define SCCWD_TX_FREE_LIST_EMPTY    0x21000004
#define SCCWD_INVALID_KEY           0x21000005
#define SCCWD_RX_OVERRUN            0x21000006
#define SCCWD_RX_FREE_LIST_EMPTY    0x21000007
#define SCCWD_LOCK_CREATE           0x21000008
#define SCCWD_INVALID_STATE         0x21000009

/* ----- OSAPI errors ----- */
#define SCCOSAPI_BASE               0x22000000
#define SCCOSAPI_INVALID_KEY        0x22000001      /* invalid key provided to OSAPI function */
#define SCCOSAPI_INTERRUPTED        0x22000002      /* attempt to lock was interrupted */

/* ----- ELF FILE errors ----- */
#define SCCELF_BASE                 0x23000000
#define SCCELF_FILE_NOT_FOUND       0x23000000
#define SCCELF_IDREC_NOT_FOUND      0x23000001
#define SCCELF_SECTION_NOT_FOUND    0x23000002
#define SCCELF_SYMBOL_NOT_FOUND     0x23000003
#define SCCELF_INVALID_INDEX        0x23000004

/* ----- Flash errors ----- */
#define SCCFLASH_BASE               0x24000000
#define SCCFLASH_BUSY_TIMEOUT       0x24000000
#define SCCFLASH_ERROR_CMDSEQ       0x24000001
#define SCCFLASH_ERROR_BLOCKERASE   0x24000002
#define SCCFLASH_ERROR_LOCK         0x24000003
#define SCCFLASH_ERROR_BLOCKLOCKED  0x24000004
#define SCCFLASH_ERROR_VPPLOW       0x24000005
#define SCCFLASH_ERROR_TOOBIG       0x24000006
#define SCCFLASH_ERROR_INPRODAREA   0x24000007

/* ----- Mutex errors ----- */
#define SCCMTX_BASE                 0x25000000
#define SCCMTX_LOCKED               0x25000000      /* mutex is locked */
#define SCCMTX_INVALID_ID           0x25000001      /* mutex id is invalid */

/* ----- SS7 Microcode errors ----- */
#define SCCSS7UCODE_BASE            0x26000000
#define SCCSS7UCODE_NULL            0x26000000      /* microcode NULL */
#define SCCSS7UCODE_TRAP            0x26000001      /* Illegal trap value */

#endif  /* _COSERROR_H */
