/*****************************************************************************
 *
 * cosflash.h - CPKOS Flash Layout Definitions
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                         PROPRIETARY NOTICE
 *
 *  (c)Copyright 1988-2009 Dialogic Corporation.  All Rights Reserved
 *
 *  The following is licensed program material and is considered
 *  proprietary and confidential.  No part of this program may be
 *  reproduced or used in any form or by any means without the express
 *  written permission of Dialogic Corporation.
 *
 *****************************************************************************/

#ifndef _COSFLASH_H
#define _COSFLASH_H

/* ----- dependent includes ----- */
#include "txnuser.h"
#include "txidrec.h"

/* ----- TX flash defines ----- */
#define TXFLASH_TOC_VERSION     1
#define TXFLASH_TOC_REVISION    1

#define TXFLASH_TOC_SECTION_CNT 128     /* number of image sections that can be represented in table of contents */

/*---------------------------------------------------------------------------*
 * Flash Table of Contents Header
 *---------------------------------------------------------------------------*/
typedef struct __txflash_toc_hdr
{
    U32         tocId;              /* table of contents [TOC] ID */
#define TXFLASH_TOC_ID  0x46544F43      /* 'FTOC' */

    U8          tocVersion;         /* version of the table of contents */
    U8          tocRevision;        /* revision of the table of contents */
    U8          avail1[2];
    U32         tocSectionCount;    /* count of TOC sections immediately following header */
    U8          avail2[52];
} TXFLASH_TOC_HDR;

/*---------------------------------------------------------------------------*
 * Flash Table of Contents Section
 *---------------------------------------------------------------------------*/
typedef struct __txflash_toc_section
{
    U8          sectionOwner;       /* indication of image owner (group responsible for maintaining section) */
#define TXFLASH_OWNER_AVAIL     0x00    /* section is available for use (not currently representing any image area) */
#define TXFLASH_OWNER_CPKOS     0x10    /* section refers to a CPKOS [TX Communications Processor Kernel (Operating System)] image */
#define TXFLASH_OWNER_MANUF     0x20    /* section refers to a manufacturing image */
#define TXFLASH_OWNER_DEVICE    0x30    /* section refers to a hardware device-specific image */

    U8          sectionInfo;        /* additional information (owner-specific) */
#define TXFLASH_CPKOS_LOADER    0x11    /* kernel image loader code */
#define TXFLASH_CPKOS_PROD      0x12    /* production kernel image */
#define TXFLASH_CPKOS_PROD_SYM  0x13    /* production kernel symbol table */
#define TXFLASH_CPKOS_FALLBACK  0x14    /* fallback (safety) kernel image */
#define TXFLASH_MANUF_BOOT      0x21    /* manufacturing boot code and diagnostics */
#define TXFLASH_DEVICE_SS7U     0x31    /* SS7 microcode image */
#define TXFLASH_DEVICE_TSI      0x32    /* Time Slot Interchanger image */

    U8          sectionIdIndex;     /* index into TXIDREC array where this section's ID record information is stored */
    U8          avail1[1];
    U32         sectionOffset;      /* byte offset (from flash base address) where image described by section's TXIDREC begins */
    U32         sectionSize;        /* size of section (in bytes) [when known; 0 = size unknown] */
    U32         legacyIdOffset;     /* offset into flash where TXIDREC is stored for legacy boards (prior to boards with an on-flash table of contents) */
    U32         legacyFlags;        /* flags for handling legacy flash images */
#define TXFLASH_LEGACY_ID_PRESENT   (1<<0)     /* set = legacyIdOffset holds valid offset to location of TXIDREC */

    U8          avail2[8];
} TXFLASH_TOC_SECTION;

/*---------------------------------------------------------------------------*
 * Flash Table of Contents
 *---------------------------------------------------------------------------*/
typedef struct __txflash_toc
{
    TXFLASH_TOC_HDR     hdr;                                /* table of contents header */
    TXFLASH_TOC_SECTION section[TXFLASH_TOC_SECTION_CNT];   /* table of contents section array [index=TXID_IMAGE_xxx (see txidrec.h)] */
    TXIDREC             idrec[TXFLASH_TOC_SECTION_CNT];     /* table of contents ID record array */
} TXFLASH_TOC;

#endif  /* _COSFLASH_H */
