/*****************************************************************************
 *
 * cosinfo.h - CPKOS System Information Definitions
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                         PROPRIETARY NOTICE
 *
 *  (c)Copyright 1988-2009 Dialogic Corporation.  All Rights Reserved
 *
 *  The following is licensed program material and is considered
 *  proprietary and confidential.  No part of this program may be
 *  reproduced or used in any form or by any means without the express
 *  written permission of Dialogic Corporation.
 *
 *****************************************************************************/

#ifndef _COSINFO_H
#define _COSINFO_H

/* ----- dependent includes ----- */
#include "txnuser.h"
#include "txipmsg.h"
#include "txinfomsg.h"
#include "coseprom.h"
#include "cosflash.h"

/* ----- well-known System Information name ----- */
#define NOBJ_MEMORY_SYSINFO     "System Information"

/* ----- System Utilization definitions ----- */
#define SYSUTIL_BUCKETS     16  /* number of utilization buckets maintained */
#define SYSUTIL_BUCKET_WRAP 0xF /* mask to apply when handling CPU bucket wrap */

/*****************************************************************************
 * System Utilization block - area used to compute CPU utilization
 *****************************************************************************/
typedef struct __sysutil
{
    U32         idle_count;     /* counter maintained by $idle task */
    U32         idle_peak;      /* peak value that idle_count has ever reached */
    U32         last_bucket;    /* last cpu_bucket that was updated */
    U32         cpu_bucket[SYSUTIL_BUCKETS];    /* per second history of idle_count */

    U32         avail[5];       /* room for future expansion */
} SYSUTIL;

/*****************************************************************************
 * System Time block - time of day / time passage handling area
 *****************************************************************************/
typedef struct __systimestamp
{
    U32  year;
    U32  month;
    U32  day;
    U32  hour;
    U32  min;
    U32  sec;
    S32  zone;
    U32  msec;
} SYSTIMESTAMP;

typedef struct __systime
{
    U32                 boot_msecs;     /* number of milliseconds since boot */
    U32                 epoch_secs;     /* number of seconds since epoch */
    U32                 bmsec_epoch;    /* number of msecs since boot when epoch secs were recorded */
    SYSTIMESTAMP        systime;        /* system time in timestamp format */

    U32                 bmsec_wraps;    /* number of times that milliseconds since boot has wrapped */
    U32                 avail[4];       /* room for future expansion */
} SYSTIME;

/*****************************************************************************
 * System CPU-Specific block - area defining CPU-specific OS features
 *****************************************************************************/
typedef struct __syscpu
{
    U32         idle_delay;     /* delay to approximate 1/2 sec of counting */
    U32         tclk_speed;     /* TCLK value, System Specific */
#define  TCLK_125MHZ    125000000
#define  TCLK_133MHZ    133000000

    U32         extranious_int; /* Count of extranious Interrupts */

    U32         curr_temp;      /* current CPU temperature (in degrees Celcius) */
    U32         fan_state;      /* current state of the fan (TXINFO_FAN_xxx) */
    U32         comm_barrier;   /* maximum retries to sync communications processor memory barrier */
    U32         eth_barrier;    /* maximum retries to sync Ethernet memory barrier */
    U32         avail[1];       /* room for future expansion */
} SYSCPU;

/*****************************************************************************
 * System Address block for Ethernets
 *****************************************************************************/
typedef struct __sysaddreth
{
    U32         eth_restype;    /* lower resource type */
    U8          eth_addr[6];    /* NIC Ethernet Address */
    U8          eth_avail[2];

} SYSADDRETH;
#define SYSADDRETH_MAX      4   /* max number of Ethernet interfaces available */

/*****************************************************************************
 * System General block - area defining general information
 *****************************************************************************/
typedef struct __sysgeneral
{
    U32         cpnum;          /* CP number */
    U32         sys_id;
#define TX_SYSID            0x5458L                 /* 'TX'   - TX series */
#define TX_SYSID_MASK       0xFFFFFF00              /* mask of all boards in same family */

#define TX3220_SYSID        (TX_SYSID<<16|0x3342)   /* 'TX3B' - TX 3220 system */

#define TX4_SYSID           (TX_SYSID<<16|0x3400)   /* 'TX4x' - TX 4000 series */
#define TX4000_SYSID        (TX4_SYSID|0x30)        /* 'TX40' - TX 4000 system */
#define TX4000C_SYSID       (TX4_SYSID|0x31)        /* 'TX41' - TX 4000C system */
#define TX4000_20_SYSID     (TX4_SYSID|0x32)        /* 'TX42' - TX 4000/20 system */
#define TX4000_20C_SYSID    (TX4_SYSID|0x33)        /* 'TX43' - TX 4000/20C system */
#define TX5000E_SYSID       5000                    /* TX 5000E (PCI Express) system family */
#define TX5020E_SYSID       5020                    /* TX 5020E (standard PCI Express board) */
#define TX5500E_SYSID       5500                    /* TX 5500E (enhanced PCI Express board) */

    U32         serial_num;     /* Board serial number */
    U32         sdram_size;     /* Size of SDRAM in megabytes */
    U32         num_ss7_links;  /* Number of SS7 Links */
    U32         eth_count;      /* number of ethernet interfaces on board */
    SYSADDRETH  eth[SYSADDRETH_MAX];    /* addressing info for each Ethernet */

    U32         avail[6];       /* room for future expansion */
} SYSGENERAL;

/*****************************************************************************
 * System Flash block - area providing flash information
 *****************************************************************************/
typedef struct __sysflash
{
    U32             type;       /* flash type */
#define TX_FLASH_AMD_800        0x00
#define TX_FLASH_AMD_160        0x01
#define TX_FLASH_AMD_320        0x02
#define TX_FLASH_MT_800T        0x03
#define TX_FLASH_MT_800B        0x04
#define TX_FLASH_ATMEL_8192     0x05
#define TX_FLASH_ATMEL_162A     0x06
#define TX_FLASH_ATMEL_162AT    0x07
#define TX_FLASH_ST_800AB       0x08
#define TX_FLASH_SH_800B        0x09
#define TX_FLASH_SH_160B        0x0A
#define TX_FLASH_ST_160EB       0x0B
#define TX_FLASH_SH_L29         0x0C
#define TX_FLASH_CFI            0x0D    /* Common Flash Memory Interface capable device */

    U32             algorithm;  /* flash programming algorithms */
#define TX_FLASH_ALG_AMD        0x01
#define TX_FLASH_ALG_INTEL      0x02
#define TX_FLASH_ALG_ATMEL      0x03
#define TX_FLASH_ALG_CFI        0x04    /* CFI specification [release 2.0] */

    U32             size;       /* flash size in KB */
    U32             base_addr;  /* flash base address */
    TXFLASH_TOC    *flash_toc;  /* flash table of contents - identifying all images stored on flash */
    U32             toc_source; /* source of table of contents */
#define TX_FLASH_TOC_CONSTRUCTED    1   /* table of contents was constructed from information on flash */
#define TX_FLASH_TOC_FLASH_COPY     2   /* table of contents was copied diretly from flash */

    U32             avail[8];   /* room for future expansion */
} SYSFLASH;

/*****************************************************************************
 * System EEPROM block - area providing EEprom information
 *****************************************************************************/
typedef struct __syseeprom
{
    union
    {
        TX4000_EEPROM   tx4000; /* EEPROM structure used by TX 4000 board family */

        TX5000_EEPROM   tx5000; /* EEPROM structure used by TX 5000 board family */

        U8              max_eeprom[192];    /* max space occupied by board EEPROM information */
    } txtype;

} SYSEEPROM;

/*****************************************************************************
 * System Kernel block - area providing kernel information
 *****************************************************************************/
typedef struct __syskernel
{
    U16     kernel_ver;         /* kernel version */
    U16     kernel_rev;         /* kernel revision */

    U32     avail[3];           /* room for future expansion */
} SYSKERNEL;

/*****************************************************************************
 * System Layout block - area providing hardware layout information
 *****************************************************************************/
typedef struct __syslayout
{
    U32     pci;                /* board PCI type information */
#define TXLO_PCI_IFACE  (1<<0)  /* set = board contains PCI interface */
#define TXLO_PCI_CPCI   (1<<1)  /* set = board contains CompactPCI interface */
#define TXLO_PCI_CPCI_REARIO   (1<<2)  /* set = REARIO Is Good */
#define TXLO_PCIE_IFACE (1<<3)  /* set = board contains PCI Express interface */

    U32     networks;           /* number of network (T1/E1) interfaces */
    U32     ls_framers;         /* mask indicating local streams that map to framers */
    U32     link_mode;          /* overall TDM link operating mode */
#define TXLINK_NO_TDM       0   /* no TDM links have been defined */
#define TXLINK_STANDARD     1   /* standard speed (64 Kb, 56 Kb or 48 Kb) link(s) defined */
#define TXLINK_HSL_T1       2   /* high speed link(s) over T1 defined */
#define TXLINK_HSL_E1       3   /* high speed link(s) over E1 defined */

    U32     avail[4];           /* room for future expansion */
} SYSLAYOUT;

/*****************************************************************************
 * System Clocking - area providing TDM clocking information
 *****************************************************************************/
#define SYSINFO_MAX_NETWORK  16  /* maximum network connection */

/* ----- clocking control signals ----- */
#define SYSCLK_CT_C8_A      0   /* H.100|H.110 A clocks */
#define SYSCLK_CT_C8_B      1   /* H.100|H.110 B clocks */
#define SYSCLK_CT_NETREF_1  2   /* H.100|H.110 NETREF[_1] signal */
#define SYSCLK_CT_NETREF_2  3   /* H.110 NETREF_2 signal */
#define SYSCLK_COMPAT       4   /* H.100|H.110 compatibility clocks */
#define SYSCLK_NETWORK      4   /* base... add network # (1..n) */
#define SYSCLK_MAX          (SYSCLK_NETWORK+SYSINFO_MAX_NETWORK)

/*--------------------------------------------------------------------------
 * Clocking Signal Information
 *--------------------------------------------------------------------------*/
typedef struct __sysclk_sig
{
    U32     sig_dir;    /* clock signal direction */
#define SYSCLK_DIR_NONE     0   /* direction not provided | not configured */
#define SYSCLK_DIR_INPUT    1   /* input signal */
#define SYSCLK_DIR_OUTPUT   2   /* output signal */

    U32     speed;      /* speed of clock [i.e. clock frequency (in Hz)] */
#define SYSCLK_FREQ_8KHZ        8000    /*      8 KHz */
#define SYSCLK_FREQ_1_5MHZ   1544000    /*  1.544 MHz */
#define SYSCLK_FREQ_2MHZ     2048000    /*  2.048 MHz */
#define SYSCLK_FREQ_4MHZ     4096000    /*  4.096 MHz */
#define SYSCLK_FREQ_8MHZ     8192000    /*  8.192 MHz */
#define SYSCLK_FREQ_16MHZ   16384000    /* 16.384 MHz */
#define SYSCLK_FREQ_64MHZ   65536000    /* 65.536 MHz */

    U32     avail[6];           /* room for future expansion */
} SYSCLK_SIG;

typedef struct __sysclock
{
    U32         intern_osc;             /* internal oscillator frequency (in Hz) */
    U32         netref_freq;            /* frequency (in Hz) of NETREF signal(s) [when known] */
    U32         main_fsc;               /* network # providing 8 KHz frame sync */
    U32         netref_fsc;             /* network # providing 8 KHz frame sync */

    SYSCLK_SIG  signal[SYSCLK_MAX+1];   /* information for all clocking signals */

    U32         avail[4];               /* room for future expansion */
} SYSCLOCK;

/*---------------------------------------------------------------------------*
 * System-Level IP Network information
 *---------------------------------------------------------------------------*/
#define SYSIP_MAX_UDP       32          /* maximum number of UDP ports reported in SYSIP */
#define SYSIP_MAX_PROTO     8           /* maximum number of IP protocols reported in SYSIP */
#define SYSIP_MAX_ROUTE     32          /* maximum number of entries in route table */
#define SYSIP_MAX_IFACE     16          /* maximum number of interfaces */
#define SYSIP_MAX_ETH       8           /* maximum number of Ethernet interfaces */
#define SYSIP_MAX_PING      8           /* maximum number of active PINGs */

typedef struct
{
    U32                 udpPort;        /* UDP port number */
    TXIP_UDP_CONFIG     config;
    TXIP_UDP_STATUS     status;
    TXIP_UDP_STATS      stats;
} SYSIP_UDP;

typedef struct
{
    U32                 protoNum;       /* IP protocol number */
    TXIP_PROTO_CONFIG   config;
    TXIP_PROTO_STATUS   status;
    TXIP_PROTO_STATS    stats;
} SYSIP_PROTO;

typedef struct
{
    TXIP_ADDR           ipAddr;         /* Interface IP address */
    TXIP_IFACE_CONFIG   config;
    TXIP_IFACE_STATUS   status;
    TXIP_IFACE_STATS    stats;
} SYSIP_IFACE;

typedef struct
{
    U32                 linkNum;        /* Ethernet link number */
    TXIP_ETH_CONFIG     config;
    TXIP_ETH_STATUS     status;
    TXIP_ETH_STATS      stats;
} SYSIP_ETH;

typedef struct
{
    TXIP_ADDR           local;
    TXIP_ADDR           remote;
    TXIP_PING_OPTIONS   options;
    TXIP_PING_STATUS    status;
    TXIP_PING_STATS     stats;
} SYSIP_PING;

typedef struct
{
    TXIP_COMM_STATS         dhcp;
    TXIP_COMM_STATS         udp;
    TXIP_COMM_STATS         proto;
    TXIP_COMM_STATS         iface;
    TXIP_COMM_STATS         arp;
    TXIP_COMM_STATS         eth;
} SYSIP_COMM;

typedef struct __sysip
{
    TXIP_ADDR               mateIpAddr;                 /* IP address of mate entity (redundancy mate) */

    TXIP_NETWORK_STATUS     netStatus;                  /* quick overview of network "status" [counts of entries in other SYSIP arrays] */
    volatile SYSIP_UDP     *udp[SYSIP_MAX_UDP];         /* set of UDP ports monitored in SYSIP */
    volatile SYSIP_PROTO   *proto[SYSIP_MAX_PROTO];     /* set of IP protocols monitored in SYSIP */
    volatile TXIP_ROUTE    *route[SYSIP_MAX_ROUTE];     /* routing table */
    volatile SYSIP_IFACE   *iface[SYSIP_MAX_IFACE+1];   /* set of IP interfaces monitored in SYSIP (index = interface number; entry [0] RESERVED) */
    volatile SYSIP_ETH     *eth[SYSIP_MAX_ETH+1];       /* set of Ethernet links monitored in SYSIP (index = interface number; entry [0] RESERVED) */
    volatile SYSIP_PING    *ping[SYSIP_MAX_PING];       /* set of PINGs monitored in SYSIP */
    volatile SYSIP_COMM     comm;                       /* IP communications summary statistics */
    U32                     avail[32];                  /* room for future expansion */
} SYSIP;

/*****************************************************************************
 * System Monitoring (System Status Information)
 *****************************************************************************/
#define SYSMON_NAME_SIZE        32      /* bytes reserved for the name of the monitored field */
#define SYSMON_HIST_COUNT       60      /* keep a histogram of the last 60 samples taken */
#define SYSNON_HIST_INTERVAL    60000   /* number of milliseconds between histogram entries */

/*---------------------------------------------------------------------------*
 * Structure used for monitoring a system status value
 *---------------------------------------------------------------------------*/
typedef struct __sysmon_value
{
    S8          name[SYSMON_NAME_SIZE];     /* name of the given value */
    U32         type;                       /* for field types see cossdc.h */
    U32         level;                      /* level that the monitored value is considered to be at currently */
#define SYSMON_LEVEL_OFF        0           /* value is not in use so level considered OFF */
#define SYSMON_LEVEL_UNKNOWN    1           /* value has not been determined yet */
#define SYSMON_LEVEL_LOW        2
#define SYSMON_LEVEL_NORMAL     3
#define SYSMON_LEVEL_RAISED     4
#define SYSMON_LEVEL_HIGH       5
#define SYSMON_LEVEL_URGENT     6

    U32         curr;                       /* last value provided in a resource response (considered current until a fresher response is received) */
    U32         min;                        /* minimum value ever sampled */
#define SYSMON_MIN_INIT 0xFFFFFFFF

    U32         max;                        /* maximum value ever sampled */
    U32         histAvg;                    /* average of all samples within the current histogram period */
    U32         histSamples;                /* total number of samples represented in histAvg */
    U32         histLast;                   /* last index used to write into histogram */
    U32         hist[SYSMON_HIST_COUNT];    /* histogram of readings taken over time */
    U32         avail[7];                   /* room for future expansion */
} SYSMON_VALUE;

/*---------------------------------------------------------------------------*
 * Structure used for monitoring a system state  (Currently unused)
 *---------------------------------------------------------------------------*/
typedef struct __sysmon_state
{
    S8          name[SYSMON_NAME_SIZE];     /* name of the entity whos state is being monitored */
    U32         type;                       /* for field types see cossdc.h (should always be TXSDC_STATE) */
    U32         state;                      /* last state provided in a resource response (considered the current state until a fresher response is received) */
#define SYSMON_STATE_UNUSED         0
#define SYSMON_STATE_UNKNOWN        1       /* state has not been determined yet */
#define SYSMON_STATE_NORMAL         2
#define SYSMON_STATE_FAILURE        (1<<31)

    U32         failTotal;                  /* total number of times that state has transitioned into the FAILURE state */
    U32         alarm;                      /* alarm level (SYSMON_LEVEL_xxx [OFF = not in any alarm condition]) */
    U32         avail[8];                   /* room for future expansion */
} SYSMON_STATE;

/*---------------------------------------------------------------------------*
 * Monitored entity category
 *---------------------------------------------------------------------------*/
#define SYSMON_VALUE_COUNT      16          /* maiximum number of values per category */
typedef struct __sysmon_category
{
    SYSMON_VALUE    value[SYSMON_VALUE_COUNT];
} SYSMON_CATEGORY;

#define SYSMON_MAX_CAT      3   /* three categories - temperature, fan, and power */

/*---------------------------------------------------------------------------*
 * System Status Monitoring Information (not all fields available on all boaard types)
 *---------------------------------------------------------------------------*/
#define SYSMON_TEMP_COUNT       8           /* maximum number of temperature sensors that can be monitored via SYSINFO */
#define SYSMON_POWER_COUNT      16          /* maximum number of power levels that can be monitored via SYSINFO */
#define SYSMON_FAN_COUNT        8           /* maximum number of fans that can be monitored via SYSINFO */

/* ----- well-known indexes into any of the TEMP/CURRENT/FAN arrays ----- */
#define SYSMON_INDEX_CPU        0           /* index into TEMP/CURRENT/FAN array to access main-CPU-specific monitor */
#define SYSMON_INDEX_BOARD      1           /* index into TEMP/CURRENT/FAN array to access board-level monitor */
#define SYSMON_INDEX_PRIMARY    2           /* index into TEMP/CURRENT/FAN array to access primary monitor */
#define SYSMON_INDEX_SECONDARY  3           /* index into TEMP/CURRENT/FAN array to access secondary monitor */

typedef struct __sysmon
{
    U32             histInterval;               /* number of seconds represented by each histogram entry */
    SYSMON_CATEGORY category[SYSMON_MAX_CAT];
    U32             avail[8];                   /* room for future expansion */
} SYSMON;

/*****************************************************************************
 * System Information block - TX board system level information
 *
 * locate using sysinfoGet( );
 *****************************************************************************/
typedef struct __sysinfo
{
    U32         sysinfo_id;
#define SYSINFO_ID  0x53595349L /* 'SYSI' */

    SYSUTIL     util;           /* system utilization area */
    SYSTIME     time;           /* time handling area */
    SYSCPU      cpu;            /* CPU-specific area */
    SYSGENERAL  general;        /* system General area */
    SYSKERNEL   kernel;         /* kernel information area */
    SYSFLASH    flash;          /* flash information area */
    SYSEEPROM   eeprom;         /* Interesting EEPROM info */
    SYSLAYOUT   layout;         /* hardware layout info */
    SYSCLOCK    clock;          /* TDM clocking information */
    SYSIP       ip;             /* IP network information */
    SYSMON     *smon;           /* system monitoring */
    void       *swTestResults;  /* pointer to software test results (set to test-specific results record upon completion of test */
    U32         avail[30];      /* room for future expansion */
} SYSINFO;

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
 * PROTOTYPES
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
volatile SYSINFO   *sysinfoGet  ( void );

#endif  /* _COSINFO_H */
