/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**                          Copyright 2006 - 2009                      **
**                Dialogic Corporation.  All Rights Reserved            **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Dialogic Corporation.                         **
**                                                                      **
*************************************************************************/
#ifndef _MONDEFS_H_
#define _MONDEFS_H_

/* Some miscellaneous definitions for monitor */
#define MON_MAXFILTERS  3               /* Max filters per link */
#define MON_ALL_LINKS   0xFF            /* Define for all links (for filters) */

/* Trace levels for the monitor task */
#define MON_TRACE_ERROR     0x01
#define MON_TRACE_WARNING   0x02
#define MON_TRACE_DEBUG     0x10

/********************************************************************/
/*** Monitor General configuration structure                      ***/
/********************************************************************/

typedef struct _MonGenCfg
{
    U8      recvLssu;                   /* Send up LSSUs? */
    U8      recvFisu;                   /* Send up FISUs? */
    U8      mtp2Hdr;                    /* Include the mtp2Hdr */
    U8      spare;
    U16     numBuffers;                 /* Number of buffers in limited pool */
    U16     sizeBuffers;                /* Size of each buffer in limited pool */
    TimerCfg tInd;                      /* Max time to wait for data ind. */
} MonGenCfg;

/********************************************************************/
/*** Monitor Link configuration structure                         ***/
/********************************************************************/

typedef struct
{
    U16     linkNum;                    /* Link Number */
    U16     linkType;                   /* ANSI/ITU */
#define MON_LINK_ANSI   0
#define MON_LINK_ITU    1
#define MON_LINK_ANSI_EXT   2
#define MON_LINK_ITU_EXT    3
    U16     portNum;                    /* Port number (from tdmcfg) */
    U16     portType;                   /* Port type (tdm v. serial) */
#define MON_PTYPE_SERIAL    0
#define MON_PTYPE_TDM       4
    U16     dataEnc;                    /* Data encoding (NRZ/NRZI) */
#define DAT_NRZ      0                  /* data format is NRZ */
#define DAT_NRZI     1                  /* data format is NRZI */
    U16     maxFrameLen;                /* Maximum SS7 frame length */
    U16     numBuffers;                 /* Number of buffers in limited pool */
    U16     sizeBuffers;                /* Size of each buffer in limited pool */
} MonLinkCfg;

/********************************************************************/
/*** Monitor General statistics structure                         ***/
/********************************************************************/

typedef struct _MonGenStats
{
    S16   action;           /* Action to take (ZEROSTS or NOZEROSTS) */
    S16   spare1;           /* Alignment */
    U32   numAppCongDrop;   /* Number of messages dropped due to congestion
                             * (limited pool up to host is full) */
} MonGenStats;

/********************************************************************/
/*** Monitor Link statistics structure                            ***/
/********************************************************************/

typedef struct _MonLinkStats         /* MTP level 2 link statistics */
{
    S16   action;           /* Action to take (ZEROSTS or NOZEROSTS) */
    S16   spare1;           /* Alignment */
    U32   numMsuRx;         /* Number of MSUs received */
    U32   numLssuRx;        /* Number of LSSUs sent up to host */
    U32   numFisuRx;        /* Number of FISUs sent up to host */
    U32   numRxFiltered;    /* Number of messages dropped due to filter configuration */
    U32   numSUERMFail;     /* Number of times the SUERM has failed on this link  */
} MonLinkStats;


/********************************************************************/
/*** Monitor Filter definition structure                          ***/
/********************************************************************/

typedef struct _MonFilterDef
{
    U8  type;
#define MON_FILTER_SIO  1               /* Filter types */
#define MON_FILTER_DPC  2
#define MON_FILTER_OPC  3
    U8  spare;
    U16 spare2;
    U32 value;
} MonFilterDef;

/********************************************************************/
/*** Monitor Filter List definition structure                     ***/
/********************************************************************/

typedef struct _MonFilterList
{
    U8              numActive;
    U8  spare;
    U16 spare2;
    MonFilterDef    filter[MON_MAXFILTERS];
} MonFilterList;

#endif  /* _MONDEFS_H_  */
