/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**                          Copyright 2006 - 2009                      **
**                Dialogic Corporation.  All Rights Reserved            **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Dialogic Corporation.                         **
**                                                                      **
*************************************************************************/
#ifndef _MONMAPI_H_
#define _MONMAPI_H_

#include "mondefs.h"

#ifdef UNIX

#ifndef TRUE
#define TRUE  1
#endif

#ifndef FALSE
#define FALSE 0
#endif

#endif /* Unix */

/* Macro for initializing a timer (value 0 => disable) */
#define MON_SET_TIMER( atimer, avalue )    \
            atimer.value = avalue;         \
    if( atimer.value != 0 )                 \
        atimer.enable = 1;                  \
    else                                    \
        atimer.enable = 0;

/* Error type returned by API functions */
typedef S16 MON_STATUS;   /* redefine for portability */

/* Mgmt API return codes */
#define MON_SUCCESS    0           /* Successful */
#define MON_LNKTYPE    0x1001      /* Invalid link/switch type */
#define MON_PARAM      0x1002      /* Invalid parameter */
#define MON_BOARD      0x1004      /* board number out of range */
#define MON_DRIVER     0x1005      /* CPI driver returned error        */
#define MON_HANDLE     0x1006      /* Board not previously initialized */
#define MON_TIMEOUT    0x1007      /* No response from board */
#define MON_RESPONSE   0x1008      /* Wrong response from board */

/* define the calling conventions for the API functions for various
 * operating systems - NOTE: API callers should never define the
 * preprocessor constant "_TX_MON_SOURCE_"; this is only for use
 * by the MON API source code itself.
 */

/* WIN32: MON API is a DLL */
#ifdef  WIN32                            /* WIN32 */
#ifdef _MON_SOURCE_
    /* MON API source definition */
#define     MONFUNC               __declspec( dllexport ) WINAPI
#else
    /* MON API caller's definition */
#define     MONFUNC               WINAPI
#endif  /* _MON_SOURCE_ */

/* all others: statically linked library */
#else
#define    MONFUNC
#endif


#ifdef __cplusplus
extern "C" {
#endif

MON_STATUS MONFUNC MonMgmtInit       ( U8 board, U8 srcEnt, U8 srcInst );
MON_STATUS MONFUNC MonMgmtTerm       ( U8 board );

MON_STATUS MONFUNC MonInitGenCfg     ( U8 board, MonGenCfg *pGenCfg );
MON_STATUS MONFUNC MonSetGenCfg      ( U8 board, MonGenCfg *pGenCfg );
MON_STATUS MONFUNC MonGetGenCfg      ( U8 board, MonGenCfg *pGenCfg );

MON_STATUS MONFUNC MonInitLinkCfg    ( U8 board, MonLinkCfg *pLinkCfg, S16 linkNo, U8 linkType );
MON_STATUS MONFUNC MonSetLinkCfg     ( U8 board, MonLinkCfg *pLinkCfg );
MON_STATUS MONFUNC MonGetLinkCfg     ( U8 board, MonLinkCfg *pLinkCfg, S16 linkNo);

MON_STATUS MONFUNC MonSetFilter      ( U8 board, MonFilterDef *pFilter, S16 linkNo );
MON_STATUS MONFUNC MonGetFilters     ( U8 board, MonFilterList *pList, S16 linkNo );
MON_STATUS MONFUNC MonClearFilters   ( U8 board, S16 linkNo);

MON_STATUS MONFUNC MonGetGenStats    ( U8 board, MonGenStats *pStats, Bool bReset);
MON_STATUS MONFUNC MonGetLinkStats   ( U8 board, MonLinkStats *pStats, S16 linkNo, Bool bReset);

MON_STATUS MONFUNC MonStartLink      ( U8 board, S16 linkNum);
MON_STATUS MONFUNC MonStopLink       ( U8 board, S16 linkNum);

MON_STATUS MONFUNC MonSetTrace       ( U8 board, U16 traceLevel);

#ifdef __cplusplus
}
#endif


#endif  /* _MONMAPI_H_  */
