/*****************************************************************************
 *
 * txidrec.h - TX Identification Record
 *
 * NOTE: TXIDREC follows Motorola Linker Indentification Record Format
 *****************************************************************************/

/*****************************************************************************
 *
 *                         PROPRIETARY NOTICE
 *
 *  (c)Copyright 1988-2009 Dialogic Corporation.  All Rights Reserved
 *
 *  The following is licensed program material and is considered
 *  proprietary and confidential.  No part of this program may be
 *  reproduced or used in any form or by any means without the express
 *  written permission of Dialogic Corporation.
 *
 *****************************************************************************/

#ifndef _TXIDREC_H
#define _TXIDREC_H

/* dependent include files */
#include "txnuser.h"

#define TXID_MNAMELEN   10
#define TXID_SNAMELEN   24
#define TXID_MAXDESCR   200
#define DESCRIPTION(n,v,r,b,d,m,y,t) (#n " version " #v "." #r " Build " #b "\n(c)Copyright " #y " Dialogic Corporation \nBuilt "" " #m " "  #d ", " #y " " #t)

/*---------------------------------------------------------------------------*
 * TXID HTIME
 *---------------------------------------------------------------------------*/
typedef struct
{
    U8  hh;     /* hours */
    U8  mm;     /* minutes */
    U8  ss;     /* seconds */
} TXID_HTIME;

/*---------------------------------------------------------------------------*
 * TXID HDATE
 *---------------------------------------------------------------------------*/
typedef struct
{
    U8  mm;     /* month */
    U8  dd;     /* day */
    U8  yy;     /* year */
} TXID_HDATE;

/*---------------------------------------------------------------------------*
 * TXID Header
 *---------------------------------------------------------------------------*/
typedef struct
{
    U8          bcount;     /* byte count of record */
    U8          rectyp;     /* record type */
#define TXID_RT_IDENT_V1 0x31    /* "1" = legacy identification record */
#define TXID_RT_IDENT_V2 0x32    /* "2" = version 2 identification record */

    U8          mname[TXID_MNAMELEN];    /* module name */
    U8          vnum;       /* version number */
    U8          rnum;       /* revision number */

    S8          lang;       /* language type */
#define TXID_LT_EST      0x50    /* "P" = EST/PPC TX Load module */
#define TXID_LT_SYM      0x53    /* "S" = TX/PPC symbol table */
#define TXID_LT_ELF      0x45    /* "E" = PPC ELF file */

    S8          sname[TXID_SNAMELEN];    /* source filename */
    TXID_HTIME  ctime;      /* creation time (hhmmss) */
    TXID_HDATE  cdate;      /* creation date (mmddyy) [where year is number of years since 1900] */
    U8          patch;      /* patch number */
    U8          align[2];
} TXID_HDR;

/*---------------------------------------------------------------------------*
 * TXID IMAGE HEADER
 *
 * NOTE: !!!!! Fields in this structure are stored in little-endian byte order!
 *---------------------------------------------------------------------------*/
typedef struct
{
    U16     imagetype;  /* [Little Endian] image type */
#define TXID_IMAGE_MANUF_BOOT       1       /* manufacturing boot code and diagnostics */
#define TXID_IMAGE_CPKOS_PROD       2       /* production kernel image */
#define TXID_IMAGE_CPKOS_PROD_SYM   3       /* production kernel symbol table */
#define TXID_IMAGE_CPKOS_LOADER     4       /* kernel image loader code */
#define TXID_IMAGE_CPKOS_FALLBACK   5       /* fallback (safety) kernel image */
#define TXID_IMAGE_DEVICE_SS7U      6       /* SS7 microcode image */
#define TXID_IMAGE_DEVICE_TSI       7       /* Time Slot Interchanger image */
#define TXID_IMAGE_CPKOS_TOC        8       /* Table of contents */
#define TXID_IMAGE_CPKOS_TASK       9       /* Downloadable task */
#define TXID_IMAGE_MIN              1
#define TXID_IMAGE_MAX              9

    U16     cputype;    /* [Little Endian] type of processor that an image pertains to (when applicable) */
#define TXID_CPU_PPC7450        1
#define TXID_CPU_PPC_PQII       2

    U32     imagesize;  /* [Little Endian] size of image in bytes, not including TXIDREC */
#define TXID_MAX_IMGSIZE_PROD       0x400000  /* max image size for production kernel, 4M */
} TXID_IMAGEHDR;

/*---------------------------------------------------------------------------*
 * TXID Record - TX image identification information
 *---------------------------------------------------------------------------*/
typedef struct txidrec
{
    TXID_HDR        idhdr;                  /* file information header */
    TXID_IMAGEHDR   imagehdr;               /* image information header */
    S8              descr[TXID_MAXDESCR];   /* variable length description */
} TXIDREC;

#endif  /* _TXIDREC_H */
