/*****************************************************************************
 *
 * txmgr.h - TX manager message definitions
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                         PROPRIETARY NOTICE
 *
 *  (c)Copyright 1988-2009 Dialogic Corporation.  All Rights Reserved
 *
 *  The following is licensed program material and is considered
 *  proprietary and confidential.  No part of this program may be
 *  reproduced or used in any form or by any means without the express
 *  written permission of Dialogic Corporation.
 *
 *****************************************************************************/

#ifndef _TXMGR_H
#define _TXMGR_H

#include "txnuser.h"                /* primitive field type definitions */
#include "dpriface.h"

/*###########################################################################*
 *
 * MANAGER COMMAND INTERFACE
 *
 * The manager task provides host applications with a variety of management
 * services.  The header structure defined below is used for all
 * communication to the manager task.  Other include files provide additional
 * definitions for particular management functions.
 *###########################################################################*/

/*---------------------------------------------------------------------------
 * Command Header
 *---------------------------------------------------------------------------*/
typedef struct __mgr_cmdhdr
{
    U8              version;    /* version of message protocol in use */
#define MGR_VERSION         2   /* current version */

    U8              subver;     /* sub-version of message protocol */
#define MGR_SUBVER   1          /* current sub-version */

    U8              category;   /* category of commands */
#define MGR_FLASH    1          /* internal protocol: FLASH Control */
#define MGR_EEPROM   2          /* internal protocol: EEPROM Control */
#define MGR_LED      3          /* internal protocol: LED Control */
#define MGR_NMI      4          /* internal protocol: NMI Control */
#define MGR_TEST     5          /* internal protocol: DIAGNOSTIC TEST Control */

    U8              command;    /* command to perform */
#define MGR_CATEG    0x80       /* base for category-specific commands */

    U8              dir;        /* direction of command */
#define MGR_REQUEST  1          /* Request where Reply expected */
#define MGR_REPLY    2          /* Reply to previous Request */
#define MGR_NOTIF    3          /* Notification (no Reply expected) */

    U8              boardType;  /* Responses contain the board type */
#define MGR_BT_TX4000 4
    U8              avail[2];
} MGR_CMDHDR;

/*###########################################################################*
 *
 * HOST-SIDE MANAGER CONTROL PERFORMED VIA CPI_READ/WRITE_CONTROL
 *
 *###########################################################################*/

/*
 *  Control Registers (used for cpi_read/write_control)
 */
#define TXMGR_TEST_REG          7           /* basereg to use when performing test */
#define TXMGR_MEMSWEEP_TEST     (1<<0)      /* perform the memory sweep test */

/*---------------------------------------------------------------------------
 * Control Test Results
 *---------------------------------------------------------------------------*/
typedef struct __mgr_test_results
{
    U32             test;       /* test that was performed [see TXMGR_TEST_REG] */
    U32             state;      /* current state of the test [optional, test-specific] */
    U8              avail1[32]; /* available for future test configuration information */

    U32             result;     /* 0 = success; else error code (SCCMGR_xxx) */
    U32             failAddr;   /* address that test failed on (if result != 0) */
    U32             expected;   /* expected value */
    U32             actual;     /* actual value */
    U32             info;       /* any additional information */

    U8              avail2[32];
} MGR_TEST_RESULTS;

#endif /* _TXMGR_H */
