
/******************************************************************
 **
 **                   PROPRIETARY NOTICE
 **
 **  (c)Copyright 2000-2009 Dialogic Corporation. All Rights Reserved.
 **
 **
 **  The following is licensed program material and is considered
 **  proprietary and confidential.  No part of this program may be
 **  reproduced or used in any form or by any means without the
 **  express written permission of Dialogic Corporation
 **
 *****************************************************************/

/* MVIP T1/E1 test application */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#ifdef WIN32
#include <windows.h>
#endif
#include "txcpi.h"
#include "t1e1type.h"
#include "t1e1api.h"
#include "mviptype.h"
#include "mvipapi.h"

/* ----- Modification History: --------------------------------------------- */
#define MH_VERSION      5           /* identifier of major release version */
#define MH_REVISION     0           /* identifier of release revision */

/* Macro for extracting the value of a single bit from an unsigned long
 *  0 <= bitno <= 31  (0 = LSB, 31 = MSB)
 */
#define BITVAL( bitmask, bitno )                \
    (((bitmask & ((unsigned long) 1 << bitno)) != 0) ? 1 : 0)

void            HelpCommand(U32 *parms);

/* t1e1 routines */
void            configCarrier(U32 *parms);
void            carrierStatus(U32 *parms);
void            perfReport(U32 *parms);
void            ctrlCarrier(U32 *parms);
void            conditionChan(U32 *parms);
void            configChan(U32 *parms);
void            channelStatus(U32 *parms);
void            supvCarrier(U32 *parms);
void            setCP(U32 *parms);
void            quit(U32 *parms);

/* mvip routines */
void            configClock(U32 *parms);
void            queryOutput(U32 *parms);
void            querySwCaps(U32 *parms);
void            resetSwitch(U32 *parms);
void            sampleInput(U32 *parms);
void            setOutput(U32 *parms);
void            setOutputFdx(U32 *parms);
void            switchStatus(U32 *parms);
void            tristateSwitch(U32 *parms);

static char    *usagestr = "Usage: %s [-b board]\n";

/* lookup tables for translating names into values and vice versa */
struct lookupEnt
{
    char           *name;
    short           value;
    char            pad[2];
};

/* t1e1 parameter name tables */
struct lookupEnt FramingLookupTbl[] = { /* framing type values */
    {"NONE", FRAMING_NONE},
    {"D4", FRAMING_D4},
    {"ESF", FRAMING_ESF},
    {"CCS", FRAMING_CCS},
    {"CAS", FRAMING_CAS},
    {"CCSCRC4", FRAMING_CCSCRC4},
    {"CASCRC4", FRAMING_CASCRC4},
    {0, 0}                          /* end of table */
};

struct lookupEnt CodingLookupTbl[] = {  /* line coding values */
    {"NONE", ENCODING_NONE},
    {"NOZCS", ENCODING_NOZCS},
    {"B7S", ENCODING_BIT7_ZCS},
    {"B8ZS", ENCODING_B8ZS},
    {"HDB3", ENCODING_HDB3},
    {0, 0}                          /* end of table */
};

struct lookupEnt DisEnaLookupTbl[] = {  /* General disable/enable  */
    {"DIS", 0},
    {"ENA", 1},
    {0, 0}                          /* end of table */
};

struct lookupEnt NetLookupTbl[] = { /* network interface values */
    {"A", NET_T1A},
    {"B", NET_T1B},
    {"C", NET_T1C},
    {"D", NET_T1D},
    {"E", NET_T1E},
    {"F", NET_T1F},
    {"G", NET_T1G},
    {"H", NET_T1H},
    {0, 0}                          /* end of table */
};

struct lookupEnt TimingLookupTbl[] = {  /* loop timing values */
    {"SLAVE", LOOPTIMING_SLAVE},
    {"MASTER", LOOPTIMING_MASTER},
    {0, 0}                          /* end of table */
};

struct lookupEnt LoopbackLookupTbl[] = {    /* loopback mode values */
    {"NONE", LOOPBACK_NONE},
    {"LOCAL", LOOPBACK_LOCAL},
    {"REM", LOOPBACK_REMOTE},
    {0, 0}                          /* end of table */
};

struct lookupEnt AlarmLookupTbl[] = {   /* alarm mode values */
    {"NONE", NO_ALARM},
    {"YELLOW", YELLOW_ALARM},
    {"BLUE", BLUE_ALARM},
    {"TS16AIS", TS16AIS_ALARM},
    {"TS16SIG", TS16SIG_ALARM},
    {"DISTMF", DISTANTMF_ALARM},
    {0, 0}                          /* end of table */
};

struct lookupEnt SyncLookupTbl[] = {    /* alarm mode values */
    {"NONE", CARRIER_NONE},
    {"SYNC", CARRIER_SYNC},
    {"NOSYNC", CARRIER_NOSYNC},
    {"LOST", CARRIER_LOST},
    {0, 0}                          /* end of table */
};

/* mvip parameter names */
struct lookupEnt ClockLookupTbl[] = {   /* main clock signal values */
    {"MVIP", CLOCK_REF_MVIP},
    {"SEC8K", CLOCK_REF_SEC8K},
    {"OSC", CLOCK_REF_OSC},
    {"NET", CLOCK_REF_NET},
    {0, 0}                          /* end of table */
};

struct lookupEnt Sec8kLookupTbl[] = {   /* Sec8K clock values */
    {"SLAVE", SEC8K_NOT_DRIVEN},
    {"OSC", SEC8K_DRIVEN_BY_OSC},
    {"NET", SEC8K_DRIVEN_BY_NET},
    {0, 0}                          /* end of table */
};

struct lookupEnt ModeLookupTbl[] = {    /* connect mode values */
    {"DIS", DISABLE_MODE},
    {"PAT", PATTERN_MODE},
    {"CONN", CONNECT_MODE},
    {0, 0}                          /* end of table */
};

struct lookupEnt TriLookupTbl[] = { /* Tristate request values */
    {"ENA", ENABLE_OUTPUT},
    {"DIS", TRISTATE_OUTPUT},
    {0, 0}                          /* end of table */
};

struct lookupEnt CmdLookupTbl[] = {
    {"CL", 1},
    {"LS", 2},
    {"PR", 3},
    {"LC", 4},
    {"CD", 5},
    {"CC", 6},
    {"CS", 7},
    {"SV", 8},
    {"Q", 9},
    {"CP", 10},
    {"CK", 11},
    {"QO", 12},
    {"QS", 13},
    {"RS", 14},
    {"SI", 15},
    {"SO", 16},
    {"SF", 17},
    {"SS", 18},
    {"TR", 19},
    {"ALL", 0},
    {0, 0}
};

static char     configLineHelp[] =
    "    Syntax: CL <net> <framing> <coding> <bldout> <robbed> <timing>\n\n"
    "    Configure T1/E1 line\n\n"
    "     where:\n"
    "            <net>      [A | B | C | D | E | F | G | H]\n"
    "            <framing>  [NONE | D4 | ESF | CCS | CAS | CCSCRC4 | CASCRC4]\n"
    "            <coding>   [NONE | NOZCS | B7S | B8ZS | HDB3]\n"
    "            <bldout>   0 - 4\n"
    "            <robbed>   [DIS | ENA]\n"
    "            <timing>   [SLAVE | MASTER]\n\n";

static char     lineStatusHelp[] =
    "    Syntax: LS <net> <flags>\n\n"
    "    T1/E1 status report\n\n"
    "     where:\n"
    "            <net>      [A | B | C | D | E | F | G | H]\n"
    "            <flags>    1 = reset\n\n";

static char     perfReportHelp[] =
    "    Syntax: PR <net> <flags>\n\n"
    "    T1/E1 performance report\n\n"
    "     where:\n"
    "            <net>      [A | B | C | D | E | F | G | H]\n"
    "            <flags>    1 = reset\n\n";

static char     lineCtrlHelp[] =
    "    Syntax: LC <net> <loopback> <alarm>\n\n"
    "    T1/E1 line control\n\n"
    "     where:\n"
    "            <net>      [A | B | C | D | E | F | G | H]\n"
    "            <loopback> [NONE | LOCAL | REM]\n"
    "            <alarm>    [NONE | YELLOW | BLUE | TS16AIS | DISTMF]\n\n";

static char     condChanHelp[] =
    "    Syntax: CD <net> <channel> <state> <pattern>\n\n"
    "    Condition channel\n\n" "     where:\n"
    "            <net>      [A | B | C | D | E | F | G | H]\n"
    "            <channel>  T1/E1 channel # (0 - 31)\n"
    "            <state>    [DIS | ENA]\n"
    "            <pattern>  0 - 255 or 0x00 - 0xFF\n\n";

static char     configChanHelp[] =
    "    Syntax: CC <net> <channel> <trans>\n\n"
    "    Configure T1/E1 channel\n\n"
    "     where:\n"
    "            <net>      [A | B | C | D | E | F | G | H]\n"
    "            <channel>  T1/E1 channel # (0 - 31)\n"
    "            <trans>    0 = NORMAL"
    "                       1 = TRANSPARENT\n\n";

static char     chanStatusHelp[] =
    "    Syntax: CS <net>\n\n"
    "    T1/E1 channel status\n\n"
    "     where:\n"
    "            <net>      [A | B | C | D | E | F | G | H]\n\n";

static char     supvCarrierHelp[] =
    "    Syntax: SV <net> <mask>\n\n"
    "    Supervise carrier\n\n"
    "     where:\n"
    "            <net>      [A | B | C | D | E | F | G | H]\n"
    "            <mask>     any combination of:"
    "                         1 = STATUS"
    "                         2 = ALARMS"
    "                         4 = 15 min REPORT"
    "                         8 = 24 hr  REPORT\n\n";

static char     quitHelp[] =
    "    Syntax: Q \n\n"
    "    Quit test application\n\n\n";

static char     setCPHelp[] =
    "    Syntax: CP <cpnum>\n\n"
    "    T1/E1 channel status\n\n"
    "     where:\n"
    "            <cpnum>    1 - 32\n\n";

static char     configClockHelp[] =
    "    Syntax: CK <clock> <sclock> <net>\n\n"
    "    Configure MVIP clock\n\n"
    "     where:\n"
    "            <clock>    [MVIP | SEC8K | OSC | NET]\n"
    "            <sclock>   [SLAVE | OSC | NET]\n"
    "            <net>      [A | B | C | D | E | F | G | H]\n\n";

static char     queryOutputHelp[] =
    "    Syntax: QO <stream> <timeslot>\n\n"
    "    Query output\n\n"
    "     where:\n"
    "            <stream>    0 - 31  (HBUS)\n"
    "                       40 - 41  (T1|E1-A..T1|E1-B)         ! TX 3220[C] ONLY !\n"
    "                            72  (COMMUNICATIONS PROCESSOR) PRIMARY INTERFACE\n"
    "                            74  (COMMUNICATIONS PROCESSOR) SECONDARY INTERFACE\n"
    "                       80 - 95  (80 = trunk 1; 81 = trunk 2; (board-specific max)\n\n"
    "            <timeslot>  0 - 127 (HBUS); 0 - 23 (T1); 0 - 31 (E1)\n\n";

static char     querySwitchCapsHelp[] =
    "    Syntax: QS \n\n"
    "    Query switch caps\n\n\n";

static char     resetSwitchHelp[] =
    "    Syntax: RS\n\n"
    "    Reset switch\n\n\n";

static char     sampleInputHelp[] =
    "    Syntax: SI <stream> <timeslot>\n\n"
    "    Sample input\n\n"
    "     where:\n"
    "            <stream>    0 - 31  (HBUS)\n"
    "                       40 - 41  (T1|E1-A..T1|E1-B)         ! TX 3220[C] ONLY !\n"
    "                            72  (COMMUNICATIONS PROCESSOR) PRIMARY INTERFACE\n"
    "                            74  (COMMUNICATIONS PROCESSOR) SECONDARY INTERFACE\n"
    "                       80 - 95  (80 = trunk 1; 81 = trunk 2; (board-specific max)\n\n"
    "            <timeslot>  0 - 127 (HBUS); 0 - 23 (T1); 0 - 31 (E1)\n\n";

static char     setOutputHelp[] =
    "    Syntax: SO <outstream> <outtimeslot> <mode>\n"
    "               <instream> <intimeslot> <pattern>\n\n"
    "    Set output\n\n"
    "     where:\n"
    "            <outstream>    0 - 31  (HBUS)\n"
    "                          40 - 41  (T1|E1-A..T1|E1-B)         ! TX 3220[C] ONLY !\n"
    "                               72  (COMMUNICATIONS PROCESSOR) PRIMARY INTERFACE\n"
    "                               74  (COMMUNICATIONS PROCESSOR) SECONDARY INTERFACE\n"
    "                          80 - 95  (80 = trunk 1; 81 = trunk 2; (board-specific max)\n\n"
    "            <outtimeslot>  0 - 127 (HBUS); 0 - 23 (T1); 0 - 31 (E1)\n\n"
    "            <mode>          [CONN | PAT | DIS]\n"
    "            <instream>     0 - 31  (HBUS)\n"
    "                          40 - 41  (T1|E1-A..T1|E1-B)         ! TX 3220[C] ONLY !\n"
    "                               72  (COMMUNICATIONS PROCESSOR) PRIMARY INTERFACE\n"
    "                               74  (COMMUNICATIONS PROCESSOR) SECONDARY INTERFACE\n"
    "                          80 - 95  (80 = trunk 1; 81 = trunk 2; (board-specific max)\n\n"
    "            <intimeslot>   0 - 127 (HBUS); 0 - 23 (T1); 0 - 31 (E1)\n\n"
    "            <pattern>      0 - 255 or 0x00 - 0xFF\n\n";

static char     setOutputFdxHelp[] =
    "    Syntax: SF <outstream> <outtimeslot> <mode>\n"
    "               <instream> <intimeslot> <pattern>\n\n"
    "    Set output full duplex\n\n"
    "     where:\n"
    "            <outstream>    0 - 31  (HBUS)\n"
    "                          40 - 41  (T1|E1-A..T1|E1-B)         ! TX 3220[C] ONLY !\n"
    "                               72  (COMMUNICATIONS PROCESSOR) PRIMARY INTERFACE\n"
    "                               74  (COMMUNICATIONS PROCESSOR) SECONDARY INTERFACE\n"
    "                          80 - 95  (80 = trunk 1; 81 = trunk 2; (board-specific max)\n\n"
    "            <outtimeslot>  0 - 127 (HBUS); 0 - 23 (T1); 0 - 31 (E1)\n\n"
    "            <mode>          [CONN | PAT | DIS]\n"
    "            <instream>     0 - 31  (HBUS)\n"
    "                          40 - 41  (T1|E1-A..T1|E1-B)         ! TX 3220[C] ONLY !\n"
    "                               72  (COMMUNICATIONS PROCESSOR) PRIMARY INTERFACE\n"
    "                               74  (COMMUNICATIONS PROCESSOR) SECONDARY INTERFACE\n"
    "                          80 - 95  (80 = trunk 1; 81 = trunk 2; (board-specific max)\n\n"
    "            <intimeslot>   0 - 127 (HBUS); 0 - 23 (T1); 0 - 31 (E1)\n\n"
    "            <pattern>      0 - 255 or 0x00 - 0xFF\n\n";

static char     switchStatusHelp[] =
    "    Syntax: SS \n\n"
    "    Get switch status\n\n\n";

static char     tristateSwitchHelp[] =
    "    Syntax: TR  <mode>\n\n"
    "    Tristate switch\n\n"
    "     where:\n"
    "            <mode>          [DIS | ENA]\n\n";

static char     allHelp[] =
    "    CL     Configure T1/E1 line\n"
    "    LS     T1/E1 status report\n"
    "    PR     T1/E1 performance report\n"
    "    LC     T1/E1 line control\n"
    "    CD     Condition channel\n"
    "    CC     Configure T1/E1 channel\n"
    "    CS     T1/E1 channel status\n"
    "    SV     Supervise carrier\n"
    "    Q      Quit test application\n"
    "    CK     Configure MVIP clock\n"
    "    QO     Query output\n"
    "    QS     Query switch caps\n"
    "    RS     Reset switch\n"
    "    SI     Sample input\n"
    "    SO     Set output\n"
    "    SF     Set output full duplex\n"
    "    SS     Query Switch Status\n"
    "    TR     Tristate switch\n\n";

static char    *helpTbl[] = {
    allHelp,
    configLineHelp,
    lineStatusHelp,
    perfReportHelp,
    lineCtrlHelp,
    condChanHelp,
    configChanHelp,
    chanStatusHelp,
    supvCarrierHelp,
    quitHelp,
    setCPHelp,
    configClockHelp,
    queryOutputHelp,
    querySwitchCapsHelp,
    resetSwitchHelp,
    sampleInputHelp,
    setOutputHelp,
    setOutputFdxHelp,
    switchStatusHelp,
    tristateSwitchHelp
};

/* structure for command parse tables */
#define MAX_PARMS    10

struct CmdSpec
{
    char           *command;
    struct lookupEnt *parmTbl[MAX_PARMS];
    void            (*func) (U32 *);
};

/* Help Command */
struct CmdSpec  helpCmd = {
    "?",                        /* command name */
    {CmdLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    HelpCommand                     /* handler function */
};

/* t1e1 API commands */
struct CmdSpec  configLineCmd = {
    "CL",                       /* command name */
    {NetLookupTbl, FramingLookupTbl, CodingLookupTbl, 0, DisEnaLookupTbl,
     TimingLookupTbl,
     0, 0, 0, 0},
    configCarrier                   /* handler function */
};

struct CmdSpec  lineStatusCmd = {
    "LS",                       /* command name */
    {NetLookupTbl, 0,
     0, 0, 0, 0, 0, 0, 0, 0},
    carrierStatus                   /* handler function */
};

struct CmdSpec  perfReportCmd = {
    "PR",                       /* command name */
    {NetLookupTbl, 0,
     0, 0, 0, 0, 0, 0, 0, 0},
    perfReport                      /* handler function */
};

struct CmdSpec  lineCtrlCmd = {
    "LC",                       /* command name */
    {NetLookupTbl, LoopbackLookupTbl, AlarmLookupTbl,
     0, 0, 0, 0, 0, 0, 0},
    ctrlCarrier                     /* handler function */
};

struct CmdSpec  condChanCmd = {
    "CD",                       /* command name */
    {NetLookupTbl, 0, DisEnaLookupTbl, 0,
     0, 0, 0, 0, 0, 0},
    conditionChan                   /* handler function */
};

struct CmdSpec  configChanCmd = {
    "CC",                       /* command name */
    {NetLookupTbl, 0, 0,
     0, 0, 0, 0, 0, 0, 0},
    configChan                      /* handler function */
};

struct CmdSpec  chanStatusCmd = {
    "CS",                       /* command name */
    {NetLookupTbl,
     0, 0, 0, 0, 0, 0, 0, 0, 0},
    channelStatus                   /* handler function */
};

struct CmdSpec  supvCarrierCmd = {
    "SV",                       /* command name */
    {NetLookupTbl, 0,
     0, 0, 0, 0, 0, 0, 0, 0},
    supvCarrier                     /* handler function */
};

struct CmdSpec  quitCmd = {
    "Q",                        /* command name */
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    quit                            /* handler function */
};

struct CmdSpec  setCpCmd = {
    "CP",                       /* command name */
    {0,
     0, 0, 0, 0, 0, 0, 0, 0, 0},
    setCP                           /* handler function */
};

/* mvip api commands */
struct CmdSpec  configClockCmd = {
    "CK",                       /* command name */
    {ClockLookupTbl, Sec8kLookupTbl, NetLookupTbl,
     0, 0, 0, 0, 0, 0, 0},
    configClock                     /* handler function */
};

struct CmdSpec  queryOutputCmd = {
    "QO",                       /* command name */
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    queryOutput                     /* handler function */
};

struct CmdSpec  querySwitchCapsCmd = {
    "QS",                       /* command name */
    {0, 0,
     0, 0, 0, 0, 0, 0, 0, 0},
    querySwCaps                     /* handler function */
};

struct CmdSpec  resetSwitchCmd = {
    "RS",                       /* command name */
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    resetSwitch                     /* handler function */
};

struct CmdSpec  sampleInputCmd = {
    "SI",                       /* command name */
    {0, 0,
     0, 0, 0, 0, 0, 0, 0, 0},
    sampleInput                     /* handler function */
};

struct CmdSpec  setOutputCmd = {
    "SO",                       /* command name */
    {0, 0, ModeLookupTbl, 0, 0, 0,
     0, 0, 0, 0},
    setOutput                       /* handler function */
};

struct CmdSpec  setOutputFdxCmd = {
    "SF",                       /* command name */
    {0, 0, ModeLookupTbl, 0, 0, 0,
     0, 0, 0, 0},
    setOutputFdx                    /* handler function */
};

struct CmdSpec  switchStatusCmd = {
    "SS",                       /* command name */
    {0, 0,
     0, 0, 0, 0, 0, 0, 0, 0},
    switchStatus                    /* handler function */
};

struct CmdSpec  tristateSwitchCmd = {
    "TR",                       /* command name */
    {TriLookupTbl,
     0, 0, 0, 0, 0, 0, 0, 0, 0},
    tristateSwitch                  /* handler function */
};



struct CmdSpec *CmdTable[] = {
    &configLineCmd,
    &lineStatusCmd,
    &perfReportCmd,
    &lineCtrlCmd,
    &condChanCmd,
    &configChanCmd,
    &chanStatusCmd,
    &supvCarrierCmd,
    &quitCmd,
    &setCpCmd,
    &configClockCmd,
    &queryOutputCmd,
    &querySwitchCapsCmd,
    &resetSwitchCmd,
    &sampleInputCmd,
    &setOutputCmd,
    &setOutputFdxCmd,
    &switchStatusCmd,
    &tristateSwitchCmd,
    &helpCmd,
    0
};


/* lookup table to map error numbers into messages */
typedef struct
{
    char           *msg;
    short           err_no;
}
MSGTBL;

/******************************************************************************
* MVIP and T1E1 Error Messages
******************************************************************************/
#define UNKNOWN_ERR            1000

MSGTBL          ErrMsgs[] = {
    {
     "INVALID HANDLE", -1},
    {
     "NO DRIVER", -2},
    {
     "OPEN FAIL", -3},
    {
     "BAD BOARD", -4},
    {
     "SEND ERROR", -5},
    {
     "NO RESPONSE", -6},
    {
     "RECV ERROR", -7},
    {
     "SUCCESS", 0},
    {
     "INVALID COMMAND", 200},
    {
     "DLL INVALID DEVICE", 201},
    {
     "DEVICE ERROR", 202},
    {
     "NO RESOURCES", 204},
    {
     "INVALID STREAM", 210},
    {
     "INVALID TIMESLOT", 211},
    {
     "MISSING PARAMETERS", 212},
    {
     "INVALID CLOCK PARM", 213},
    {
     "INVALID MODE", 216},
    {
     "INVALID MINOR SWITCH", 217},
    {
     "INVALID PARAMETER", 218},
    {
     "NO PATH", 220},
    {
     "SWITCH VERIFY ERROR", 221},
    {
     "INTERNAL CONFLICT", 222},
    {
     "CONNECTION NOT SUPPORTED", 223},
    {
     "INVALID LENGTH", 224},
    {
     "INVALID CARRIER", 225},
    {
     "INVALID FRAMING", 226},
    {
     "INVALID CODING", 227},
    {
     "INVALID BLDOUT", 228},
    {
     "INVALID LOOPBACK", 229},
    {
     "Unknown Error Number", UNKNOWN_ERR}
};

/* Misc. globals */
char            valStr[20];
TXBOARD_T       cp = 1;
short           t1handle = -1;
short           allT1Handles[MAXBOARD];
short           mviphandle = -1;
short           allMvipHandles[MAXBOARD];
short           quittinTime = 0;

/* getname
 *
 * Serches lookup table for a value match and returns a pointer to
 * the name that corresponds to that value.  Returns a pointer to a
 * global string containing ascii representation of that value if
 * no matching value found in table.
 */
char           *getname(struct lookupEnt *tbl, short val)
{
    while (tbl->name != 0)
    {                               /* while not end of table */
        if (tbl->value == val)
        {
            /* match found */
            return (tbl->name);
        }
        tbl++;
    }                               /* end while */

    /* falling through loop means search failed - convert number to ascii */
    sprintf(valStr, "%d", val);
    return (valStr);

}                                   /* end getname() */


/* getval
 *
 * Serches lookup table for a string match and generates the resulting
 * value if found.  Returns 1 if string found, 0 if not found.
 */
short getval(struct lookupEnt *tbl, char *s, U32 *val)
{
    while (tbl->name != 0)
    {                               /* while not end of table */
        if (strcmp(tbl->name, s) == 0)
        {
            /* match found */
            *val = (U32)tbl->value;
            return (1);
        }
        tbl++;
    }                               /* end while */

    /* falling through loop means search failed */
    return (0);
}                                   /* end getval() */

/* errMsg
 *
 * Function to lookup string for a particular error number and
 * return a pointer to the string.
 */
char           *errMsg(short err_no)
{

    short           i;

    /* search for error number match; note that UNKNOWN_ERR is always
     * * last entry in table so search will terminate at a valid slot
     * * even if errno not found in table.
     */
    i = 0;
    while (ErrMsgs[i].err_no != UNKNOWN_ERR && ErrMsgs[i].err_no != err_no)
        i++;
    return (ErrMsgs[i].msg);
}

void configCarrier(U32 *parms)
{
    short           status;
    TxT1E1Config    cfgblk;

    printf("TxT1E1ConfigCarrier( %d, %d, %d, %d, %d, %d, %d, %d )\n",
           t1handle, parms[0], parms[1], parms[2], parms[3],
           parms[4], parms[5], parms[6]);

    cfgblk.frameMode = (unsigned char)parms[1];
    cfgblk.coding = (unsigned char)parms[2];
    cfgblk.lineBldOut = (unsigned char)parms[3];
    cfgblk.debounce = 0;
    cfgblk.robbedBit = (unsigned char)parms[4];
    cfgblk.loopMaster = (unsigned char)parms[5];
    status = TxT1E1ConfigCarrier(t1handle, (unsigned char)parms[0], &cfgblk);

    printf("Status = %d [%s]\n", status, errMsg(status));
}

void carrierStatus(U32 *parms)
{
    short           status;
    TxT1E1Config    cfgblk;
    unsigned char   alarmState, lastAlmState, syncState, tempalarm;
    unsigned short  currTicks, currInts, pcvs, lcvs, slips, ebits;
    TxT1E1Stats     currStats, summStats;

    printf("TxT1E1CarrierStatus( %d, %d, %d )\n", t1handle, parms[0],
           parms[1]);

    status = TxT1E1CarrierStatus(t1handle, (unsigned char)parms[0],
                                 (unsigned char)parms[1],
                                 &alarmState, &lastAlmState, &syncState,
                                 &cfgblk, &currTicks, &currStats, &currInts,
                                 &summStats, &pcvs, &lcvs, &slips, &ebits);

    printf("Status = %d [%s]\n\n", status, errMsg(status));
    if (status == T1E1_SUCCESS)
    {
        printf("SyncState  = %s\n", getname(SyncLookupTbl, (short)syncState));
        printf("Framing    = %s\n",
               getname(FramingLookupTbl, (short)cfgblk.frameMode));
        printf("Coding     = %s\n",
               getname(CodingLookupTbl, (short)cfgblk.coding));
        printf("LoopTiming = %s\n",
               getname(TimingLookupTbl, (short)cfgblk.loopMaster));
        printf("LineBldOut = %d\n", cfgblk.lineBldOut);
        printf("Debounce   = %d\n", cfgblk.debounce);
        printf("RobbedBit  = %s\n\n",
               getname(DisEnaLookupTbl, (short)cfgblk.robbedBit));
        if (alarmState != NO_ALARM)
        {
            if (tempalarm = alarmState & BLUE_ALARM)
            {
                printf("AlarmState = %s\n",
                       getname(AlarmLookupTbl, (short)tempalarm));
            }
            else
            {
                if (tempalarm = alarmState & YELLOW_ALARM)
                    printf("AlarmState = %s\n",
                           getname(AlarmLookupTbl, (short)tempalarm));
                if (tempalarm = alarmState & TS16AIS_ALARM)
                    printf("AlarmState = %s\n",
                           getname(AlarmLookupTbl, (short)tempalarm));
                if (tempalarm = alarmState & TS16SIG_ALARM)
                    printf("AlarmState = %s\n",
                           getname(AlarmLookupTbl, (short)tempalarm));
                if (tempalarm = alarmState & DISTANTMF_ALARM)
                    printf("AlarmState = %s\n",
                           getname(AlarmLookupTbl, (short)tempalarm));
            }
        }
        printf
            ("\nStats:    ES      LES     BES     SES     UAS     SLS     LOFC\n");
        printf
            ("------    --      ---     ---     ---     ---     ---     ----\n");
        printf("Current:  %-8u%-8u%-8u%-8u%-8u%-8u%-8u\n", currStats.errSecs,
               currStats.lineErrSecs, currStats.burstyErrSecs,
               currStats.severlyErrSecs, currStats.unaSecs,
               currStats.slipSecs, currStats.lofCnt);
        printf("Summary:  %-8u%-8u%-8u%-8u%-8u%-8u%-8u\n", summStats.errSecs,
               summStats.lineErrSecs, summStats.burstyErrSecs,
               summStats.severlyErrSecs, summStats.unaSecs,
               summStats.slipSecs, summStats.lofCnt);
        printf("\nCurrent Interval: %d secs, Valid Intervals: %d\n",
               currTicks, currInts);
        printf("PCVs: %u,  LCVs: %u,  Slips: %u  E-Bits: %u\n", pcvs, lcvs,
               slips, ebits);
    }
}

void perfReport(U32 *parms)
{
    short           status, i;
    unsigned short  currTicks, currInts, pcvs, lcvs, slips, ebits;
    TxT1E1Stats     currStats, summStats, intStats[96];
    char            junk[80];

    printf("TxT1E1PerfReport( %d, %d, %d )\n", t1handle, parms[0], parms[1]);

    status = TxT1E1PerfReport(t1handle, (unsigned char)parms[0],
                              (unsigned char)parms[1],
                              &currTicks, &currStats,
                              &currInts, &intStats[0], &summStats,
                              &pcvs, &lcvs, &slips, &ebits);

    printf("Status = %d [%s]\n", status, errMsg(status));
    if (status == T1E1_SUCCESS)
    {
        printf("Current Interval: %d secs, Valid Intervals: %d\n",
               currTicks, currInts);
        printf("PCVs: %u,  LCVs: %u,  Slips: %u,  E-Bits: %u\n\n", pcvs, lcvs,
               slips, ebits);
        printf
            ("Interval  ES      LES     BES     SES     UAS     SLS     LOFC\n");
        printf
            ("--------  --      ---     ---     ---     ---     ---     ----\n");
        printf("Current:  %-8u%-8u%-8u%-8u%-8u%-8u%-8u\n", currStats.errSecs,
               currStats.lineErrSecs, currStats.burstyErrSecs,
               currStats.severlyErrSecs, currStats.unaSecs,
               currStats.slipSecs, currStats.lofCnt);
        for (i = 0; i < currInts; i++)
        {
            printf("%8u  %-8u%-8u%-8u%-8u%-8u%-8u%-8u\n", i,
                   intStats[i].errSecs, intStats[i].lineErrSecs,
                   intStats[i].burstyErrSecs, intStats[i].severlyErrSecs,
                   intStats[i].unaSecs, intStats[i].slipSecs,
                   intStats[i].lofCnt);
            if (((i % 16) == 0) && (i < (currInts - 1)))
            {
                /* pause to let user read */
                printf("--more--");
                fgets(junk, 80, stdin);
                printf("\n");
            }
        }
        printf("Summary:  %-8u%-8u%-8u%-8u%-8u%-8u%-8u\n", summStats.errSecs,
               summStats.lineErrSecs, summStats.burstyErrSecs,
               summStats.severlyErrSecs, summStats.unaSecs,
               summStats.slipSecs, summStats.lofCnt);
    }
}

void ctrlCarrier(U32 *parms)
{
    short           status;

    printf("TxT1E1CtrlCarrier( %d, %d, %d, %d )\n",
           t1handle, parms[0], parms[1], parms[2]);

    status = TxT1E1CtrlCarrier(t1handle, (unsigned char)parms[0],
                               (unsigned char)parms[1],
                               (unsigned char)parms[2]);

    printf("Status = %d [%s]\n", status, errMsg(status));
}

void conditionChan(U32 *parms)
{
    short           status;

    printf("TxT1E1ConditionChan( %d, %d, %d, %d, %d )\n",
           t1handle, parms[0], parms[1], parms[2], parms[3]);

    status = TxT1E1ConditionChan(t1handle, (unsigned char)parms[0],
                                 (unsigned char)parms[1],
                                 (unsigned char)parms[2],
                                 (unsigned char)parms[3]);

    printf("Status = %d [%s]\n", status, errMsg(status));
}

void configChan(U32 *parms)
{
    short           status;

    printf("TxT1E1ConfigChan( %d, %d, %d, %d )\n",
           t1handle, parms[0], parms[1], parms[2]);

    status = TxT1E1ConfigChan(t1handle, (unsigned char)parms[0],
                              (unsigned char)parms[1],
                              (unsigned char)parms[2]);

    printf("Status = %d [%s]\n", status, errMsg(status));
}

void channelStatus(U32 *parms)
{
    short           status, i;
    U32             trans, rxABCD[4], txABCD[4];
    char            junk[80];

    printf("TxT1E1ChannelStatus( %d, %d )\n", t1handle, parms[0]);

    status = TxT1E1ChannelStatus(t1handle, (unsigned char)parms[0],
                                 &trans, &rxABCD[0], &txABCD[0]);

    printf("Status = %d [%s]\n", status, errMsg(status));
    if (status == T1E1_SUCCESS)
    {
        printf("           Rx         Tx\n");
        printf("Chan  T  A B C D    A B C D\n");
        printf("----  -  - - - -    - - - -\n");
        for (i = 0; i < 32; i++)
        {
            printf("%4u  %-1u  %-1u %-1u %-1u %-1u    %-1u %-1u %-1u %-1u\n",
                   i, BITVAL(trans, i),
                   BITVAL(rxABCD[0], i), BITVAL(rxABCD[1], i),
                   BITVAL(rxABCD[2], i), BITVAL(rxABCD[3], i),
                   BITVAL(txABCD[0], i), BITVAL(txABCD[1], i),
                   BITVAL(txABCD[2], i), BITVAL(txABCD[3], i));
            if (i == 15)
            {
                /* pause to let user read */
                printf("--more--");
                fgets(junk, 80, stdin);
                printf("\n");
            }
        }                           /* end for loop */
    }
}

void supvCarrier(U32 *parms)
{
    short           status;

    printf("TxT1E1SuperviseCarrier( %d, %d, %d )\n",
           t1handle, parms[0], parms[1]);

    status = TxT1E1SuperviseCarrier(t1handle, (unsigned char)parms[0],
                                    (unsigned char)parms[1]);

    printf("Status = %d [%s]\n", status, errMsg(status));
}

/* configClock */
void configClock(U32 *parms)
{
    short           status;

    printf("TxConfigClock( %d, %d, %d, %d )\n",
           mviphandle, parms[0], parms[1], parms[2]);

    status = TxConfigClock(mviphandle, (CLOCK_T) parms[0],
                           (SEC8K_T) parms[1], (unsigned char)parms[2]);

    printf("Status = %d [%s]\n", status, errMsg(status));
}

/* queryOutput */
void queryOutput(U32 *parms)
{
    short           status;
    MODE_T          mode;
    STREAM_T        iStream;
    TIMESLOT_T      iTimeslot;
    DATA_T          message;

    printf("TxQueryOutput( %d, %d, %d, ... )\n", mviphandle, parms[0],
           parms[1]);

    status = TxQueryOutput(mviphandle, (STREAM_T) parms[0],
                           (TIMESLOT_T) parms[1],
                           &mode, &iStream, &iTimeslot, &message);

    printf("Status = %d [%s]\n", status, errMsg(status));
    if (status == SUCCESS)
    {
        printf("mode       = %s\n", getname(ModeLookupTbl, (short)mode));
        printf("inStream   = %d\n", iStream);
        printf("inTimeslot = %d\n", iTimeslot);
        printf("message    = 0x%X\n", message);
    }
}

/* querySwCaps */
void querySwCaps(U32 *parms)
{
    short           status;
    int             revision;
    unsigned short  domain;
    unsigned short  routing;
    unsigned short  blocking;
    unsigned short  networks;
    unsigned short  channels[2];
    short           i;

    printf("TxQuerySwitchCaps( %d, ... )\n", mviphandle);

    status = TxQuerySwitchCaps(mviphandle, 0, &revision, &domain, &routing,
                               &blocking, &networks, channels);

    printf("Status = %d [%s]\n", status, errMsg(status));
    if (status == SUCCESS)
    {
        printf("revision   = %d\n", revision);
        printf("domain     = 0x%X\n", domain);
        printf("routing    = 0x%X\n", routing);
        printf("blocking   = 0x%X\n", blocking);
        printf("networks   = %d\n", networks);
        for (i = 0; i < networks; i++)
        {
            printf("channels[%d] = %d\n", i, channels[i]);
        }
    }

}

/* resetSwitch */
void resetSwitch(U32 *parms)
{
    short           status;

    printf("TxResetSwitch( %d )\n", mviphandle);
    status = TxResetSwitch(mviphandle);
    printf("Status = %d [%s]\n", status, errMsg(status));
}

/* sampleInput */
void sampleInput(U32 *parms)
{
    short           status;
    DATA_T          message;

    printf("TxSampleInput( %d, %d, %d, ... )\n", mviphandle, parms[0],
           parms[1]);

    status = TxSampleInput(mviphandle, (STREAM_T) parms[0],
                           (TIMESLOT_T) parms[1], &message);

    printf("Status = %d [%s]\n", status, errMsg(status));
    if (status == SUCCESS)
    {
        printf("sample     = 0x%X\n", message);
    }
}

/* setOutput */
void setOutput(U32 *parms)
{
    short           status;

    printf("TxSetOutput( %d, %d, %d, %d, %d, %d, %d )\n", mviphandle,
           parms[0], parms[1], parms[2], parms[3], parms[4], parms[5]);

    status =
        TxSetOutput(mviphandle, (STREAM_T) parms[0], (TIMESLOT_T) parms[1],
                    (MODE_T) parms[2], (STREAM_T) parms[3],
                    (TIMESLOT_T) parms[4], (DATA_T) parms[5]);

    printf("Status = %d [%s]\n", status, errMsg(status));
}

/* setOutputFdx */
void setOutputFdx(U32 *parms)
{
    short           status;

    printf("TxSetOutputFdx( %d, %d, %d, %d, %d, %d, %d )\n", mviphandle,
           parms[0], parms[1], parms[2], parms[3], parms[4], parms[5]);

    status = TxSetOutputFdx(mviphandle, (STREAM_T) parms[0],
                            (TIMESLOT_T) parms[1], (MODE_T) parms[2],
                            (STREAM_T) parms[3], (TIMESLOT_T) parms[4],
                            (DATA_T) parms[5]);

    printf("Status = %d [%s]\n", status, errMsg(status));
}

/* switchStatus */
void switchStatus(U32 *parms)
{
    short           status;
    CLOCK_T         clock;
    SEC8K_T         sec8k;
    NETWORK_T       network;
    TRISTATE_T      state;

    printf("TxSwitchStatus( %d, ... )\n", mviphandle);

    status = TxSwitchStatus(mviphandle, &clock, &sec8k, &network, &state);

    printf("Status = %d [%s]\n", status, errMsg(status));
    if (status == SUCCESS)
    {
        printf("clock      = %s\n", getname(ClockLookupTbl, (short)clock));
        printf("sec8k      = %s\n", getname(Sec8kLookupTbl, (short)sec8k));
        printf("network    = 0x%X\n", network);
        printf("state      = %s\n", getname(TriLookupTbl, (short)state));
    }

}

/* tristateSwitch */
void tristateSwitch(U32 *parms)
{
    short           status;

    printf("TxTristateSwitch( %d, %d )\n", mviphandle, parms[0]);
    status = TxTristateSwitch(mviphandle, (TRISTATE_T) parms[0]);
    printf("Status = %d [%s]\n", status, errMsg(status));
}

void HelpCommand(U32 *parms)
{
    printf("%s", helpTbl[*parms]);
}

void setCP(U32 *parms)
{
    TXBOARD_T       newcp;

    newcp = (unsigned char)parms[0];

    if (newcp < 1 || newcp > MAXBOARD)
    {
        printf("CP must be in range 1..%d\n", MAXBOARD);
    }

    /* if not open already, open it */
    if (allMvipHandles[newcp - 1] < 0)
    {
        allMvipHandles[newcp - 1] = TxMvipOpen(newcp, 0);
        if (allMvipHandles[newcp - 1] < 0)
        {
            printf("TxMvipOpen failed [%d]\n", allMvipHandles[newcp - 1]);
            allMvipHandles[newcp - 1] = -1;
            return;
        }
    }

    if (allT1Handles[newcp - 1] < 0)
    {
        allT1Handles[newcp - 1] = TxT1E1Open(newcp, 0);
        if (allT1Handles[newcp - 1] < 0)
        {
            printf("TxT1E1Open failed [%d]\n", allT1Handles[newcp - 1]);
            allT1Handles[newcp - 1] = -1;
        }
    }

    /* set new cp as the current cp */
    cp = newcp;
    t1handle = allT1Handles[cp - 1];
    mviphandle = allMvipHandles[cp - 1];

}

void quit(U32 *parms)
{
    quittinTime = 1;
}


/* processCmd
 *
 * Searches command table for command match, converts parameters to
 * array of shorts, and invokes the appropriate command handler.
 */
void processCmd(char *tokens[], short cnt)
{
    short           i, j;
    short           found;
    U32             params[MAX_PARMS];  /* numeric parameters array */
    struct lookupEnt *lp;
    unsigned int    scan_uint;

    /* first search command table for command match */
    for (i = 0; CmdTable[i] != 0; i++)
    {
        if (strcmp(tokens[0], CmdTable[i]->command) == 0)
        {
            break;                  /* command matched */
        }
    }                               /* end for */

    /* if no match found display error message and return */
    if (CmdTable[i] == 0)
    {
        printf("Unrecognized command: %s\n", tokens[0]);
        return;
    }

    /* convert remaining tokens (i.e., parameters) to short integers,
     * using lookup tables  where specified to convert text strings
     * (names) into numbers.  Note that all parameters are positional.
     */
    for (j = 0; j < MAX_PARMS; j++)
        params[j] = 0;              /* first initialize to defaults */

    for (j = 1; j < cnt; j++, lp++)
    {
        lp = CmdTable[i]->parmTbl[j - 1];
        if (lp)
        {                           /* lookup table for this param ? */
            found = getval(lp, tokens[j], &params[j - 1]);
            if (found)
            {
                continue;           /* parameter succesfully converted */
            }
        }

        /* no lookup table or lookup failed - try to convert as a number */
        if (sscanf(tokens[j], "%i", &scan_uint) != 1)
        {
            /* conversion failed */
            printf("Unrecognized parameter: %s\n", tokens[j]);
            return;
        }
        params[j - 1] = scan_uint;
    }                               /* end for */

    /* call command handler function */

    if (CmdTable[i]->func != 0)
    {
        CmdTable[i]->func(params);
    }
}

/* tokenize
 *
 * Takes input line and creates an array of pointers to individual
 * tokens in the input line.  First separator after each token in
 * input line is replaced with a null so that each token becomes
 * a null terminated string.
 */
short tokenize(char *buf, char *tokens[])
{
    char           *p = buf;
    char           *endp;
    short           cnt;

    /* initialize token pointers to all null */
    for (cnt = 0; cnt <= MAX_PARMS; cnt++)
        tokens[cnt] = 0;

    /* begin scanning for tokens */
    cnt = 0;
    while (*p != 0 && cnt <= MAX_PARMS)
    {
        /* skip leading space or tabs */
        while (*p == ' ' || *p == '\t')
            p++;

        /* beginning of token (or end of line) */
        if (*p != 0)
        {
            /* scan for end of token, converting to upper case as we go */
            endp = p;
            while (*endp != ' ' && *endp != '\t' && *endp != 0)
            {
                if (islower(*endp))
                {
                    *endp = 'A' + (*endp - 'a');    /* convert to upper */
                }
                endp++;
            }

            /* now have a begin & end of token */
            tokens[cnt++] = p;
            if (*endp == 0)
            {
                p = endp;           /* end of input - terminate scan */
            }
            else
            {
                *endp = 0;          /* null terminate token */
                p = &endp[1];       /* start next scan at after null */
            }
        }                           /* end if *p != 0 */
    }                               /* end while *p != 0 .... */

    return (cnt);
}

void usage(char *myname)
{
    printf(" \n");
    printf(usagestr, myname);
    printf(" \n");
    printf("where                                               [DEFAULT]\n");
    printf
        ("  -b : sets TX board number to connect to           [board #1]\n");

    exit(1);
}

int main(int argc, char *argv[])
{

    char            buffer[132];
    char           *tokens[MAX_PARMS + 1];
    short           cnt;
    short           i;
    char           *p;
    char           *myname;

    myname = argv[0];

    printf(" \n");
    printf("%s - T1/E1 Demonstration program V%d.%d\n", myname, MH_VERSION, MH_REVISION);
    printf("(c)Copyright 2000-2009 Dialogic Corporation.  All Rights Reserved.\n");
    printf(" \n");

    /* initialize array of board handles */
    for (i = 0; i < MAXBOARD; i++)
    {
        allMvipHandles[i] = -1;
        allT1Handles[i] = -1;
    }

    /* parse command line */
    i = 1;
    while (i < argc)
    {
        p = argv[i];
        if (*p == '-')
        {
            p++;
            switch (*p)
            {
                case 'b':          /* Set Board number */
                case 'B':
                    p++;
                    if (isdigit(*p))
                        cp = (TXBOARD_T)atoi(p);
                    else
                    {
                        i++;
                        if ( (i < argc) && (isdigit(*argv[i])) )
                            cp = (TXBOARD_T)atoi(argv[i]);
                        else
                        {
                            printf("\nError: expecting board number after -b option.\n");
                            exit(1);
                        }
                    }
                    if ( (cp < 1) || (cp > MAXBOARD) )
                    {
                        printf("\nError: illegal board number specified.\n");
                        usage(myname);
                    }
                    break;

                default:
                    usage(myname);
                    break;
            }                       /* end switch */
        }
        else
        {
            usage(myname);
        }
        ++i;
    }   /* end while */

    /* open the TxMvip API */
    allMvipHandles[cp - 1] = TxMvipOpen(cp, 0);
    if (allMvipHandles[cp - 1] < 0)
    {
        printf("TxMvipOpen failed [%d]\n", allMvipHandles[cp - 1]);
        exit(1);
    }
    mviphandle = allMvipHandles[cp - 1];

    /* open the TxT1E1 API for board 1 by default */
    allT1Handles[cp - 1] = TxT1E1Open(cp, 0);
    if (allT1Handles[cp - 1] < 0)
    {
        printf("TxT1E1Open failed [%d]\n", allT1Handles[cp - 1]);
    }
    t1handle = allT1Handles[cp - 1];

    /* command processing loop */
    printf("CP %-1u>", cp);
    while ((quittinTime == 0) && fgets(buffer, 132, stdin) != 0)
    {
        while ( (strlen(buffer) > 0) && (buffer[strlen(buffer) - 1] < ' ') )
            buffer[strlen(buffer) - 1] = '\0';  /* remove any trailing non-printable characters */
        cnt = tokenize(buffer, tokens);
        if (cnt > 0)
            processCmd(tokens, cnt);
        printf("CP %-1u>", cp);     /* display prompt */
    }                               /* end while */

    for (i = 0; i < MAXBOARD; i++)
    {
        if (allT1Handles[i] >= 0)
        {
            TxT1E1Close(allT1Handles[i]);
        }
        if (allMvipHandles[i] >= 0)
        {
            TxMvipClose(allMvipHandles[i]);
        }
    }

    return( 0 );
}                                   /* end main() */
