
/******************************************************************
 **
 **                     PROPRIETARY NOTICE
 **
 **  (c)Copyright 2006-2009 Dialogic Corporation. All Rights Reserved.
 **
 **
 **  The following is licensed program material and is considered
 **  proprietary and confidential.  No part of this program may be
 **  reproduced or used in any form or by any means without the
 **  express written permission of Dialogic Corporation
 **
 *****************************************************************/

/* TX T1E1 status capturing daemon */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "txcpi.h"
#include "tx_core.h"
#include "t1e1type.h"
#include "t1e1api.h"                /* need access to T1E1 API to perform auto-registration */
#include "dpruser.h"

/* ----- Modification History: --------------------------------------------- */
#define MH_VERSION      4           /* identifier of major release version */
#define MH_REVISION     0           /* identifier of release revision */

extern CPI_WAIT_TYPE stdInput;

#define MAX_TRUNKS_SHOWN    8       /* maximum number of trunks per TX board */

typedef struct __last_status
{
    char            t1e1[MAX_TRUNKS_SHOWN + 1];  /* last reported status of T1 or E1 interfaces */
#define T1E1_NEVER    '-'       /* never received STATUS_CARRIER message */
#define T1E1_LOST     'L'       /* last received: CARRIER_LOST */
#define T1E1_NOSYNC   'N'       /* last received: CARRIER_NOSYNC */
#define T1E1_SYNC     'S'       /* last received: CARRIER_SYNC */
#define T1E1_NOTEXIST 'x'       /* carrier assumed to not exist (failed to register) */
#define T1E1_UNKNOWN  '?'       /* last received: unknown state */

}
LAST_STATUS;

#define T1E1_ALL_NOTIFS (T1E1_REG_STATUS|T1E1_REG_ALARMS|T1E1_REG_15MIN_REPORT|T1E1_REG_24HR_REPORT)

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 * GLOBALS
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

unsigned char   cmd_flags = 0;  /* command-line controlled flags */

#define AUTO_REGISTER   (1<<0)      /* register for T1/E1 monitoring (all CPs,full monitoring) */
#define STATE_BANNER    (1<<1)      /* show last reported state banner */
#define QUIET_STATUS    (1<<2)      /* quiet line status messages (info also shown in banner) */

LAST_STATUS     state[MAXBOARD + 1];    /* last reported state information for each CP */

/* ----- BANNER handling ----- */
char            banner_hdr [80+2];
char            banner_info[80+2];
short           bpos[MAXBOARD + 1][MAX_TRUNKS_SHOWN + 1];  /* position of status character in banner line */

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 * FUNCTIONS
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/

/****************************************************************************
 * show_usage - display command line usage
 ****************************************************************************/
void show_usage(char *myname)
{
    printf("\nUsage> %s <-c chan> -r -s -q\n", myname);
    printf("  where:\n");
    printf("    -c = specify an alternate host-side CPI channel\n");
    printf("    -r = register for T1/E1 monitoring (all CPs,full monitoring)\n");
    printf("    -s = show last reported state banner after each standard message\n");
    printf("    -q = quiet line status messages (info also shown in banner)\n");
}

/****************************************************************************
 * T1E1 status change message
 ****************************************************************************/
typedef struct _t1e1msg
{
    CPIHDR          hdr;
    union
    {
        TxT1E1UHdr      uhdr;   /* status code & carrier (A or B) */
        TxT1E1UStat     status; /* alarm or state change */
        TxT1E1Perf15    perf15; /* 15-min. performance report */
        TxT1E1Perf24    perf24; /* 24-hr. performance report */
    }
    body;
}
T1E1MSG;

/****************************************************************************
 * procesMsg
 *
 * this function handles incoming status/performance messages by
 * printing to standard output.
 ****************************************************************************/
void processMsg(T1E1MSG * msg, unsigned char cp)
{
    short           carrier;
    short           i;
    TxT1E1Stats    *currStats;  /* current interval statistics */
    char           *pTime;      /* timestamp (ascii) */
    time_t          now;        /* current time */
    char            line_hdr[64];   /* info placed at beginning of each line */

    /* first get current time and get rid of "year\n" from end */
    now = time(NULL);
    pTime = ctime(&now);
    pTime[19] = '\0';

    /* decide what information is included in line header */
    carrier = (short)msg->body.uhdr.carrier;
    sprintf(line_hdr, "[CP #%d - trunk %d] %s",
            (unsigned short)cp, carrier, pTime);

    msg->body.uhdr.statusCode = cpi_cptoh_s(msg->body.uhdr.statusCode);
    switch (msg->body.uhdr.statusCode)
    {

        case CARRIER_ALARM:
            if ((msg->body.status.alarmState & BLUE_ALARM) &&
                !(msg->body.status.lastAlmState & BLUE_ALARM))
            {
                printf( "%s: Enter Blue Alarm State\n",
                        line_hdr );
            }
            if ((msg->body.status.alarmState & YELLOW_ALARM) &&
                !(msg->body.status.lastAlmState & YELLOW_ALARM))
            {
                printf( "%s: Enter Yellow Alarm State\n",
                        line_hdr );
            }
            if ((msg->body.status.alarmState & TS16AIS_ALARM) &&
                !(msg->body.status.lastAlmState & TS16AIS_ALARM))
            {
                printf( "%s: Enter TS16 AIS Alarm State\n",
                        line_hdr );
            }
            if ((msg->body.status.alarmState & TS16SIG_ALARM) &&
                !(msg->body.status.lastAlmState & TS16SIG_ALARM))
            {
                printf( "%s: Enter TS16 Signal Alarm State\n",
                        line_hdr );
            }
            if ((msg->body.status.alarmState & DISTANTMF_ALARM) &&
                !(msg->body.status.lastAlmState & DISTANTMF_ALARM))
            {
                printf( "%s: Enter Distant MF Alarm State\n",
                        line_hdr );
            }

            if ((msg->body.status.lastAlmState & BLUE_ALARM) &&
                !(msg->body.status.alarmState & BLUE_ALARM))
            {
                printf( "%s: Clear Blue Alarm State\n",
                        line_hdr );
            }
            if ((msg->body.status.lastAlmState & YELLOW_ALARM) &&
                !(msg->body.status.alarmState & YELLOW_ALARM))
            {
                printf( "%s: Clear Yellow Alarm State\n",
                        line_hdr );
            }
            if ((msg->body.status.lastAlmState & TS16AIS_ALARM) &&
                !(msg->body.status.alarmState & TS16AIS_ALARM))
            {
                printf( "%s: Clear TS16 AIS Alarm State\n",
                        line_hdr );
            }
            if ((msg->body.status.lastAlmState & TS16SIG_ALARM) &&
                !(msg->body.status.alarmState & TS16SIG_ALARM))
            {
                printf( "%s: Clear TS16 Signal Alarm State\n",
                        line_hdr );
            }
            if ((msg->body.status.lastAlmState & DISTANTMF_ALARM) &&
                !(msg->body.status.alarmState & DISTANTMF_ALARM))
            {
                printf( "%s: Clear Distant MF Alarm State\n",
                        line_hdr );
            }
            break;

        case STATUS_CARRIER:
            if (msg->body.status.syncState == CARRIER_LOST)
            {
                if (!(cmd_flags & QUIET_STATUS))
                    printf( "%s: Carrier Lost\n",
                            line_hdr );
                if (carrier <= MAX_TRUNKS_SHOWN)
                    state[cp].t1e1[carrier] = T1E1_LOST;
            }
            else if (msg->body.status.syncState == CARRIER_NOSYNC)
            {
                if (!(cmd_flags & QUIET_STATUS))
                    printf( "%s: Synchronization Lost\n",
                            line_hdr );
                if (carrier <= MAX_TRUNKS_SHOWN)
                    state[cp].t1e1[carrier] = T1E1_NOSYNC;
            }
            else if (msg->body.status.syncState == CARRIER_SYNC)
            {
                if (!(cmd_flags & QUIET_STATUS))
                    printf( "%s: Synchronized\n",
                            line_hdr );
                if (carrier <= MAX_TRUNKS_SHOWN)
                    state[cp].t1e1[carrier] = T1E1_SYNC;
            }
            else
            {
                if (!(cmd_flags & QUIET_STATUS))
                    printf( "%s: Status %d\n",
                            line_hdr, msg->body.status.syncState );
                if (carrier <= MAX_TRUNKS_SHOWN)
                    state[cp].t1e1[carrier] = T1E1_UNKNOWN;
            }

            if (carrier <= MAX_TRUNKS_SHOWN)
            {
                if (bpos[cp][carrier] != 0)
                    banner_info[bpos[cp][carrier]] = state[cp].t1e1[carrier];
            }
            break;

        case PERFREP_15MIN:
            currStats = &msg->body.perf15.currStats;
            printf( "%s: 15 Minute Performance Report:\n",
                    line_hdr );
            printf
                ("\nStats:    ES      LES     BES     SES     UAS     SLS     LOFC\n");
            printf
                ("------    --      ---     ---     ---     ---     ---     ----\n");
            printf("Current:  %-8u%-8u%-8u%-8u%-8u%-8u%-8u\n",
                   cpi_cptoh_s(currStats->errSecs),
                   cpi_cptoh_s(currStats->lineErrSecs),
                   cpi_cptoh_s(currStats->burstyErrSecs),
                   cpi_cptoh_s(currStats->severlyErrSecs),
                   cpi_cptoh_s(currStats->unaSecs),
                   (unsigned short)currStats->slipSecs,
                   (unsigned short)currStats->lofCnt);
            printf("\nPCVs: %u,  LCVs: %u,  Slips: %u,   E-Bits: %u\n\n",
                   cpi_cptoh_s(msg->body.perf15.pcvs),
                   cpi_cptoh_s(msg->body.perf15.lcvs),
                   cpi_cptoh_s(msg->body.perf15.slips),
                   cpi_cptoh_s(msg->body.perf15.ebits));
            break;

        case PERFREP_24HR:
            printf
                ( "\n%s: 24 Hour Performance: PCVs: %u,  LCVs: %u,  Slips: %u,  E-Bits: %u\n\n",
                  line_hdr, cpi_cptoh_s(msg->body.perf24.pcvs ),
                 cpi_cptoh_s(msg->body.perf24.lcvs),
                 cpi_cptoh_s(msg->body.perf24.slips),
                 cpi_cptoh_s(msg->body.perf24.ebits));
            printf
                ("Interval  ES      LES     BES     SES     UAS     SLS     LOFC\n");
            printf
                ("--------  --      ---     ---     ---     ---     ---     ----\n");
            for (i = 0; i < cpi_cptoh_s(msg->body.perf24.validInts); i++)
            {
                currStats = &msg->body.perf24.intStats[i];
                printf("[%-2u]:     %-8u%-8u%-8u%-8u%-8u%-8u%-8u\n",
                       i,
                       cpi_cptoh_s(currStats->errSecs),
                       cpi_cptoh_s(currStats->lineErrSecs),
                       cpi_cptoh_s(currStats->burstyErrSecs),
                       cpi_cptoh_s(currStats->severlyErrSecs),
                       cpi_cptoh_s(currStats->unaSecs),
                       (unsigned short)currStats->slipSecs,
                       (unsigned short)currStats->lofCnt);
            }
            currStats = &msg->body.perf24.sumStats;
            printf("Summary:  %-8u%-8u%-8u%-8u%-8u%-8u%-8u\n",
                   cpi_cptoh_s(currStats->errSecs),
                   cpi_cptoh_s(currStats->lineErrSecs),
                   cpi_cptoh_s(currStats->burstyErrSecs),
                   cpi_cptoh_s(currStats->severlyErrSecs),
                   cpi_cptoh_s(currStats->unaSecs),
                   (unsigned short)currStats->slipSecs,
                   (unsigned short)currStats->lofCnt);
            break;

        default:
            printf( "%s: Unrecognized Message received: 0x%x\n",
                    line_hdr, msg->body.uhdr.statusCode );
    }                               /* end switch */

    if (cmd_flags & STATE_BANNER)
    {                               /* show last reported state banner */
        printf("%s\n", banner_hdr);
        printf("%s\n", banner_info);
    }

}

/****************************************************************************
 * ExitRoutine - Decide whether to exit the tx_core polling loop
 ****************************************************************************/
short ExitRoutine(TX_HANDLE handle, short events)
{
    /* check if input event really did occur */
    if (tx_core_process_events(TX_CORE_INPUT, events) == FALSE)
        return (TX_CORE_SUCCESS);

    /* make sure a key was hit */
    if (tx_core_kbhit())
        return (TX_CORE_EXIT_POLL_LOOP);

    return (TX_CORE_SUCCESS);
}

/****************************************************************************
 * GetMsg - get next message from the TX board
 ****************************************************************************/
short GetMsg(TX_HANDLE handle, short events)
{
    CPIPKT          inbuf;
    short           ret, len = CPIBUFSIZE;
    unsigned char   ch, cp;

    /* check if input event really did occur */
    if (tx_core_process_events(TX_CORE_INPUT, events) == FALSE)
        return (TX_CORE_SUCCESS);

    if ((ret = cpi_get_data(handle, &inbuf, &len)) == CPI_ERROR)
    {
        printf("Receive Error (%d)\n", cpi_get_last_error());
        return (TX_CORE_ERROR);
    }
    else
    {
        cpi_get_board(handle, &cp, &ch);
        processMsg((T1E1MSG *) & inbuf, cp);
        return (TX_CORE_SUCCESS);
    }
}

/****************************************************************************
 * Main function
 ****************************************************************************/
int main(int argc, char *argv[])
{
    S32             cps[MAXBOARD + 1];
    TX_HANDLE       fds[MAXBOARD + 1];
    short           i, j;
    short           chan = PT_T1E1S;
    char           *myname;
    char           *str = NULL;
    char            sbuf[CPSTRSIZE];
    char           *vstr = sbuf;    /* dummy for old cpi_init */
    short           port;
    short           t1handle, status;
    short           some_on_cp;
    char            workbuf[80];
    short           bpos_next;

    printf("\nT1/E1 Monitor V%d.%d : Hit Return to exit\n", MH_VERSION, MH_REVISION);
    printf("(c)Copyright 1996-2009 Dialogic Corporation.  All Rights Reserved.\n");

  /*------------------------------------------------------------------------*
   * Parse Command Line
   *------------------------------------------------------------------------*/
    myname = argv[0];

    i = 1;
    while (i < argc)
    {
        str = argv[i];
        if (*str == '-')
        {
            switch (*++str)
            {
                case 'c':          /* which channel should we speak through */
                    chan = atoi(++str);
                    break;

                case 'r':
                    cmd_flags |= AUTO_REGISTER; /* register for T1/E1 monitoring (all CPs,full) */
                    break;

                case 's':
                    cmd_flags |= STATE_BANNER;  /* show last reported state banner */
                    break;

                case 'q':
                    cmd_flags |= QUIET_STATUS;  /* quiet line status messages (info also shown in banner) */
                    break;

                default:
                    show_usage(myname);
                    exit(1);
            }
        }
        ++i;
    }

    if (chan < 1)
    {
        show_usage(myname);
        exit(2);
    }

  /*------------------------------------------------------------------------*
   * Initialize Last-Reported State Area
   *------------------------------------------------------------------------*/
    for (i = 1; i <= MAXBOARD; i++)
    {                               /* for each possible CP */
        for (j = NET_T1A; j < (NET_T1A + MAX_TRUNKS_SHOWN); j++)
        {                           /* for each possible T1 (or E1) */
            state[i].t1e1[j] = T1E1_NOTEXIST;   /* assume all carriers do not exist */
        }
    }

  /*------------------------------------------------------------------------*
   * Initialize Core Library
   *------------------------------------------------------------------------*/
    if (tx_core_init() != TX_CORE_SUCCESS)
    {
        printf("Could not initialize core routines\n");
        exit(1);
    }

    cpi_init(0, &vstr);

    cpi_get_resources(MAXBOARD, cps);

  /*------------------------------------------------------------------------*
   * Open channels that T1/E1 status messages arrive over
   *------------------------------------------------------------------------*/
    for (i = 1; i <= MAXBOARD; i++)
    {
        if (cps[i])
        {                           /* there is a TX board with CP #i */
            port = PORT(i, chan);
            if ((fds[i] =
                 cpi_open(port, CPIM_PORT, NULL)) == CPI_INVALID_HANDLE)
            {
                printf("%s : could not open driver %x\n", argv[0],
                       cpi_get_last_error());
            }
            else
            {
                tx_core_install_handler(cpi_wait_obj(fds[i]), fds[i], GetMsg);
                tx_core_set_poll(cpi_wait_obj(fds[i]), TX_CORE_INPUT);
            }
        }
    }

  /*------------------------------------------------------------------------*
   * Register to receive T1/E1 status messages from each TX board
   *
   * NOTE: A banner line is maintained that provides a 1-character status
   *       for each given trunk.  This banner line is built dynamically
   *       so that the maximum number of trunks can be included.
   *       If more trunks exist than can be represented in a single 80-char
   *       banner then only a subset of the trunks will be shown.
   *------------------------------------------------------------------------*/
    memset( banner_hdr,  0, 80 );
    memset( banner_info, 0, 80 );
    if (cmd_flags & AUTO_REGISTER)
    {   /* auto-register for all T1E1 notifications */
        for (i = 1; i <= MAXBOARD; i++)
        {   /* for each possible TX board */
            if ((cps[i]) && (fds[i] != CPI_INVALID_HANDLE))
            {   /* there is a TX board with CP #i */
                t1handle = TxT1E1Open((unsigned char)i, 0); /* open T1E1 API */
                if (t1handle < 0)
                    printf("Error $%X from TxT1E1Open [CP #%d]\n", t1handle, i);
                else
                {
                    some_on_cp = 0;                 /* so far no T1E1's on this CP */
                    for (j = NET_T1A; j < (NET_T1A + MAX_TRUNKS_SHOWN); j++)
                    {   /* for each possible carrier */
                        status = TxT1E1SuperviseCarrier( t1handle,
                                                         (unsigned char)j,
                                                         T1E1_ALL_NOTIFS );
                        if (status == T1E1_SUCCESS)
                        {   /* successfully registered for notifs from CP #i, trunk #j */
                            state[i].t1e1[j] = T1E1_NEVER;  /* never received status of trunk */
                            if ( (some_on_cp == 0) && (strlen(banner_info) < (80-3)) )
                            {   /* first T1E1 for given CP number [and room in banner] */
                                sprintf( workbuf, "|%d:", i );
                                strcat( banner_hdr, "|  " );
                                strcat( banner_info, workbuf );

                                some_on_cp = 1;
                            }

                            if (strlen(banner_info) < (80-3))
                            {
                                sprintf( workbuf, "%d ", j );
                                strcat( banner_hdr, workbuf );
                                bpos_next = strlen(banner_info);
                                banner_info[bpos_next] = T1E1_NEVER;
                                bpos[i][j] = bpos_next;     /* remember position where this trunk's state shown */
                                while( strlen(banner_info) < strlen(banner_hdr) )
                                    strcat( banner_info, " " );
                            }
                        }
                    }

                    TxT1E1Close(t1handle);
                }
            }   /* END: there is a TX board with CP #i open */
        }   /* END: for each possible TX board */
    }   /* END: auto-register for all T1E1 notifications */

  /*------------------------------------------------------------------------*
   * Show T1E1 line state key if state banner activated
   *------------------------------------------------------------------------*/
    if (cmd_flags & STATE_BANNER)
    {                               /* show key for breaking down T1E1 line status */
        printf("\nT1E1 Line States (used in state banner):\n");
        printf("  %c = Trunk does not exist\n", T1E1_NOTEXIST);
        printf("  %c = Never received STATUS_CARRIER message\n", T1E1_NEVER);
        printf("  %c = Carrier Lost\n", T1E1_LOST);
        printf("  %c = Not Synchronized\n", T1E1_NOSYNC);
        printf("  %c = Synchronized\n\n", T1E1_SYNC);

        /* ----- show banner (which lines are being monitored) */
        printf("%s\n", banner_hdr);
        printf("%s\n", banner_info);
    }

  /*------------------------------------------------------------------------*
   * Install Operator Command Handler
   *------------------------------------------------------------------------*/
    tx_core_install_handler(stdInput, CPI_INVALID_HANDLE, ExitRoutine);
    tx_core_set_poll(stdInput, TX_CORE_INPUT);

  /*------------------------------------------------------------------------*
   * Enter TX core polling loop
   *------------------------------------------------------------------------*/
    tx_core_poll_loop(TX_CORE_INFINITE);

  /*------------------------------------------------------------------------*
   * Close all CPI channels
   *------------------------------------------------------------------------*/
    for (i = 1; i <= MAXBOARD; ++i)
        if (fds[i])
            cpi_close(fds[i]);

    return (0);
}
