/*
 * Test monapp to send and receive MON packets and generate other MON calls.
 * Always displays any MTP packets (or other events) received. User hits <CR>
 * and can get a prompt to enter MON primitive command.
 *
 * To run:
 *
 *  monapp
 *
 * User Commands: None
 *
 */

#ifdef UNIX
#include <stddef.h>
#include <strings.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <ctype.h>
#endif	                                /* UNIX */

#ifdef __OS2__
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#endif	                                /* __OS2__ */

#ifdef WIN32
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#endif	                                /* WIN32 */

#include "ss7type.h"
#include "tx_core.h"
#include "txcpi.h"
#include "monapp.h"

#define MAX_PARMS	10
#define DFLT_ENT   0x1b                 /* Default application entity ID */

/* globals (with defaults) for parameter values */
int Board   = 1;            /* MON's Board Number */
char invisible = 0;
char mtp2Header = 0;
char showData = 0;
char isupLevel = 0;
char ituUsed = 0;                       /* default assumption is that ansi is used */
void           *monChanKey = (void *)0xAABBCCDD; /* any key value, passed to CPIA callback routines */

/* other globals */
TX_HANDLE txHandle;
MONTXPKT TxBuf;
MONRXPKT RxBuf;                  /* packet buffer */

struct statistics
{
	unsigned long startTimeSec;
	unsigned long lastTime;
	unsigned long rateTime;
	unsigned long rate;
	unsigned long rateWithHeader;
	unsigned long correct;
	unsigned long error;
	unsigned long MSU;
	unsigned long FISU;
	unsigned long LSSU;
	unsigned long Bad;
	unsigned long nbBytes;
	unsigned long nbBytesForRate;
	unsigned long nbBytesHeader;
	unsigned long nbBytesForRateHeader;
	unsigned long bad;
};
#define MAX_LINK_NUM        128

struct statistics stats[ MAX_LINK_NUM ] = {0};
char pattern[ MAXMONPKT + 1 ];
unsigned long nbDatInd;

unsigned long statsIsup[ MAX_LINK_NUM ][ 0x32 ];

struct msgName
{
	char *codeName;
	char *name;
};

struct msgName messageName[] =
{
	{ /*0x00,*/ "xxxx", "xxx" },
	{ /*0x01,*/ "IAM ", "Initial Address Message" },
	{ /*0x02,*/ "SAM ", "Subsequent Address Message" },
	{ /*0x03,*/ "INR ", "Information Request" },
	{ /*0x04,*/ "INF ", "Information" },
	{ /*0x05,*/ "COT ", "Continuity" },
	{ /*0x06,*/ "ACM ", "Address Complete" },
	{ /*0x07,*/ "CON ", "Connect" },
	{ /*0x08,*/ "FOT ", "Forward Transfer" },
	{ /*0x09,*/ "ANM ", "Answer" },
	{ /*0x0A,*/ "Rsv ", "Reserved" },
	{ /*0x0B,*/ "Rsv ", "Reserved" },
	{ /*0x0C,*/ "REL ", "Release" },
	{ /*0x0D,*/ "SUS ", "Suspend" },
	{ /*0x0E,*/ "RES ", "Resume" },
	{ /*0x0F,*/ "Rsv ", "Reserved" },
	{ /*0x10,*/ "RLC ", "Release Complete" },
	{ /*0x11,*/ "CCR ", "Continuity Check Request" },
	{ /*0x12,*/ "RSC ", "Reset Circuit" },
	{ /*0x13,*/ "BLO ", "Blocking" },
	{ /*0x14,*/ "UBL ", "Unblocking" },
	{ /*0x15,*/ "BLA ", "Blocking Acknowledgement" },
	{ /*0x16,*/ "UBA ", "Unblocking Acknowledgement" },
	{ /*0x17,*/ "GRA ", "Circuit Group Reset Acknowledgement" },
	{ /*0x18,*/ "CGB ", "Circuit Group Blocking" },
	{ /*0x19,*/ "CGU ", "Circuit Group Unblocking" },
	{ /*0x1A,*/ "CGBA", "Circuit Group Blocking Acknowledgement" },
	{ /*0x1B,*/ "CGUA", "Circuit Group Unblocking Acknowledgement" },
	{ /*0x1C,*/ "CMR ", "Call Modification Request" },
	{ /*0x1D,*/ "CMC ", "Call Modification Completed" },
	{ /*0x1E,*/ "CMRJ", "Call Modification Reject" },
	{ /*0x1F,*/ "FRJ ", "Facility Reject" },
	{ /*0x20,*/ "FAA ", "Facility Accepted" },
	{ /*0x21,*/ "FAR ", "Facility Request" },
	{ /*0x22,*/ "Rsv ", "Reserved" },
	{ /*0x23,*/ "Rsv ", "Reserved" },
	{ /*0x24,*/ "LPA ", "Loop Back Acknowledgement" },
	{ /*0x25,*/ "Rsv ", "Reserved" },
	{ /*0x26,*/ "Rsv ", "Reserved" },
	{ /*0x27,*/ "DRS ", "Delayed Release" },
	{ /*0x28,*/ "PAM ", "Pass Along" },
	{ /*0x29,*/ "GRS ", "Circuit Group Reset" },
	{ /*0x2A,*/ "CQM ", "Circuit Group Query" },
	{ /*0x2B,*/ "CQR ", "Circuit Group Query Response" },
	{ /*0x2C,*/ "CPG ", "Call Progress" },
	{ /*0x2D,*/ "USR ", "User to User Information" },
	{ /*0x2E,*/ "UCIC", "Unequipped Circuit Identification Code" },
	{ /*0x2F,*/ "CFN ", "Confusion" },
	{ /*0x30,*/ "OLM ", "Overload" },
	{ /*0x31,*/ "CRG ", "Charge information" },
};

extern CPI_WAIT_TYPE stdInput;
void showHex(unsigned char *Buf, int ByteCnt);

/*
  func to dump data in hex+ascii format

IN:  Buf        data to show
	 ByteCnt    #Bytes in Buf
OUT: <file>      gets multiple lines
*/
void showHex(unsigned char *Buf, int ByteCnt)
{
	unsigned short Li, LMax, ToDo, Bi, Ai;
	char Ascii[80];
	unsigned char B;

	/* Dump to std out in hex followed by ascii */

	for (ToDo = ByteCnt, Bi = 0; ToDo > 0; ToDo -= LMax)
    {
    	LMax = (ToDo > 16) ? 16 : ToDo;         /* max bytes this line */
    	printf("%04X| ", Bi);
    	Ai = 0;
    	for (Li = 0; Li < LMax; Li++)           /* each byte this line */
        {
        	B = *Buf++;                         /* next byte to show */
        	printf("%02X ", B);                 /* as hex */
        	if (!isprint(B))                    /* printable ascii? */
            	B = '.';                       /* no, use this */
        	Ascii[Ai++] = B;                    /* into ascii portiob */
        }  /* for Li */
        printf("|");
    	for (Li = LMax; Li < 16; Li++)          /* pad short ln if necessary */
        	printf("   ");                      /* so asci lines up */
        printf("Ascii[");
    	Ascii[Ai] = 0;                          /* terminate the ascii */
    	printf("%s]\n", Ascii);                  /* show the ascii */
    	Bi += LMax;
	}	/* for ToDo */

}	/* ShowHex */

/* tokenize
 *
 * Takes input line and creates an array of pointers to individual
 * tokens in the input line.  First separator after each token in
 * input line is replaced with a null so that each token becomes
 * a null terminated string.
 */
short tokenize( char *buf, char *tokens[] )
{
	char	*p = buf;
	char	*endp;
	short   cnt;

	/* initialize token pointers to all null */
	for( cnt = 0; cnt <= MAX_PARMS; cnt++ )
		tokens[cnt] = 0;

	/* begin scanning for tokens */
	cnt = 0;
	while( *p != 0 && cnt <= MAX_PARMS )
	{
		/* skip leading space or tabs */
		while( *p == ' ' || *p == '\t' )
			p++;

		/* beginning of token (or end of line) */
		if( *p != 0 )
		{
			/* scan for end of token, converting to upper case as we go */
			endp = p;
			while( *endp != ' ' && *endp != '\t' && *endp != 0 )
			{
				if( islower( *endp ) )
				{
					*endp = 'A' + (*endp - 'a');	/* convert to upper */
				}
				endp++;
			}

			/* now have a begin & end of token */
			tokens[cnt++] = p;
			if( *endp == 0 )
			{
				p = endp;	/* end of input - terminate scan */
			}
			else
			{
				*endp = 0;		/* null terminate token */
				p = &endp[1];	/* start next scan at after null */
			}
		}   /* end if *p != 0 */
	}   /* end while *p != 0 .... */

	return( cnt );
}

/* cmdPrompt
 *
 * Prompts user to enter command, collects parameters, and issues appropriate
 * MON commands
 */
short cmdPrompt( void )
{
	char    buffer[ 255 ];
	char	*tokens[MAX_PARMS +1];
	short   cnt;
	int idxLink = 0;
	unsigned long totalCorrect = 0;
	unsigned long totalError = 0;
	unsigned long totalMSU = 0;
	unsigned long totalFISU = 0;
	unsigned long totalLSSU = 0;
	unsigned long totalRate = 0;
	unsigned long totalRateWithHeader = 0;
	unsigned long totalBad = 0;
	unsigned long totalNbBytes = 0;
	unsigned long totalNbBytesHeader = 0;
	int msgType = 0;
	int statsType = 0;
	unsigned long totalIsup[ 0x32 ];
	int nbChar;

	memset( totalIsup, 0, sizeof( totalIsup ));

	/* prompt for a command */
	printf( "Command?>" );
	tx_core_gets(buffer, 255);
	printf( "\n" );
	cnt = tokenize( buffer, tokens );

	if( cnt == 0 )
		return(TX_CORE_SUCCESS);

	if (strncmp( tokens[0], "Q", 1 ) == 0)
	{
		return (TX_CORE_EXIT_POLL_LOOP);
	}
	else if ( strcmp( tokens[ 0 ], "MTP2" ) == 0)
	{
		if ( cnt != 2 )
		{
			printf( "Parameter error MTP2 0|1, 0 means not mtp2 Header, 1 means mtp2 header \n");
			return(TX_CORE_SUCCESS);
		}
		if ( tokens[ 1 ][ 0 ] == '0' )
			mtp2Header = 0;
		else
			mtp2Header = 1;
	}
	else if ( strcmp( tokens[ 0 ], "ISUP" ) == 0)
	{
		if ( cnt != 2 )
		{
			printf( "Parameter error ISUP 0|1, 0 not isup level, 1 isup level\n");
			return(TX_CORE_SUCCESS);
		}
		if ( tokens[ 1 ][ 0 ] == '0' )
			isupLevel = 0;
		else
			isupLevel = 1;
	}
	else if ( strcmp( tokens[ 0 ], "SHOW" ) == 0)
	{
		if ( cnt != 2 )
		{
			printf( "Parameter error SHOW 0|1, 0 to not display data, 1 to diplay data \n");
			return(TX_CORE_SUCCESS);
		}
		if ( tokens[ 1 ][ 0 ] == '0' )
			showData = 0;
		else
			showData = 1;
	}
	else if ( strcmp( tokens[ 0 ], "STATS" ) == 0)
	{
		int link = -1;
		if ( cnt < 2 )
		{
			printf("Parameter error STATS ISUP|GEN link_number\n");
			return( TX_CORE_SUCCESS );
		}
		if (strcmp( tokens[ 1 ], "ISUP") == 0 )
			statsType = 0;
		else if (strcmp( tokens[ 1 ], "GEN") == 0 )
			statsType = 1;
		else
		{
			printf("Parameter error STATS ISUP|GEN link_number\n");
			return( TX_CORE_SUCCESS );
		}

		if ( cnt > 2 )
		{
			if ( strcmp( tokens[ 2 ], "ALL") == 0 )
				link = -1;
			else
			{
                if (( (sscanf( tokens[ 2 ], "%i", &link )) != 1 ) || (( link < 0 ) || ( link > MAX_LINK_NUM -1 )))
				{
					printf( "Parameter error [%s] - link expected\n", tokens[ 2 ]);
					return(TX_CORE_SUCCESS);
				}
			}
		}
		else
			link = -1;

		if (statsType == 1 )
		{
			for ( idxLink = 0; idxLink < MAX_LINK_NUM; idxLink++)
			{
				if (( link == -1 ) || ( link == idxLink ))
					printf("Link[%d] OK[%lu] NOK[%lu] MSU[%lu] FISU[%lu] LSSU[%lu] Bad[%lu] Bytes[%lu]+Header[%lu]=[%lu] rate %d(%d)/sec\n", idxLink, stats[ idxLink ].correct, stats[ idxLink ].error, stats[ idxLink ].MSU,stats[ idxLink ].FISU,stats[ idxLink ].LSSU, stats[ idxLink ].bad, stats[ idxLink ].nbBytes,stats[ idxLink ].nbBytesHeader, stats[ idxLink ].nbBytes + stats[ idxLink ].nbBytesHeader, stats[ idxLink ].rate, stats[ idxLink ].rateWithHeader);
				totalCorrect += stats[ idxLink ].correct;
				totalError += stats[ idxLink ].error;
				totalMSU += stats[ idxLink ].MSU;
				totalFISU += stats[ idxLink ].FISU;
				totalLSSU += stats[ idxLink ].LSSU;
				totalBad += stats[ idxLink ].bad;
				totalNbBytes += stats[ idxLink ].nbBytes;
				totalNbBytesHeader += stats[ idxLink ].nbBytesHeader;
				totalRate += stats[ idxLink ].rate;
				totalRateWithHeader += stats[ idxLink ].rateWithHeader;
			}
			if ( link == -1 )
			{
				printf("---------------------------------------------------------------------------------------------------------\n");
				printf("Total OK[%lu] NOK[%lu] MSU[%lu] FISU[%lu] LSSU[%lu] Bad[%lu] Bytes[%lu]+Header[%lu]=[%lu] rate %d(%d)/sec\n", totalCorrect, totalError, totalMSU, totalFISU, totalLSSU, totalBad, totalNbBytes, totalNbBytesHeader, totalNbBytes + totalNbBytesHeader, totalRate, totalRateWithHeader);
				printf("Data Indication %lu\n", nbDatInd);
			}
		}
		else
		{
			for ( idxLink = 0; idxLink < MAX_LINK_NUM; idxLink++)
			{
				if (( link == -1 ) || ( link == idxLink ))
				{
					printf("Isup Statistics for link %d\n", idxLink);
					for (msgType = 0; msgType < 0x32; msgType++)
					{
						totalIsup[ msgType ] += statsIsup[ idxLink ][ msgType ];
						printf("%s[%9lu] ", messageName[ msgType ].codeName, statsIsup[ idxLink ][ msgType ]);
						if (( msgType + 1 ) % 5 == 0 )
							printf("\n");
					}
				}
			}
			if ( link == - 1 )
			{
				printf("---------------------------------------------------------------------------------------------------------\n");
				for (msgType = 0; msgType < 0x32; msgType++)
				{
					printf("%s[%9lu] ", messageName[ msgType ].codeName, totalIsup[ msgType ]);
					if (( msgType + 1 ) % 5 == 0 )
						printf("\n");
				}
			}
		}
	}
	else if ( strcmp( tokens[ 0 ], "RESET" ) == 0)
	{
		int link = 0;
        if (( cnt > 1 ) && (( (sscanf( tokens[ 1 ], "%i", &link )) != 1 ) || (( link < 0 ) || ( link > MAX_LINK_NUM -1 ))))
		{
			printf( "Parameter error [%s] - link expected\n", tokens[ 1 ]);
			return(TX_CORE_SUCCESS);
		}
		if ( cnt == 1 )
        {
			memset( stats, 0, sizeof( stats ));
            memset( statsIsup, 0, sizeof( statsIsup ));
        }
		else
        {
			memset( &stats[ link ], 0, sizeof( stats[ link ] ));
            memset( &statsIsup[link][0], 0, sizeof( statsIsup[link] ));

        }
		nbDatInd = 0;
	}
	else if ( strcmp( tokens[ 0 ], "DATA" ) == 0)
	{
		if ( cnt < 2 )
		{
			printf( "The DATA command expects data , e.g.: data bonjour\n" );
			return( TX_CORE_SUCCESS );
		}
		if (( nbChar = strlen( tokens[ 1 ] )) > 80 )
		{
			printf( "Parameter error [%s] - number of characters expected\n", tokens[ 1 ]);
			return( TX_CORE_SUCCESS );
		}
		if ( tokens[ 1 ][ 0 ] == '+' )
		{
			if ( sscanf( &tokens[ 1 ][ 1 ], "%d", &nbChar) != 1 )
			{
				printf( "Parameter error [%s] - number of characters expected\n", tokens[ 1 ]);
				return( TX_CORE_SUCCESS );
			}
		}
		if ( nbChar >= 255 )
		{
			printf("pattern to receive too long[length %d]\n", nbChar);
			return( TX_CORE_SUCCESS );
		}
		if ( tokens[ 1 ][ 0 ] == '+' )
		{
			memset( pattern, 0, sizeof( pattern ));
			memset( pattern, 'x', nbChar);
		}
		else
			strcpy( pattern, tokens[ 1 ]);
	}
	else
	{
		printf( "Valid Commands:\n" );
		printf( "Q                              Quit the program\n" );
		printf( "I                              No traces\n");
		printf( "V                              Traces\n");
		printf( "DATA pattern                   Set a pattern\n");
		printf( "STATS ISUP|GEN [link_number]   Get statistcs for one link or all links\n");
		printf( "RESET [link_number]            Reset statistcs for one link or all links\n");
		printf( "MTP2  [0|1]                    0 no mtp2 header in the data, 1 mtp2 header in the data\n");
		printf( "SHOW  [0|1]                    0 not to display data, 1 to display data\n");
        printf( "ISUP  [0|1]                    0 no isup msgs, 1 decode ISUP msg types\n");
	}
	return(TX_CORE_SUCCESS);
}
/* kbHandler
 *
 * Keyboard event handler routine, to be called by "tx_core" whenever an asynchronous
 * keyboard input is detected.
 */
short kbHandler(TX_HANDLE handle, short events)
{
	char c;

	/* check if input event really did occur */
	if (tx_core_process_events(TX_CORE_INPUT, events) == FALSE)
	return (TX_CORE_SUCCESS);

	/* make sure a key was hit */
	if (tx_core_kbhit())
	{
		c = getchar();
		if( c == 'q' || c == 'Q' )
			return (TX_CORE_EXIT_POLL_LOOP);
		else if ( c == 'i' || c == 'I' )
			invisible = 1;
		else if ( c == 'v' || c == 'V' )
			invisible = 0;
		else
			return(cmdPrompt());
	}

	return (TX_CORE_SUCCESS);
}

void parseMsg(DatIndMsg *msg )
{
	U8      *ptr = (U8 *)msg;
	unsigned long currentTime;
	int idxData = 0;
	unsigned int lengthData = 0;
	unsigned int idxBuf = 0;
	int flagError = 0;
	char buffer[ MAXMONPKT + 1 ];
	int rxlen = 0;
    int offset;

	rxlen = cpi_cptoh_s(msg->dataLen);

	/* End of message flag */
	if (rxlen == 0)
		return;

	if ( invisible == 0 )
	{
		printf("=======================================================\n");
		printf("Length: %d\tLink #:%d\tNumSpare: %d\n", cpi_cptoh_s(msg->dataLen), msg->linkNum, msg->numSpare);
		printf("Number seconds: %d\t msecs: %d\n", cpi_cptoh_l(msg->secs), cpi_cptoh_s(msg->msecs));
	}


	stats[ msg->linkNum ].nbBytes += rxlen;
	stats[ msg->linkNum ].nbBytesHeader += 12 + msg->numSpare;

	if ( stats[ msg->linkNum ].startTimeSec == 0 )
	{
		stats[ msg->linkNum ].startTimeSec = cpi_cptoh_l(msg->secs);
		stats[ msg->linkNum ].lastTime = 0;
		stats[ msg->linkNum ].rateTime = 0;
	}

	currentTime = cpi_cptoh_l(msg->secs) - stats[ msg->linkNum ].startTimeSec;

	if ( currentTime - stats[ msg->linkNum ].lastTime > 1 )
	{
		stats[ msg->linkNum ].nbBytesForRate = rxlen;
		stats[ msg->linkNum ].nbBytesForRateHeader = 12 + msg->numSpare;
		stats[ msg->linkNum ].rate = 0;
		stats[ msg->linkNum ].rateTime = currentTime;
	}
	else
	{
		stats[ msg->linkNum ].nbBytesForRate += rxlen;
		stats[ msg->linkNum ].nbBytesForRateHeader += 12 + msg->numSpare;

		if (( currentTime - stats[ msg->linkNum].rateTime) == 0 )
		{
			stats[ msg->linkNum ].rate =  stats[ msg->linkNum ].nbBytesForRate;
			stats[ msg->linkNum ].rateWithHeader =  stats[ msg->linkNum ].nbBytesForRate + stats[ msg->linkNum ].nbBytesForRateHeader;
		}
		else
		{
			stats[ msg->linkNum ].rate =  stats[ msg->linkNum ].nbBytesForRate / ( currentTime - stats[ msg->linkNum].rateTime );
			stats[ msg->linkNum ].rateWithHeader =  (stats[ msg->linkNum ].nbBytesForRate + stats[ msg->linkNum ].nbBytesForRateHeader) / ( currentTime - stats[ msg->linkNum].rateTime );
		}
	}
	stats[ msg->linkNum ].lastTime = currentTime;

	if ( mtp2Header == 1 )
	{
		if (rxlen < 3)
		{
			printf("Bad message length %d\n", rxlen);
			stats[ msg->linkNum ].bad++;
			return;
		}
		else if (rxlen == 3)
		{
			stats[ msg->linkNum ].FISU++;
			if ( invisible == 0 )
			{
				printf("FISU: BSN %d:%d FSN %d:%d\n", msg->data[0] & 0x7F, (msg->data[0] & 0x80) >> 7,msg->data[1] & 0x7f, (msg->data[1] & 0x80) >> 7);
				if ( showData == 1 )
					showHex( msg->data, rxlen );
			}
		}
		else if ((rxlen == 4) || (rxlen == 5))
		{
			stats[ msg->linkNum ].LSSU++;
			switch (msg->data[3])
			{
				case 0 :
					if ( invisible == 0 )
						printf("SIO : BSN %d:%d FSN %d:%d\n", msg->data[0] & 0x7F, (msg->data[0] & 0x80) >> 7,msg->data[1] & 0x7f, (msg->data[1] & 0x80) >> 7);
					break;
				case 1 :
					if ( invisible == 0 )
						printf("SIN : BSN %d:%d FSN %d:%d\n",msg->data[0] & 0x7F, (msg->data[0] & 0x80) >> 7,msg->data[1] & 0x7f, (msg->data[1] & 0x80) >> 7);
					break;
				case 2 :
					if ( invisible == 0 )
						printf("SIE : BSN %d:%d FSN %d:%d\n",msg->data[0] & 0x7F, (msg->data[0] & 0x80) >> 7,msg->data[1] & 0x7f, (msg->data[1] & 0x80) >> 7);
					break;
				case 3 :
					if ( invisible == 0 )
						printf("SIOS: BSN %d:%d FSN %d:%d\n",msg->data[0] & 0x7F, (msg->data[0] & 0x80) >> 7,msg->data[1] & 0x7f, (msg->data[1] & 0x80) >> 7);
					break;
				case 4 :
					if ( invisible == 0 )
						printf("SIPO: BSN %d:%d FSN %d:%d\n",msg->data[0] & 0x7F, (msg->data[0] & 0x80) >> 7,msg->data[1] & 0x7f, (msg->data[1] & 0x80) >> 7);
					break;
				case 5 :
					if ( invisible == 0 )
						printf("SIB : BSN %d:%d FSN %d:%d\n",msg->data[0] & 0x7F, (msg->data[0] & 0x80) >> 7,msg->data[1] & 0x7f, (msg->data[1] & 0x80) >> 7);
					break;
				default :
					if ( invisible == 0 )
						printf("LSSU: BSN %d:%d FSN %d:%d\n",msg->data[0] & 0x7F, (msg->data[0] & 0x80) >> 7,msg->data[1] & 0x7f, (msg->data[1] & 0x80) >> 7);
					break;
			}
			if (( invisible == 0 ) && ( showData == 1 ))
				showHex( msg->data, rxlen );
		}
	}

	if (( mtp2Header == 0 ) || (( mtp2Header == 1 ) && ( rxlen > 5 )))
	{
		lengthData = rxlen;
		idxData = 0;
		stats[ msg->linkNum ].MSU++;
		if ( mtp2Header == 1 )
		{
			idxData = 3;
			lengthData = rxlen - 3;

			if ( invisible == 0 )
			{
				printf("MSU : BSN %d:%d FSN %d:%d\n",msg->data[0] & 0x7F, (msg->data[0] & 0x80) >> 7,msg->data[1] & 0x7f, (msg->data[1] & 0x80) >> 7);
			}
		}
		if ( lengthData > MAXMONPKT )
		{
			printf("Error Length too long %d, message truncated to %d\n", lengthData, MAXMONPKT);
			lengthData = MAXMONPKT;
		}

		/*printf("Length data %d\n", lengthData);*/
		for ( idxBuf = 0; idxBuf < lengthData; idxBuf++)
			buffer[ idxBuf ] = 	msg->data[ idxData + idxBuf ];
		buffer[ idxBuf ] = 0;
		if ( isupLevel == 1 )
		{
			if (( msg->data[ idxData ] & 0x0F ) == 5 )
			{
				int msgType = -1;
				if ( invisible == 0 )
					printf("SLS = %d\n", msg->data[ idxData + 4 ] & 0xF0 >> 4);
                /* The following value of 10 would need to change for an ITU stack */
                if( ituUsed )
                {
                    offset = 7;
                }
                else
                {
                    offset = 10;
                }
                msgType = buffer[ offset ];
				if (( msgType <= 0 ) || ( msgType > 0x31 ))
					msgType = 0;
				if ( invisible == 0 )
					printf("Message : %d %s %s\n", msgType, messageName[ msgType ].codeName, messageName[ msgType ].name);
				statsIsup[ msg->linkNum ][ msgType ]++;
            }
			else
			{
				if ( invisible == 0 )
					printf("Not an isup message - SIO = %d %d\n", msg->data[ idxData ] & 0xF0 >> 4, msg->data[ idxData ] & 0x0F);
			}
		}
		else if ( pattern[ 0 ] != '\0' )
		{
			if ( lengthData != strlen( pattern ))
			{
				printf("Error : bad length pattern received\n");
				flagError = 1;
			}
			if (( flagError == 0 ) && ( strcmp( pattern, buffer ) != 0 ))
			{
				printf("Error : bad pattern received\n");
				flagError = 1;
			}
			if ( invisible == 0 )
			{
				printf("Received [");
				for ( idxBuf = 0; idxBuf < lengthData + 0; idxBuf++)
					printf("%c", msg->data[idxData + 0 + idxBuf ]);
				printf( "]\n" );
				if ( flagError > 0 )
					printf("Expected [%s]\n", pattern);
			}
			if ( flagError == 0 )
			{
				stats[ msg->linkNum ].correct++;
			}
			else
			{
				stats[ msg->linkNum ].error++;
			}
		}
		if ((invisible == 0 ) && ( showData == 1 ))
			showHex( msg->data, rxlen );
	}

	parseMsg((DatIndMsg *)(ptr + 12 + msg->numSpare + rxlen ));

}

/****************************************************************************
 * MonRxNotify - handle notification of incoming receive (from the TX board)
 *
 * INPUT:   handle - handle to CPI channel that message was received over
 *          ckhey - any control key associated with given CPI channel
 ****************************************************************************/
void MonRxNotify( TX_HANDLE handle, void *chkey )
{
    S16             ret;
    short           len = sizeof(MONRXPKT);
    unsigned short  start, end;
    DatIndHeader   *hdr;

    /* ----- read the received message ----- */
    len = sizeof( CPIPKT );
    if (cpia_get_data( handle, (CPIPKT *)&RxBuf, &len ) == CPI_ERROR)
    {
        ret = cpi_get_last_error( );
        printf( "Error 0x%X [%s] returned from cpia_get_data\n", ret, cpi_get_error_str( ret ) );

        return;
    }

	start = cpi_cptoh_s(RxBuf.hdr.start);
	end = cpi_cptoh_s(RxBuf.hdr.end);

    hdr = (DatIndHeader *)&RxBuf.data[start];

	if ((cpi_cptoh_l(hdr->monId)) != MON_MAGIC)
	{
        printf( "Received invalid magic number...tossing message\n" );

        return;
	}

    /* got an incoming message, decode it */
	switch( hdr->msgType )
	{
		case MON_DATA_IND:
            if (invisible == 0)
			{
                printf( "Received data indication\n" );
                printf( "*******************************************************\n" );
			}
			nbDatInd++;

            parseMsg( (DatIndMsg *)(&RxBuf.data[sizeof(DatIndHeader)]) );
			break;

		default:
            printf( "Unknown MON message received, message type: %d\n", hdr->msgType );
    }
}

/****************************************************************************
 * MonTxNotify - handle notification of transmit (to TX board) completed
 *
 * For the MONapp program, a transmit completes whenever a BIND (or UNBIND)
 * sent by MONapp has reached the monitor task on the board.
 *
 * INPUT:   handle - handle to CPI channel that message was transmitted over
 *          chkey  - MONapp's control key associated with given TX board
 *          msg    - address of message that has just been transmitted
 *          tx_key - MONapp's control key associated with this transmit
 *          ccode  - completion code of transmitted packet
 ****************************************************************************/
void MonTxNotify( TX_HANDLE handle, void *chkey, CPIPKT *msg,
                  void *tx_key, CPI_ERR_TYPE ccode )
{
    /* Nothing to do here since all messages transmitted to the board are
     * built using the global TxBuf (so no memory to deallocate)
     */
}

/****************************************************************************
 * MONHandler
 *
 * MON event handler routine, to be called by "tx_core"
 * whenever an asynchronous event message for MON is pending.
 ****************************************************************************/
short MONHandler( TX_HANDLE handle, short events )
{
    S16     ret;
    S16     status;

    /* check if input event really did occur */
    if (tx_core_process_events( TX_CORE_INPUT, events ) == FALSE)
    {
        return( TX_CORE_SUCCESS );
    }

    /* call into the CPI library to service any asynchronous IO over the CPI channel */
    status = cpia_intr( handle );
    if( status != CPI_SUCCESS )
    {
        ret = cpi_get_last_error( );
        printf( "Error 0x%X [%s] returned from cpia_intr\n", ret, cpi_get_error_str( ret ) );

        return( TX_CORE_EXIT_POLL_LOOP );   /* fatal */
    }

    return( TX_CORE_SUCCESS );
}

/*
 * Main routine
 *
 * Parse command line arguments, start main processing loop
 */
int main( int argc, char *argv[] )
{
	short           n;
	char            sbuf[CPSTRSIZE], *ptr = sbuf; /* dummy for old cpi_init */
	char            *ap;
	short           doQuit = 0;
    S16             dummy = 0;
	CPI_WAIT_TYPE   hSync;      /* synchronization handle for MON API */
    short           ret = 0;
    CPI_OPEN_CONFIG openCfg;
    void           *myTxKey = NULL;     /* per-transmit key under monapp's control - not currently needed so set to NULL */

	memset( stats, 0, sizeof( stats ));

	/*These values never change !! */
	TxBuf.hdr.hdr.src   = PORT(0, DFLT_ENT);
	TxBuf.hdr.hdr.dst   = PORT(Board, MON_ENT);
	TxBuf.hdr.start     = cpi_htocp_s(FILLER_COUNT);

	/* process arguments */
	for (n=1; n < argc; n++)
	{
		ap = argv[n];
		if (*ap == '-')
		{
			switch (*++ap)
			{

				case 'b' :     /* board number */
					if (*++ap == 0)
					{
						n++;
						Board = atoi(argv[n]);
					}
					else
						Board = atoi(ap);

					if( Board < 1 || Board > 8 )
					{
						printf("Board must be in range 1 - 8\n");
						exit(1);
					}
					break;

                case 't' :              /* using itu, default ansi */
                    ituUsed = 1;
                    break;

                case 'i' :              /* don't show received packets, same as i entered from keyboard(kbHandler) */
                    invisible = 1;
                    break;

				case '?' :      /* Help */
					printf ("MONapp  [-b <board>] [-?]\n");
					printf ("    -b <board>    Board number to use; default: 1\n");
                    printf ("    -t            ITU in use; default: ANSI\n");
                    printf ("    -i            invisible mode selected\n");
					exit (1);
					break;

				default:
					printf("Unrecogized Parameter: %s\n", argv[n]);
					printf ("MONapp  [-b <board>] [-?]\n");
					printf ("    -b <board>    Board number to use; default: 1\n");
                    printf ("    -t            ITU in use; default: ANSI\n");
                    printf ("    -i            invisible mode selected\n");
					exit(1);

			}   /* end switch */
		}
		else
		{
			printf("Unrecogized Parameter: %s\n", argv[n]);
			exit(1);
		}
	}   /* end for loop */

    /* open the cpi channel */
    ret = cpi_init( dummy, &ptr );
    if (ret != CPI_SUCCESS)
    {
        printf( "Error 0x%X [%s] returned from cpi_init\n", ret, cpi_get_error_str( ret ) );
        exit( 1 );
    }

    memset( (void *)&openCfg, 0, sizeof(CPI_OPEN_CONFIG) );
    openCfg.board = Board;                  /* TX board number */
    openCfg.channel = DFLT_ENT;             /* channel number to register for receives on */
    openCfg.maxInProg = TXCPI_MAX_IN_PROG;  /* maximum number of transmits or receives in progress for this channel (set to max allowed) */
    openCfg.rxnotify = MonRxNotify;         /* receive notification - trigger to call cpia_get_data */
    openCfg.txnotify = MonTxNotify;         /* transmit complete notification */
    /* NOTE: openCfg.evnotify is not used so leave as NULL */

    txHandle = cpix_open( monChanKey, &openCfg );
	if (txHandle == CPI_INVALID_HANDLE)
	{
        ret = cpi_get_last_error( );
        printf( "Error 0x%X [%s] returned from cpix_open\n", ret, cpi_get_error_str( ret ) );
        exit( 1 );
	}

    hSync = cpi_wait_obj( txHandle );
	if (hSync == CPI_INVALID_WAIT_HANDLE)
	{
        ret = cpi_get_last_error( );
        printf( "Error 0x%X [%s] returned from cpi_wait_obj\n", ret, cpi_get_error_str( ret ) );
        exit( 1 );
	}

	/* Bind this application as a user of the MON layer */
	TxBuf.hdr.hdr.src           = PORT(0, DFLT_ENT);
	TxBuf.hdr.hdr.dst           = PORT(Board, MON_ENT);
	TxBuf.hdr.hdr.length        = HDRSIZE + sizeof(BindReq);
	TxBuf.hdr.start             = cpi_htocp_s(FILLER_COUNT);
	TxBuf.hdr.end               = cpi_htocp_s(ENDSIZE + sizeof(BindReq));

	TxBuf.data.bindReq.monId    = cpi_htocp_l(MON_MAGIC);

	TxBuf.data.bindReq.msgType  = MON_BIND_REQ;
	TxBuf.data.bindReq.srcEnt   = DFLT_ENT;
	TxBuf.data.bindReq.srcInst  = 0;

    ret = cpia_send( txHandle, (CPIPKT *)&TxBuf, myTxKey );
	if (ret != CPI_SUCCESS)
	{
        ret = cpi_get_last_error( );
        printf( "Error 0x%X [%s] returned from cpia_send of BIND\n", ret, cpi_get_error_str( ret ) );
        exit( 1 );
	}
    printf( "Binding request sent\n" );

	/* install keyboard and MON event handlers with "tx_core" and let it
     * perform the main "wait for event" loop
     */
	tx_core_init();
	tx_core_install_handler( stdInput, TX_CORE_UNUSED, kbHandler );
	tx_core_install_handler( hSync, txHandle, MONHandler );
	tx_core_set_poll(stdInput, TX_CORE_INPUT);
	tx_core_set_poll(hSync, TX_CORE_INPUT);

	/**** let her rip ****/
	tx_core_poll_loop( TX_CORE_INFINITE );



    /* UN-Bind this application as a user of the MON layer */
	TxBuf.hdr.hdr.src           = PORT(0, DFLT_ENT);
	TxBuf.hdr.hdr.dst           = PORT(Board, MON_ENT);
	TxBuf.hdr.hdr.length        = HDRSIZE + sizeof(UnBindReq);
	TxBuf.hdr.start             = cpi_htocp_s(FILLER_COUNT);
	TxBuf.hdr.end               = cpi_htocp_s(ENDSIZE + sizeof(UnBindReq));

	TxBuf.data.bindReq.monId    = cpi_htocp_l(MON_MAGIC);

	TxBuf.data.bindReq.msgType  = MON_UNBIND_REQ;
	TxBuf.data.bindReq.srcEnt   = DFLT_ENT;
	TxBuf.data.bindReq.srcInst  = 0;

    ret = cpia_send( txHandle, (CPIPKT *)&TxBuf, myTxKey );
	if (ret != CPI_SUCCESS)
	{
        ret = cpi_get_last_error( );
        printf( "Error 0x%X [%s] returned from cpia_send of UNBIND\n", ret, cpi_get_error_str( ret ) );
        exit( 1 );
	}
    printf( "Unbind sent\n" );

    /* wait a little before closing the channel to give the UNBIND a chance to be sent by CPI lib before closing down the channel */
#ifdef UNIX
	sleep( 1 );
#else
	Sleep( 500 );
#endif

	/* when tx_core_poll_loop returns, user has selected "quit" */
	cpi_close(txHandle);
} /* end of main */
