/*****************************************************************************
 * cpcon.c - Utility providing operator console interface to TX board
 *****************************************************************************/

/*****************************************************************************
 *
 *                         PROPRIETARY NOTICE
 *
 *  (c)Copyright 1988 - 2009 Dialogic Corporation.  All Rights Reserved
 *
 *  The following is licensed program material and is considered
 *  proprietary and confidential.  No part of this program may be
 *  reproduced or used in any form or by any means without the express
 *  written permission of Dialogic Corporation.
 *
 *****************************************************************************/

/* TX board console interface */
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef WIN32
#include <conio.h>
#endif

#include <ctadef.h>
#include <ctaerr.h>
#include <nmstypes.h>
#include <txcpi.h>
#include <dpriface.h>
#include <hmidef.h>

/* ----- Modification History: --------------------------------------------- */
#define MH_VERSION      4           /* identifier of major release version */
#define MH_REVISION     1           /* identifier of release revision */

#define DFLTCH      PT_CONSOLE
#define DFLTBD      1
#define LINEMAX     128

unsigned short  board = DFLTBD;
unsigned short  chan = DFLTCH;

static char    *usagestr = "Usage: %s <-b board> <-c chan>\n";

#ifdef WIN32
CTA_WAITOBJ     stdInput;
#endif

TX_HANDLE       cpiHandle;
CTA_WAITOBJ     cpiWaitObj;

/* Service name/manager pair for ctaInitialize */
CTA_SERVICE_NAME HmiServiceNames[] = { {"HMI", "HMIMGR"} };

/* Service list for ctaOpenServices */
CTA_SERVICE_DESC HmiOpenSvcLst[] = { {{"HMI", "HMIMGR"}, {0}, {0}, {0}} };

/* Service list for ctaCloseServices */
char           *HmiCloseSvcLst[] = { "HMI" };

/****************************************************************************
 * GetResponse - This service routine takes packets from the TX board
 *               and dumps the data to the screen.
 ****************************************************************************/
#ifdef WIN32
DWORD NMSSTDCALL GetResponse( CTAQUEUEHD hmiQueue, CTA_WAITOBJ waitObj, void *arg )
#else
DWORD NMSSTDCALL GetResponse( CTAQUEUEHD hmiQueue, CTA_WAITOBJ *waitObj, void *arg )
#endif
{
    CPIPKT          outbuf;
    short           len = CPIBUFSIZE;
    char           *output;

    /* get data from the CP */
    if (cpi_get_data(cpiHandle, &outbuf, &len) == CPI_ERROR)
    {
        printf("Cpcon Receive Error (%d)\n", cpi_get_last_error( ));
        return ((DWORD) CPI_ERROR);
    }
    else
    {
        if (len)
        {                           /* did we actually get something */

            /* output it to the screen */
            output = ((char *)&outbuf) + sizeof(CPIHDR);

            output[len] = '\0';
            printf( output );
            fflush( stdout );
        }
    }

    return (SUCCESS);
}                                   /* GetResponse */

/****************************************************************************
 * GetCommand - This routine takes command lines and forwards them
 *              to the CP to handle.
 *
 *   parameters  - None used for OS/2 and NT (only for UNIX)
 ****************************************************************************/
#ifdef WIN32
DWORD NMSSTDCALL GetCommand(CTAQUEUEHD hmiQueue, CTA_WAITOBJ waitObj, void *arg)
#else
DWORD NMSSTDCALL GetCommand(CTAQUEUEHD hmiQueue, CTA_WAITOBJ *waitObj, void *arg)
#endif
{
    CPIPKT          inbuf;
    unsigned char  *input;
    CPIHDR         *hp;
    short           len, ret;

    /* check if keyboard input event did occur */

#ifdef WIN32
    if (kbhit( ) == FALSE)
    {
        FlushConsoleInputBuffer(stdInput);
        return (SUCCESS);
    }
#endif

    /* set up for getting the command line */

    input = ((unsigned char *)&inbuf) + sizeof(CPIHDR);
    len = LINEMAX - sizeof(CPIHDR);

    /* get the typed in command */
    fgets((char *)input, (int)len, stdin);
    while ( (strlen((const char *)input) > 0) && (input[strlen((const char *)input) - 1] < ' ') )
    {
        input[strlen((const char *)input) - 1] = '\0';  /* remove any trailing non-printable characters */
    }

    /* check for a QUIT command */
    if (!strncmp((char *)input, "quit", 4))
    {
        exit(0);
    }

    /* "q" quits too!! */
    if ((strncmp((char *)input, "q\n", 2) == 0) ||
        (strcmp((char *)input, "q") == 0))
    {
        exit(0);
    }

    if (cpiHandle == CPI_INVALID_HANDLE)
    {
        printf("\nBoard removed!");
        return (SUCCESS);
    }

    /* set up a packet for the board */
    len = strlen((char *)input) + 1;
    *(input + len - strlen("\n")) = '\r';
    hp = (CPIHDR *) & inbuf;
    hp->src = PORT(0, chan);
    hp->dst = PORT(board, chan);
    hp->length = len + sizeof(CPIHDR);

    /* send it off */
    if ((ret = cpi_send(cpiHandle, &inbuf)) == CPI_ERROR)
    {
        printf("Send Error (%d)\n", cpi_get_last_error( ));
        return ((DWORD) CPI_ERROR);
    }
    return (SUCCESS);
}                                   /* GetCommand */

/*****************************************************************************
 * HmiHandler
 *****************************************************************************/
short HmiHandler(CTAQUEUEHD hmiQueue)
{
    CTA_EVENT       ctaEvent = { 0 };
    DWORD           ret;
    char            tmpbuf[80];
    short           result = 0;

    while (result == 0)
    {
        ctaWaitEvent(hmiQueue, &ctaEvent, CTA_WAIT_FOREVER);

        if (ctaEvent.value != board)
        {
            continue;
        }
        switch (ctaEvent.id)
        {
            case HMI_EVN_STOP:
                {
                    if (cpiHandle != CPI_INVALID_HANDLE)
                    {
                        /*
                         * DO NOT CHANGE THE ORDER OF THE CODE IN THIS BLOCK !!!!!
                         * doing so will have the effect of causing CT Access to
                         * not really allow further registrations of wait objects
                         * (the call to ctaRegisterWaitObject will succeed, but
                         * the object never gets signaled!)
                         */
                        cpi_close(cpiHandle);
                        if ((ret =
                             ctaUnregisterWaitObject(hmiQueue,
                                                     &cpiWaitObj)) != SUCCESS)
                        {
                            ctaGetText(NULL_CTAHD, ret, tmpbuf,
                                       sizeof(tmpbuf));
                            fprintf(stderr,
                                    "ctaUnregisterWaitObject( ) failed: %s\n",
                                    tmpbuf);
                        }
                        cpiHandle = CPI_INVALID_HANDLE;
                        printf("\nBoard removed!");
                    }
                    break;
                }
            case HMI_EVN_INSERT:
            case HMI_EVN_HALTED:
            case HMI_EVN_NOWPRIMARY:
            case HMI_EVN_NOWBACKUP:
            case HMI_EVN_NOWSTANDALONE:
            case HMI_EVN_TASKDEAD:
                {
                    if (cpiHandle == CPI_INVALID_HANDLE)
                    {
                        short           port = PORT(board, chan);

                        if ((cpiHandle =
                             cpi_open(port, CPIM_PORT,
                                      NULL)) == CPI_INVALID_HANDLE)
                        {
                            printf("cpcon : could not open driver %d\n",
                                   cpi_get_last_error( ));
                        }
                        else
                        {
                            printf("\nBoard inserted!");
#ifdef WIN32
                            cpiWaitObj = cpi_wait_obj(cpiHandle);
#else
                            cpiWaitObj.fd = cpi_wait_obj(cpiHandle);
                            cpiWaitObj.events = (POLLIN | POLLPRI);
                            cpiWaitObj.revents = 0;
#endif
                            if ((ret = ctaRegisterWaitObject(hmiQueue, &cpiWaitObj,
                                                             GetResponse,
                                                             &cpiHandle,
                                                             CTA_PRIORITY_HIGH)) != SUCCESS)
                            {
                                ctaGetText(NULL_CTAHD, ret, tmpbuf, sizeof(tmpbuf));
                                fprintf(stderr,
                                        "ctaRegisterWaitObject( ) failed: %s\n",
                                        tmpbuf);
                            }
                        }
                    }
                }
        }
    }
    return (result);
}

/*****************************************************************************
 * InitHMI
 *****************************************************************************/
CTAQUEUEHD InitHMI( void )
{
    DWORD           ret;
    CTAQUEUEHD      hmiQueue = 0;   /* controller FSM queue handle */
    CTAHD           hmiHndl = NULL_CTAHD;   /* context handle for async events */
    char            tmpbuf[80];
    CTA_INIT_PARMS  hmiInitparms = { 0 };
    CTA_EVENT       ctaEvent = { 0 };

    /* Initialize CT Access health management (HMI) service */
    /* hmiInitparms.traceflags = CTA_TRACE_ENABLE; */ /* enable event tracing */
    hmiInitparms.size = sizeof(CTA_INIT_PARMS);
    hmiInitparms.ctacompatlevel = CTA_COMPATLEVEL;

    if ((ret = ctaInitialize(HmiServiceNames, 1, &hmiInitparms)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, tmpbuf, sizeof(tmpbuf));
        fprintf(stderr,
                "cpcon terminating, error initializing CT Access: %s\n",
                tmpbuf);
        return NULL_CTAQUEUEHD;
    }

    /* create the CT Access queue used to receive events from HMI */
    if ((ret = ctaCreateQueue(NULL, 0, &hmiQueue)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, tmpbuf, sizeof(tmpbuf));
        fprintf(stderr,
                "cpcon terminating, error creating CT Access queue: %s\n",
                tmpbuf);
        return NULL_CTAQUEUEHD;
    }

    /* create the controller FSM CT Access context for HMI async events
     * and open the HMI async events service on that context
     */
    if ((ret = ctaCreateContext(hmiQueue, 0, "DEMO", &hmiHndl)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, tmpbuf, sizeof(tmpbuf));
        fprintf(stderr,
                "cpcon terminating, error creating CT Access context: %s\n",
                tmpbuf);
        return NULL_CTAQUEUEHD;
    }

    /* open the specified HMI service on the context just created */
    HmiOpenSvcLst[0].svcargs.args[0] = 0;
    HmiOpenSvcLst[0].svcargs.args[1] = HMI_EVENTS_ALL_BOARDS;
    HmiOpenSvcLst[0].svcargs.args[2] = HMI_RCV_EVENTS;

    if ((ret = ctaOpenServices(hmiHndl, &HmiOpenSvcLst[0], 1)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, tmpbuf, sizeof(tmpbuf));
        fprintf(stderr,
                "Error [%s] opening HMI service, hot swap functionality disabled\n",
                tmpbuf);
    }
    else if ((ret =
              ctaWaitEvent(hmiQueue, &ctaEvent, CTA_WAIT_FOREVER)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, tmpbuf, sizeof(tmpbuf));
        fprintf(stderr,
                "Error [%s] opening HMI service, hot swap functionality disabled\n",
                tmpbuf);
    }
    else if ((ctaEvent.id != CTAEVN_OPEN_SERVICES_DONE) ||
             (ctaEvent.value != CTA_REASON_FINISHED))
    {
        ctaGetText(NULL_CTAHD, ret, tmpbuf, sizeof(tmpbuf));
        fprintf(stderr,
                "Error [%s] opening HMI service, hot swap functionality disabled\n",
                tmpbuf);
    }

    return (hmiQueue);
}

/*****************************************************************************
 * Main Console
 *****************************************************************************/
int main( int argc, char **argv )
{
    short           i;
    char           *myname;
    char           *str;
    char            sbuf[CPSTRSIZE], *vstr = sbuf;  /* dummy for old cpi_init */
    short           port;
    CTAQUEUEHD      hmiQueue = 0;   /* controller FSM queue handle */
    CTA_EVENT       ctaEvent = { 0 };
    DWORD           ret;
    char            tmpbuf[80];

#ifdef UNIX
    struct pollfd   stdInPoll;
#endif

    printf( "Console program V%d.%d : [Hit Return to Activate, \'quit\' to Exit]\n", MH_VERSION, MH_REVISION);
    printf( "(c)Copyright 1998-2009 Dialogic Corporation.  All Rights Reserved\n");

    /* Open the TX board indicated by command line arguments */
    myname = argv[0];
    i = 1;
    while (i < argc)
    {
        str = argv[i];
        if (*str == '-')
        {
            str++;
            switch (*str)
            {
                case 'b':          /* which board should we speak to */
                case 'B':
                    str++;
                    if (isdigit(*str))
                        board = atoi(str);
                    else
                    {
                        i++;
                        if ( (i < argc) && (isdigit(*argv[i])) )
                            board = atoi(argv[i]);
                        else
                        {
                            printf("\nError: expecting board number after -b option.\n");
                            exit(1);
                        }
                    }
                    if ((board < 1) || (board > MAXBOARD))
                    {
                        printf("Illegal board number!!\n");
                        printf(usagestr, myname);
                        exit(1);
                    }
                    break;

                case 'c':          /* which channel should we speak through */
                case 'C':
                    str++;
                    if (isdigit(*str))
                    {
                        chan = atoi(str);
                    }
                    else
                    {
                        i++;
                        if ( (i < argc) && (isdigit(*argv[i])) )
                        {
                            chan = atoi(argv[i]);
                        }
                        else
                        {
                            printf("\nError: expecting channel number after -c option.\n");
                            exit(1);
                        }
                    }
                    if ((chan < 1) || (chan > MAXCHAN))
                    {
                        printf("Illegal channel number!!\n");
                        printf(usagestr, myname);
                        exit(1);
                    }
                    break;

                default:
                    printf(usagestr, myname);
                    exit(1);
            }                       /* switch (*++str) */
        }                           /* if (*str == '-') */
        else
        {
            printf(usagestr, myname);
            exit(1);
        }
        ++i;
    }                               /* while i < argc */

    if ((hmiQueue = InitHMI( )) == NULL_CTAQUEUEHD)
    {
        exit(1);
    }

    cpi_init(0, &vstr);

    port = PORT((short)board, (short)chan);
    if ((cpiHandle = cpi_open(port, CPIM_PORT, NULL)) == CPI_INVALID_HANDLE)
    {
        printf("%s : could not open driver %d\n", argv[0],
               cpi_get_last_error( ));
    }
    else
    {
#ifdef WIN32
        cpiWaitObj = cpi_wait_obj(cpiHandle);
#else
        cpiWaitObj.fd = cpi_wait_obj(cpiHandle);
        cpiWaitObj.events = (POLLIN | POLLPRI);
        cpiWaitObj.revents = 0;
#endif

        if ((ret =
             ctaRegisterWaitObject(hmiQueue, &cpiWaitObj, GetResponse,
                                   &cpiHandle,
                                   CTA_PRIORITY_NORMAL)) != SUCCESS)
        {
            ctaGetText(NULL_CTAHD, ret, tmpbuf, sizeof(tmpbuf));
            fprintf(stderr, "ctaRegisterWaitObject( ) failed: %s\n", tmpbuf);
        }
    }

#ifdef WIN32
    stdInput = GetStdHandle(STD_INPUT_HANDLE);
    SetConsoleMode(stdInput,
                   ENABLE_LINE_INPUT | ENABLE_ECHO_INPUT |
                   ENABLE_PROCESSED_INPUT);
    ctaRegisterWaitObject(hmiQueue, &stdInput, GetCommand,
                          (void *)CPI_INVALID_HANDLE, CTA_PRIORITY_NORMAL);
#else
    stdInPoll.fd = 0;               /* This is stdin */
    stdInPoll.events = (POLLIN | POLLPRI);
    stdInPoll.revents = 0;
    ctaRegisterWaitObject(hmiQueue, &stdInPoll, GetCommand,
                          (void *)CPI_INVALID_HANDLE, CTA_PRIORITY_NORMAL);

#endif

    HmiHandler(hmiQueue);
    i = cpi_close(cpiHandle);

    return (0);
}

