/*****************************************************************************
 *
 * ss7trace.c - SS7 line-monitor like trace tool
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                         PROPRIETARY NOTICE
 *
 *  (c)Copyright 1988-2009 Dialogic Corporation.  All Rights Reserved
 *
 *  The following is licensed program material and is considered
 *  proprietary and confidential.  No part of this program may be
 *  reproduced or used in any form or by any means without the express
 *  written permission of Dialogic Corporation.
 *
 *****************************************************************************/
/* TXxxxx trace capturing daemon */

#ifdef UNIX
#include <stdio.h>
#include <stddef.h>
#include <sys/types.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
/* #include <sys/stream.h> */
#endif	                                /* UNIX */

#ifdef __OS2__
#include <stdio.h>
#include <stddef.h>
#include <ctype.h>
#include <string.h>
#endif	                                /* __OS2__ */

#ifdef WIN32
#include <stdio.h>
#include <stddef.h>
#include <string.h>
#endif                                  /* WIN32 */

#include <time.h>

#define _TX_ISUPMGMT_SOURCE_

#include "ss7type.h"
#include "txcpi.h"
#include "dpruser.h"
#include "dpriface.h"

#include "tx_core.h"
#include "ss7trace.h"
#include "txalarm.h"

#define  DEFAULTTRACECHAN 0x2f

static char *usagestr = "Usage: %s [-c chan] [-f traceFile]\n";
extern CPI_WAIT_TYPE stdInput;

FILE *fileDesc = NULL;

U32 eth_mon_lost = 0;           /* last reported number of Ethernet trace packets that were lost (not delivered to host app) */

/* Macros to control byte swabbing depending on whether source is host or a board */
#define CPI_HTOCP_L(board, x) ((board != 0) ? cpi_htocp_l(x) : x)
#define CPI_HTOCP_S(board, x) ((board != 0) ? cpi_htocp_s(x) : x)

/****************************************************************************
 * output_flush - flush all active outputs
 ****************************************************************************/
void output_flush( void )
{
    fflush( stdout );
}

/****************************************************************************
 * output_line - send output line to screen and optionally to file
 ****************************************************************************/
void output_line( char *outLine )
{
    fprintf( stdout, outLine );

    output_flush( );
}

/*****************************************************************************
 * hexdump
 *****************************************************************************/
void hexdump( char *in, char *out, short length )
{
    int   i;

    for (i = 0; i < length; i++)
    {
        out += sprintf( out, "%02X ", (unsigned char)in[i] );
    }

    for (; i < 16; i++)
    {
        out += sprintf( out, "   ", (unsigned char)in[i] );
    }

    for (i = 0; i < length; i++)
    {
        if (isprint(in[i]))
        {
            out += sprintf( out, "%c", (unsigned char)in[i] );
        }
        else
        {
            out += sprintf( out, ".", (unsigned char)in[i] );
        }
    }

    for (; i < 16; i++)
        out += sprintf(out, " ", (unsigned char)in[i]);
}

/*****************************************************************************
 * dumpHex
 *****************************************************************************/
int dumpHex( char *buffer, short len )
{
    char line[100];

    output_flush( );

    do
    {
        hexdump( buffer, line, (short)((len >= 16) ? 16 : (len % 16)) );

        output_line( "\n" );
        output_line( line );
        buffer += 16;
    } while ((len -= 16) > 0);

    output_line( "\n" );

    return( 0 );
}

/*****************************************************************************
 * ExitRoutine
 *****************************************************************************/
short ExitRoutine(TX_HANDLE handle, short events)
{
	char c;

	/* check if input event really did occur */
	if (tx_core_process_events(TX_CORE_INPUT, events) == FALSE)
		return (TX_CORE_SUCCESS);

	/* make sure a key was hit */
    if (tx_core_kbhit())
    {
		c = getchar();
		return (TX_CORE_EXIT_POLL_LOOP);
	}

	return (TX_CORE_SUCCESS);
}

/*****************************************************************************
 * ProcessLogHdr - process received log message header
 *
 * Inputs: pkt        - CPI packet holding received log message
 *         outline    - address where output line (or error message) stored
 *         timnestamp - address of timestamp record to be filled in
 *
 * Output: log message pointer (if successfully processed) [outline not used]
 *         or NULL if error detected [outline holds error message]
 *****************************************************************************/
TXLOG_MSG *ProcessLogHdr( CPIPKT *pkt, char *outline, struct tm *timestamp )
{
    TXLOG_MSG  *msg;
    time_t      secs_epoch;
    struct tm   ts_epoch;

    /* ----- perform all required field swabbing ----- */
    msg = (TXLOG_MSG *)pkt->data;
    msg->hdr.secs_epoch = cpi_cptoh_l( msg->hdr.secs_epoch );   /* seconds since epoch - log record timestamp */
    msg->hdr.logs_lost = cpi_cptoh_l( msg->hdr.logs_lost );     /* count of log records lost [not delivered to receiver] */

    /* ----- compute timestamp of log message ----- */
    /* NOTE: The TX board sends time by providing seconds since epoch.
     *       These seconds are used by the "localtime" function to calculate the local time.
     */
    secs_epoch = (time_t)msg->hdr.secs_epoch;   /* get local copy so represented as valid native time_t */
    ts_epoch = *localtime( &secs_epoch );       /* call the library function to calculate local time */

    if (timestamp != NULL)
    {
        memcpy( (void *)timestamp, (void *)&ts_epoch, sizeof(struct tm) );
    }

    return( msg );
}

/*****************************************************************************
 * ProcessTrace - process received trace message
 *
 * Inputs: pkt       - CPI packet holding received trace data
 *         outLine   - address where output line (or error message) stored
 *         timestamp - timestamp associated with received trace
 *
 * Output: trace message [outLine holds formatted header string]
 *         or NULL if error detected [outLine holds error message]
 *****************************************************************************/
TXLOG_MSG *ProcessTrace( CPIPKT *pkt, char *outLine, struct tm *timestamp )
{
    TXLOG_MSG  *msg = (TXLOG_MSG *)pkt->data;
    U16         source_board;
    U32         trace_instance;
    S8          trace_dir[32];

    /* ----- verify size of packet is big enough to hold trace message ----- */
    if (pkt->hdr.length < (sizeof(CPIHDR) + TXLOG_TRACE_MIN_SZ))
    {
        sprintf( outLine, "Message of %d bytes too small to be trace\n", pkt->hdr.length );

        return( NULL );
    }

    /* ----- determine TX board that was source of alarm ----- */
    source_board = BOARD(pkt->hdr.src);
    if (source_board > MAXBOARD)
    {
        sprintf( outLine, "Message from invalid board number: %d\n", source_board );

        return( NULL );
    }

    /* ----- output line indicating log messages lost (if applicable) ----- */
    if (msg->hdr.logs_lost != eth_mon_lost)
    {   /* number of Ethernet trace (host-channel monitor) messages reported as lost has changed */
        sprintf( outLine, "!!! %u messages reported as lost [prior lost count %u] !!!\n", msg->hdr.logs_lost, eth_mon_lost );
        output_line( outLine );             /* send output line to screen and optionally to file */
        eth_mon_lost = msg->hdr.logs_lost;
    }

    /* ----- perform all required TRACE-message field swabbing ----- */
    msg->log.trace.info.trace_id = cpi_cptoh_l( msg->log.trace.info.trace_id );         /* trace identifier */
    msg->log.trace.info.trace_type = cpi_cptoh_l( msg->log.trace.info.trace_type );     /* ID-specific type of trace record */
    msg->log.trace.info.trace_length = cpi_cptoh_l( msg->log.trace.info.trace_length ); /* byte length of trace data */

    /* ----- set trace type name, etc. ----- */
    switch( msg->log.trace.info.trace_id )
    {
        case TXLOG_ETH_RX_DATA:     /* Ethernet data packet received by the board */
            strcpy( trace_dir, "<--" );
            break;

        case TXLOG_ETH_TX_DATA:     /* Ethernet data packet transmitted by the board */
            strcpy( trace_dir, "-->" );
            break;

        default:
            sprintf( outLine, "Trace message with unknown trace ID: [0x%08X]\n", msg->log.trace.info.trace_id );

            return( NULL );
    }
    trace_instance = msg->log.trace.info.trace_type;            /* Ethernet interface number */

    /* ----- format the trace header output string ----- */
    sprintf( outLine, "\n%d:%02d:%02d.%d ETH[%u].%d %s (%u bytes):\n",
             timestamp->tm_hour,
             timestamp->tm_min,
             timestamp->tm_sec,
             0,
             trace_instance,
             source_board,
             trace_dir,
             msg->log.trace.info.trace_length );

    /* ----- output header information ----- */
    output_line( outLine );             /* send output line to screen and optionally to file */

    /* ----- output message payload ----- */
    dumpHex( (char *)msg->log.trace.data, (S16)msg->log.trace.info.trace_length );

    output_line( "\n" );                    /* issue record separator space */

    return( msg );
}

/*****************************************************************************
 * EthTrace - handle receive of packet from IP channel ($ipeth task)
 *
 * inputs: pkt - CPI packet just received from the IP channel
 *****************************************************************************/
void EthTrace( CPIPKT *inbuf )
{
    char        outLine[256];
    TXLOG_MSG  *msg;
    struct tm   timestamp;

    /* process received Ethernet trace packet */
    msg = ProcessLogHdr( inbuf, outLine, &timestamp );
    if (msg != NULL)
    {
        if (msg->hdr.log_type == TXLOG_TYPE_TRACE)
        {
            msg = ProcessTrace( inbuf, outLine, &timestamp );
        }
        else
        {
            sprintf( outLine, "ERROR: Unexpected log message type [0x%X] received\n", msg->hdr.log_type );
            msg = NULL;             /* indicate that message was NOT processed successfully */
        }
    }

    if (msg == NULL)
    {   /* message was not processed successfully so output error message */
        output_line( outLine );             /* send output line to screen and optionally to file */
    }
}

/*****************************************************************************
 * GetTrace
 *****************************************************************************/
short GetTrace(TX_HANDLE handle, short events)
{
    CPIPKT data;
    short  ret;
    short  len = CPIBUFSIZE;
    char   dir[32];
    short  length;
    U32    dpc;
    short  event;
    short  srcBoard;

    /* check if input event really did occur */
	if (tx_core_process_events(TX_CORE_INPUT, events) == FALSE)
    {
		return (TX_CORE_SUCCESS);
    }

    if ((ret = cpi_get_data(handle, (CPIPKT *)&data, &len)) == CPI_ERROR)
    {
		printf("Receive Error (%d)\n", cpi_get_last_error());
		return (TX_CORE_ERROR);
    }
    else
    {
        switch ( ((DPRH*)&data)->srcch)
        {
            case ISUP_ENT :                /* ISUP */
            {
                IsupMgmtReq *mgmt = (IsupMgmtReq*)&data;

                event = CPI_HTOCP_S( mgmt->dprh.srcbd, mgmt->req.trc.evnt );

                if (event == TRCMSGTX)
                    strcpy(dir, "-->");
                else if (event == TRCMSGRX)
                    strcpy(dir, "<--");
                else
                    strcpy(dir, "???");
                dpc = CPI_HTOCP_L( mgmt->dprh.srcbd, mgmt->req.trc.dpc );
                printf("\n%d:%02d:%02d.%d ISUP.%d %s %d.%d.%d:\n",
                                    mgmt->req.trc.dt.hour,
                                    mgmt->req.trc.dt.min,
                                    mgmt->req.trc.dt.sec,
                                    mgmt->req.trc.dt.tenths,
                                    mgmt->dprh.srcbd,
                                    dir,
                                    ((dpc >> 16) & 0xff),
                                    ((dpc >>  8) & 0xff),
                                    ((dpc >>  0) & 0xff));


                length = CPI_HTOCP_S( mgmt->dprh.srcbd, mgmt->req.trc.length );

                dumpHex((char *)mgmt->req.trc.evntParm, length);

                return (TX_CORE_SUCCESS);
            }
            case TCAP_ENT :                /* TCAP */
            {
                TcapMgmtTrace *mgmt = (TcapMgmtTrace*)&data;
                short sapNum;

                event = CPI_HTOCP_S( mgmt->dprh.srcbd, mgmt->req.trc.evnt );

                if (event == LST_MSG_TXED)
                    strcpy(dir, "-->");
                else if (event == LST_MSG_RECVD)
                    strcpy(dir, "<--");
                else
                    strcpy(dir, "???");

                sapNum = CPI_HTOCP_S( mgmt->dprh.srcbd, mgmt->hdr.elmId.elmntInst1 );

                printf("\n%d:%02d:%02d.%d TCAP.%d %s : sapNum = %d\n",
                                    mgmt->req.trc.dt.hour,
                                    mgmt->req.trc.dt.min,
                                    mgmt->req.trc.dt.sec,
                                    mgmt->req.trc.dt.tenths,
                                    mgmt->dprh.srcbd,
                                    dir,
                                    sapNum);

                length = CPI_HTOCP_S( mgmt->dprh.srcbd, mgmt->req.trc.len );
                dumpHex((char *)mgmt->req.trc.evntParm, length);

                return (TX_CORE_SUCCESS);
            }
            case SCCP_ENT :                /* SCCP */
            {
                SccpMgmtTrace *mgmt = (SccpMgmtTrace*)&data;
                short sapNum;

                event = CPI_HTOCP_S( mgmt->dprh.srcbd, mgmt->req.trc.evnt );

                if (event == LSP_MSG_TXED)
                    strcpy(dir, "-->");
                else if (event == LSP_MSG_RECVD)
                    strcpy(dir, "<--");
                else
                    strcpy(dir, "???");

                sapNum = CPI_HTOCP_S( mgmt->dprh.srcbd, mgmt->hdr.elmId.elmntInst1 );

                printf("\n%d:%02d:%02d.%d SCCP.%d %s : sapNum = %d\n",
                                    mgmt->req.trc.dt.hour,
                                    mgmt->req.trc.dt.min,
                                    mgmt->req.trc.dt.sec,
                                    mgmt->req.trc.dt.tenths,
                                    mgmt->dprh.srcbd,
                                    dir,
                                    sapNum);

                length = CPI_HTOCP_S( mgmt->dprh.srcbd, mgmt->req.trc.len );
                dumpHex((char *)mgmt->req.trc.evntParm, length);

                return (TX_CORE_SUCCESS);
            }
            case MTP3_ENT :                /* MTP3 */
            {
                mtp3MgmtTrace *mgmt = (mtp3MgmtTrace *)&data;
                short sapNum;

                short srcEnt = CPI_HTOCP_S( mgmt->dprh.srcbd, mgmt->hdr.elmId.elmntInst3 );

                if ( srcEnt == M3UA_ENT )
                {
                    M3UAMgmtTrace *m3uaMgmt = (M3UAMgmtTrace*)&data;
                    event = CPI_HTOCP_S( m3uaMgmt->dprh.srcbd, m3uaMgmt->req.trc.evnt );
		    srcBoard = m3uaMgmt->dprh.srcbd;
                    switch(event)
                    {
                        case LIT_MSG_RX:
                            strcpy(dir, "<--MSG--");
                        break;
                        case LIT_MSG_TX:
                            strcpy(dir, "--MSG-->");
                        case LIT_MGMT_RX:
                        strcpy(dir, "<--MGMT--");
                        break;
                        case LIT_DATA_RX:
                            strcpy(dir, "<--DATA--");
                        break;
                        case LIT_SSNM_RX:
                            strcpy(dir, "<--SSNM--");
                        break;
                        case LIT_ASPSM_RX:
                            strcpy(dir, "<--ASPSM--");
                        break;
                        case LIT_ASPTM_RX:
                            strcpy(dir, "<--ASPTM--");
                        break;
                        case LIT_RKM_RX:
                            strcpy(dir, "<--RKM--");
                        break;
                        case LIT_MGMT_TX:
                            strcpy(dir, "--MGMT-->");
                        case LIT_DATA_TX:
                            strcpy(dir, "--DATA-->");
                        break;
                        case LIT_SSNM_TX:
                            strcpy(dir, "--SSNM-->");
                        break;
                        case LIT_ASPSM_TX:
                            strcpy(dir, "--ASPSM-->");
                        break;
                        case LIT_ASPTM_TX:
                            strcpy(dir, "--ASPTM-->");
                        break;
                        case LIT_RKM_TX:
                            strcpy(dir, "--RKM-->");
                        break;
                        default:
                            strcpy(dir, "???");
                    }

                    sapNum = CPI_HTOCP_S(/*m3uaMgmt->dprh.srcbd*/ srcBoard, m3uaMgmt->req.trc.suId );
                    printf("\n%d:%02d:%02d.%d M3UA.%d %s : Service User: %d\n",
                                    m3uaMgmt->req.trc.dt.hour,
                                    m3uaMgmt->req.trc.dt.min,
                                    m3uaMgmt->req.trc.dt.sec,
                                    m3uaMgmt->req.trc.dt.tenths,
                                    /*m3uaMgmt->dprh.srcbd*/ srcBoard,
                                    dir,
                                    sapNum);

                    length = CPI_HTOCP_S( /*m3uaMgmt->dprh.srcbd*/ srcBoard, m3uaMgmt->req.trc.len );
                    dumpHex((char *)&(m3uaMgmt->req.trc.evntParm[0]), length);
                }
                else
                {
                    event = CPI_HTOCP_S( mgmt->dprh.srcbd, mgmt->req.trc.evnt );
                    if (event == LSN_MSG_TX)
                        strcpy(dir, "-->");
                    else if (event == LSN_MSG_RX)
                        strcpy(dir, "<--");
                    else
                        strcpy(dir, "???");
                    sapNum = CPI_HTOCP_S( mgmt->dprh.srcbd, mgmt->hdr.elmId.elmntInst1 );

                    printf("\n%d:%02d:%02d.%d MTP3.%d %s : Link # %d\n",
                                    mgmt->req.trc.dt.hour,
                                    mgmt->req.trc.dt.min,
                                    mgmt->req.trc.dt.sec,
                                    mgmt->req.trc.dt.tenths,
                                    mgmt->dprh.srcbd,
                                    dir,
                                    sapNum);

                    length = CPI_HTOCP_S( mgmt->dprh.srcbd, mgmt->req.trc.len );
                    dumpHex((char *)&(mgmt->req.trc.evntParm[0]), length);
                }
                return (TX_CORE_SUCCESS);
            }
            case TUP_ENT :                /* TUP */
            {
                TupMgmtTrace *mgmt = (TupMgmtTrace *)&data;

                event = CPI_HTOCP_S( mgmt->dpr.srcbd, mgmt->data.trc.evnt );

                if (event == TRCMSGTX)
                    strcpy(dir, "-->");
                else if (event == TRCMSGRX)
                    strcpy(dir, "<--");
                else
                    strcpy(dir, "???");

                dpc = CPI_HTOCP_L( mgmt->dpr.srcbd, mgmt->data.trc.dpc );
                printf("\n%d:%02d:%02d.%d TUP.%d %s %d.%d.%d:\n",
                                    mgmt->data.trc.dt.hour,
                                    mgmt->data.trc.dt.min,
                                    mgmt->data.trc.dt.sec,
                                    mgmt->data.trc.dt.tenths,
                                    mgmt->dpr.srcbd,
                                    dir,
                                    ((dpc >> 16) & 0xff),
                                    ((dpc >>  8) & 0xff),
                                    ((dpc >>  0) & 0xff));

                length = CPI_HTOCP_S( mgmt->dpr.srcbd, mgmt->data.trc.length );

                dumpHex((char *)mgmt->data.trc.evntParm, length);

                return (TX_CORE_SUCCESS);
            }
            case IUP_ENT :                /* IUP */
            {
                IupMgmtTrace *mgmt = (IupMgmtTrace *)&data;

               /* printf("event=%d\n", mgmt->trc.evnt); */
                event = CPI_HTOCP_S( mgmt->hdr.dpr.srcbd, mgmt->trc.evnt);

                /* printf("event=%d\n", event); */
                if (event == TRCMSGTX)
                    strcpy(dir, "-->");
                else if (event == TRCMSGRX)
                    strcpy(dir, "<--");
                else
                    strcpy(dir, "???");

                dpc = CPI_HTOCP_L( mgmt->hdr.dpr.srcbd, mgmt->trc.dpc );
                printf("\n%d:%02d:%02d.%d IUP.%d %s %d.%d.%d:\n",
                                    mgmt->trc.dt.hour,
                                    mgmt->trc.dt.min,
                                    mgmt->trc.dt.sec,
                                    mgmt->trc.dt.tenths,
                                    mgmt->hdr.dpr.srcbd,
                                    dir,
                                    ((dpc >> 16) & 0xff),
                                    ((dpc >>  8) & 0xff),
                                    ((dpc >>  0) & 0xff));

                length = CPI_HTOCP_S( mgmt->hdr.dpr.srcbd, mgmt->trc.length );

                dumpHex((char *)mgmt->trc.evntParm, length);

                return (TX_CORE_SUCCESS);
            }
            case SCTP_ENT:
            {
                SCTPMgmtTrace *mgmt = (SCTPMgmtTrace *)&data;
                S16 sapId;
                event = CPI_HTOCP_S( mgmt->dprh.srcbd, mgmt->req.trc.evnt );
                if (event == LSB_MSG_TXED)
                    strcpy(dir, "-->");
                else if (event == LSB_MSG_RECVD)
                    strcpy(dir, "<--");
                else
                    strcpy(dir, "???");

                sapId = CPI_HTOCP_S( mgmt->dprh.srcbd, mgmt->req.trc.sapId );
                printf("\n%d:%02d:%02d.%d SCTP.%d %s Sap ID: %u\n",
                                    mgmt->req.trc.dt.hour,
                                    mgmt->req.trc.dt.min,
                                    mgmt->req.trc.dt.sec,
                                    mgmt->req.trc.dt.tenths,
                                    mgmt->dprh.srcbd,
                                    dir,
                                    sapId);

                length = CPI_HTOCP_S( mgmt->dprh.srcbd, mgmt->req.trc.len );

                dumpHex((char *)mgmt->req.trc.evntParm, length);

                return (TX_CORE_SUCCESS);
            }

            case PT_IP:             /* IP Control Channel */
                EthTrace( &data );
                return (TX_CORE_SUCCESS);
                break;

            default:
                return (TX_CORE_SUCCESS);
        }
	}
}

/*****************************************************************************
 * main
 *****************************************************************************/
int main( int argc, char* argv[] )
{
    S32 cps[MAXBOARD + 1];
	TX_HANDLE fds[MAXBOARD + 1];
	short i;
    short chan = DEFAULTTRACECHAN;
	char *myname;
	char *str = NULL;
	char sbuf[CPSTRSIZE], *vstr = sbuf; /* dummy for old cpi_init */
	short port;
	char fileName[FILENAME_MAX];

    printf("Trace Monitor SS7 V5.1: Hit Enter to exit\n");
    printf("(c)Copyright 1988-2009, Dialogic Corporation\n");

	myname = argv[0];

	i = 1;
    while (i < argc)
    {
		str = argv[i];
        if (*str == '-')
        {
            switch (*++str)
            {
                case 'c':   /* which channel should we speak through */
                    if (isdigit( *(str + 1) ))
                    {
                        chan = atoi( ++str );
                    }
                    else
                    {
                        i++;
                        if (i < argc)
                        {
                            chan = atoi( argv[i] );
                        }
                        else
                        {
                            chan = 0;
                        }
                    }
                    break;

                case 'f':
                    str++;
                    if (*str == '\0')
                    {
                        i++;
                        if (i < argc)
                        {
                            strcpy( fileName, argv[i] );
                        }
                        else
                        {
                            printf( "Output filename is missing!\n" );
                            printf( usagestr, myname );
                            exit( 1 );
                        }
                    }
                    else
                    {
                        strcpy( fileName, str );
                    }

                    fileDesc = freopen( fileName, "w", stdout );
                    if (!fileDesc)
                    {
                        printf("Cannot open file %s\n", fileName);
                        printf(usagestr, myname);
                        exit(1);
                    }
                    break;

                default:
                    printf(usagestr, myname);
                    exit(1);
			}
		}
		++i;
	}

    if (chan < 1)
    {
		printf(usagestr, argv[0]);
		exit(2);
	}

    /* Process console data */
    if (tx_core_init() != TX_CORE_SUCCESS)
    {
		printf("Could not initialize core routines\n");
		exit(1);
	}

	cpi_init(0, &vstr);

	cpi_get_resources(MAXBOARD, cps);

    for (i = 0; i <= MAXBOARD; ++i)
    {
        if (cps[i])
        {
			port = PORT(i,chan);
            if ((fds[i] = cpi_open(port, CPIM_PORT, NULL)) == CPI_INVALID_HANDLE)
            {
                printf("%s : could not open board %d driver %x\n", argv[0], i, cpi_get_last_error());
            }
            else
            {
                tx_core_install_handler(cpi_wait_obj(fds[i]), fds[i], GetTrace);
				tx_core_set_poll(cpi_wait_obj(fds[i]), TX_CORE_INPUT);
			}
		}
	}

	tx_core_install_handler(stdInput, CPI_INVALID_HANDLE, ExitRoutine);
	tx_core_set_poll(stdInput, TX_CORE_INPUT);

	tx_core_poll_loop(TX_CORE_INFINITE);

	/* close all the handles */
    for (i = 0; i <= MAXBOARD; ++i)
    {
		if (fds[i])
        {
			cpi_close(fds[i]);
        }
    }

	if (fileDesc)
    {
		fclose(fileDesc);
        fileDesc = NULL;
    }

	return (0);
}

