/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**  (c)Copyright 2001-2009 Dialogic Corporation.  All Rights Reserved   **
**                                                                      **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Dialogic Corporation.                         **
**                                                                      **
*************************************************************************/

/*************************************************************************
**                                                                      **
**    ISUP trace definition. See file: isupmgmt.h & apidef.h            **
**                                                                      **
*************************************************************************/

typedef struct _entityId           /* entity id */

{
   U8 ent;                       /* entity */
   U8 inst;                     /* instance */
   U16 spare1;
} EntityId;

typedef struct _elmntId            /* element id */

{
   S16   elmnt;                   /* element */
   S16   elmntInst1;              /* element instance 1 */
   S16   elmntInst2;              /* element instance 2 */
   S16   elmntInst3;              /* element instance 3 */
} ElmntId;

typedef struct _mgmtHeader       /* header */

{
   U16 msgLen;                  /* message length   - optional */
   U8 msgType;                  /* message type     - mandatory */
   U8 version;                  /* version          - optional */
   U16 seqNmb;                  /* sequence number  - optional */
   U16 spare1;
   EntityId entId;              /* entity id        - mandatory */
   ElmntId elmId;               /* element id       - mandatory */
} MgmtHeader;

typedef struct _IsupTrc
{
    DateTime dt;
    U16 evnt;
    U8 sapType;
    U8 fill1;
    U32 dpc;
    S16 length;
    S16 fill2;
    U8 evntParm[280];
} IsupTrc;                    /* trace */

#define TRCMSGTX         11          /* trace - message transmitted */
#define TRCMSGRX         12          /* trace- message received */

typedef struct _IsupMgmtReq
{
  DPRH          dprh;
  S16           start;
  S16           end;
  U8            event;
  U8            fill1;
  U16           fill2;
  MgmtHeader    hdr;

  union
  {
    IsupTrc         trc;
  } req;
}IsupMgmtReq;

/*************************************************************************
**                                                                      **
**    TCAP trace definition                                             **
**                                                                      **
*************************************************************************/

typedef struct _TcapTrc             /* See File: lst.x : struct stMngmt */
{
     DateTime dt;                   /* date and time */
     U16      evnt;                 /* event */
     U16      len;                  /* trace length */
     U8       evntParm[280];        /* event parameters */
} TcapTrc;

typedef struct _TcapMgmtTrace
{
  DPRH          dprh;
  S16           start;
  S16           end;
  U8            event;
  U8            fill;
  U8            fill2;
  U8            fill3;
  MgmtHeader    hdr;

  union
  {
    TcapTrc     trc;
  } req;

} TcapMgmtTrace;

#define LST_MSG_RECVD      1    /* Message received    File: lst.h */
#define LST_MSG_TXED       2    /* Message transmitted File: lst.h */

/*************************************************************************
**                                                                      **
**    SCCP trace definition                                             **
**                                                                      **
*************************************************************************/

typedef struct _SccpTrc             /* See File: lsp.x : struct spMngmt */
{
     DateTime dt;                   /* date and time */
     U16      evnt;                 /* event */
     U16      len;                  /* trace length */
     U8       evntParm[280];        /* event parameters */
} SccpTrc;

typedef struct _SccpMgmtTrace
{
  DPRH          dprh;
  S16           start;
  S16           end;
  U8            event;
  U8            fill;
  U8            fill2;
  U8            fill3;
  MgmtHeader    hdr;

  union
  {
    SccpTrc     trc;
  } req;

} SccpMgmtTrace;

#define LSP_MSG_RECVD      1    /* Message received    File: lsp.h */
#define LSP_MSG_TXED       2    /* Message transmitted File: lsp.h */

/*************************************************************************
**                                                                      **
**    MTP3 trace definition                                             **
**                                                                      **
*************************************************************************/

typedef struct _mtp3Trc             /* See File: lsn.x : struct stMngmt */
{
         DateTime dt;            /* date and time */
         U16      evnt;          /* event */
         U16      len;                  /* trace length */
         U8       evntParm[280];  /* event parameters (Maximum packet is 273 with SIO byte but length is in 1st byte only */
} mtp3Trc;

typedef struct _mtp3MgmtTrace
{
  DPRH          dprh;
  S16           start;
  S16           end;
  U8            event;
  U8            fill;
  U8            fill2;
  U8            fill3;
  MgmtHeader    hdr;

  union
  {
         mtp3Trc     trc;
  } req;

} mtp3MgmtTrace;

/*************************************************************************
**                                                                      **
**    IUP trace definition                                             **
**                                                                      **
*************************************************************************/
typedef struct _iupHdrM {
    DPRH    dpr;
    U16     start;
    U16     end;
    U8 event;
    U8 spare1;                          /* Spare for alignment */
    U8 spare2;                          /* Spare for alignment */
    U8 spare3;                          /* Spare for alignment */
    MgmtHeader    mgmthdr;
    U32     id;

#define SDUID_      0x21534455      /* !SDU */
#define MDUID_      0x214d4455      /* !MDU */
#define PDUID_      0x21504455      /* !PDU */
    void   *flink;
    void   *blink;
    S16     ccode;
    S16     fill1;
    U32     rvalu;
} IupHdrM;

typedef struct _IUPTrc
{
    DateTime dt;
    U16 evnt;
    /*U16 spare1;*/
    U8 sapType;
    U8 fill1;                            /* Spare for alignment */
    U32 dpc;
    S16 length;
    S16 fill4;                          /* Spare for alignment */
    U8 evntParm[64];
} IUPTrc;                    /* trace */

/* Management Data Unit structure */
typedef struct _iupMgmtTrace
{
    IupHdrM  hdr;
    U16     sapId;
    U16     groupId;
    U32     circId;
    U16     circCnt;
    /*U16     fill1;*/
    U8      evntType;
    U8      entType;
    IUPTrc         trc;
} IupMgmtTrace;

/*************************************************************************
**                                                                      **
**    TUP trace definition                                             **
**                                                                      **
*************************************************************************/
typedef struct _TUPTrace
{
    DateTime dt;
    U16 evnt;
    U8 sapType;
    U8 fill;                            /* Spare for alignment */
    U32 dpc;
    S16 length;
    S16 fill1;
#ifndef MAX_LSI_TRC
#define MAX_LSI_TRC      280
#endif                              /* Spare for alignment */
    U8 evntParm[MAX_LSI_TRC];
} TUPTrace;                    /* trace */


typedef struct _tupMgmtTrace
{
    DPRH    dpr;
    U16     start;
    U16     end;
    U8 event;
    U8 spare1;                          /* Spare for alignment */
    U8 spare2;                          /* Spare for alignment */
    U8 spare3;                          /* Spare for alignment */
    U32     id;
  #ifndef SDUID_
  #define SDUID_      0x21534455      /* !SDU */
  #endif

  #ifndef MDUID_
  #define MDUID_      0x214d4455      /* !MDU */
  #endif

  #ifndef PDUID_
  #define PDUID_      0x21504455      /* !PDU */
  #endif
    S16     ccode;
    S16     fill1;                      /* Spare for alignment */
    U32     rvalu;
    U16     sapId;
    U16     groupId;
    U32     circId;
    U16     circCnt;
    U8      evntType;
    U8      entType;

    union
    {
      TUPTrace         trc;
    } data;
} TupMgmtTrace;

#define LSN_MSG_RX           0        /* event - message received */
#define LSN_MSG_TX           1        /* event - message transmitted */

/****************************************************************
    Sigtran (M3UA and SCTP) Trace
*****************************************************************/
/* Sigtran Mmgmt Header */
/* evnt in trc */
#define LIT_MSG_RX       1     /* Trace for the message received */
#define LIT_MSG_TX       2     /* Trace for the message transmitted */
#define LIT_MGMT_RX      3     /* Trace for the MGMT msg received */
#define LIT_DATA_RX      4     /* Trace for the M3UA DATA msg received */
#define LIT_SSNM_RX      5     /* Trace for the M3UA SSNM msg received */
#define LIT_ASPSM_RX     6     /* Trace for the M3UA ASPSM msg received */
#define LIT_ASPTM_RX     7     /* Trace for the M3UA ASPTM msg received */
#define LIT_RKM_RX       8     /* Trace for the M3UA RKM msg received */
#define LIT_MGMT_TX      9    /* Trace for Management */
#define LIT_DATA_TX      10   /* Trace for M3UA data transfer message class */
#define LIT_SSNM_TX      11   /* Trace for SS7 network management msg class */
#define LIT_ASPSM_TX     12   /* Trace for ASP state maintenance msg class */
#define LIT_ASPTM_TX     13   /* Trace for ASP traffic maintenance msg class */
#define LIT_RKM_TX       14   /* Trace for Routing Key Mgmt message class */

typedef struct resp
{
    U8 selector;                /* selector */
    U8 prior;                   /* priority */
    U8 route;                   /* route */
    U8 spare1;                  /* alignment */
    MemoryId mem;               /* memory */
}Resp;

typedef struct _sigtMgmtHeader       /* header */
{
    U16 msgLen;                 /* message length   - optional */
    U8 msgType;                 /* message type     - mandatory */
    U8 version;                 /* version          - optional */
    U16 seqNmb;                 /* sequence number  - optional */
    U16 spare1;                 /* alignment */
    EntityId entId;             /* entity id        - mandatory */
    ElmntId elmId;              /* element id       - mandatory */
    U32 transId;                /* transaction Id - mandatory */
    Resp response;              /* response parameters - mandatory */
}SigtMgmtHeader;

#ifndef LIT_MAX_TRC_LEN
#define LIT_MAX_TRC_LEN     200
#endif

/* common status */
typedef struct cmStatus
{
   U16 status;       /* status of request */
   U16 reason;       /* failure reason */
}CmStatus;

/* M3UA Trace structure */

typedef struct _M3UATrc               /* Trace */
{
   DateTime          dt;            /* date and time */
   S16               suId;          /* SCTSAP Id */
   U16               evnt;          /* event */
   U16               len;           /* trace length */
   U16               spare1;        /* alignment */
   U32               msgType;       /* message Type */
   U8                evntParm[LIT_MAX_TRC_LEN];  /* event parameters */
} M3UATrc;

/* M3UA management structure */
typedef struct _M3UAMgmtTrace
{
    DPRH        dprh;                           /* TXn000 DPR header */
    U16         start;                          /* offset of 1st byte */
    U16         end;                            /* offset of last byte + 1 */
    U8          event;                          /* Type of event request */
    U8          spare1;                         /* Alignment */
	U16			spare2;
    SigtMgmtHeader  hdr;                         /* Common mgmt header */
    CmStatus    cfm;
    U32         spare3;
    union {
		M3UATrc		trc;
     } req;
} M3UAMgmtTrace;

/* SCTP Trace Structure */
#ifndef LSB_MAX_TRC_LEN
#define LSB_MAX_TRC_LEN     512
#endif
#define LSB_MSG_RECVD       1        /* Message received */
#define LSB_MSG_TXED        2        /* Message transmitted */

/* SCTP Message Trace Structure */
typedef struct _sctpTrc          /* Trace */
{
    DateTime        dt;             /* date and time */
    S16             sapId;          /* TSAP Id */
    U16             evnt;           /* event */
    U16             len;            /* trace length */
    U16             spare1;         /* alignment */
    U8              evntParm[LSB_MAX_TRC_LEN];       /* event parameters */
} SCTPTrc;

/* SCTP  management structure */
typedef struct _sctpMgmtTrace
{
    DPRH        dprh;               /* TXn000 DPR header */
    U16         start;              /* offset of 1st byte */
    U16         end;                /* offset of last byte + 1 */
    U8          event;              /* Type of event request */
    U8          spare1;             /* Alignment */
    U16         spare2;             /* Alignment */
    SigtMgmtHeader  hdr;            /* header */
    CmStatus    cfm;
    U32         spare3;
    union
	{
		SCTPTrc 		trc; 		/* trace */
    }req;
} SCTPMgmtTrace;

